@@ -3,7 +3,7 @@
  *  ddcutil standalone application mainline
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -11,8 +11,6 @@
 
 #include <assert.h>
 #include <ctype.h>
-#include <dynvcp/dyn_dynamic_features.h>
-#include <dynvcp/dyn_parsed_capabilities.h>
 #include <errno.h>
 #include <setjmp.h>
 #include <stdbool.h>
@@ -25,6 +23,7 @@
 #include "util/error_info.h"
 #include "util/failsim.h"
 #include "util/report_util.h"
+#include "util/string_util.h"
 #include "util/sysfs_util.h"
 /** \endcond */
 
@@ -36,20 +35,23 @@
 #include "base/ddc_errno.h"
 #include "base/ddc_packets.h"
 #include "base/displays.h"
+#include "base/dynamic_sleep.h"
 #include "base/linux_errno.h"
 #include "base/parms.h"
 #include "base/sleep.h"
 #include "base/status_code_mgt.h"
+#include "base/thread_retry_data.h"
 #include "base/tuned_sleep.h"
+#include "base/thread_sleep_data.h"
 
 #include "vcp/parse_capabilities.h"
 #include "vcp/vcp_feature_codes.h"
 
 #include "dynvcp/dyn_dynamic_features.h"
+#include "dynvcp/dyn_parsed_capabilities.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "i2c/i2c_do_io.h"
-
+#include "i2c/i2c_strategy_dispatcher.h"
 #include "adl/adl_shim.h"
 
 #include "usb/usb_displays.h"
@@ -68,10 +70,13 @@
 #include "cmdline/cmd_parser.h"
 #include "cmdline/parsed_cmd.h"
 
+#include "app_ddcutil/app_capabilities.h"
 #include "app_ddcutil/app_dynamic_features.h"
 #include "app_ddcutil/app_dumpload.h"
+#include "app_ddcutil/app_probe.h"
 #include "app_ddcutil/app_getvcp.h"
 #include "app_ddcutil/app_setvcp.h"
+#include "app_ddcutil/app_vcpinfo.h"
 
 #include "app_sysenv/query_sysenv.h"
 #ifdef USE_USB
@@ -87,27 +92,105 @@
 #endif
 
 
-// Default race class for this file
+// Default trace class for this file
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_TOP;
 
 
 //
-// Initialization and Statistics
+// Report core settings and command line options
+//
+
+static
+void report_performance_options(int depth) {
+      int d1 = depth+1;
+      rpt_label(depth, "Performance and Retry Options:");
+      rpt_vstring(d1, "Deferred sleep enabled:                      %s", sbool( is_deferred_sleep_enabled() ) );
+      rpt_vstring(d1, "Sleep suppression (reduced sleeps) enabled:  %s", sbool( is_sleep_suppression_enabled() ) );
+      bool dsa_enabled =  tsd_get_dsa_enabled_default();
+      rpt_vstring(d1, "Dynamic sleep adjustment enabled:            %s", sbool(dsa_enabled) );
+      if ( dsa_enabled )
+        rpt_vstring(d1, "Sleep multiplier factor:                %5.2f", tsd_get_sleep_multiplier_factor() );
+      rpt_nl();
+}
+
+
+static
+void report_utility_options(Parsed_Cmd * parsed_cmd, int depth) {
+
+    bool special_option_explained = false;
+    if (parsed_cmd->flags & CMD_FLAG_F1) {
+       rpt_vstring(depth, "Utility option --f1 enabled: Timeout on i2c-dev read()");
+       special_option_explained = true;
+    }
+    if (parsed_cmd->flags & CMD_FLAG_F2) {
+       rpt_vstring(depth, "Utility option --f2 enabled: Dynamic sleep adjustment");
+       special_option_explained = true;
+    }
+    if (parsed_cmd->flags & CMD_FLAG_F3)  {
+       rpt_vstring(depth, "Utility option --f3 enabled: Disable post-read sleep suppression");
+       special_option_explained = true;
+    }
+    if (parsed_cmd->flags & CMD_FLAG_F4) {
+       rpt_vstring(depth, "Utility option --f4 enabled: Read strategy tests");
+       special_option_explained = true;
+    }
+    if (parsed_cmd->flags & CMD_FLAG_F5) {
+       rpt_vstring(depth, "Utility option --f5 enabled: Deferred sleep");
+       special_option_explained = true;
+    }
+    if (parsed_cmd->flags & CMD_FLAG_F6) {
+       rpt_vstring(depth, "Utility option --f6 enabled: Force I2C bus");
+       special_option_explained = true;
+    }
+    if (parsed_cmd->i1 >= 0) {    // default is -1
+       rpt_vstring(depth, "Utility option --i1 = %d:     Unused", parsed_cmd->i1);
+       special_option_explained = true;
+    }
+    if (special_option_explained) {
+       rpt_nl();
+       // f0puts("\n", fout);
+    }
+ }
+
+
+static
+void report_settings(Parsed_Cmd * parsed_cmd, int depth) {
+
+    show_reporting();  // uses fout()
+
+    rpt_vstring( depth, "%.*s%-*s%s",
+              0,"",
+              28, "Force I2C slave address:",
+              sbool(i2c_force_slave_addr_flag));
+    rpt_vstring( depth, "%.*s%-*s%s",
+              0,"",
+              28, "User defined features:",
+              (enable_dynamic_features) ? "enabled" : "disabled" );  // "Enable user defined features" is too long a title
+              // sbool(enable_dynamic_features));
+    rpt_nl();
+    // f0puts("\n", output_dest);
+
+    report_performance_options(depth);
+
+    report_utility_options(parsed_cmd, depth);
+}
+
+
+//
+// Initialize and Report Statistics
 //
 
 // static long start_time_nanos;
 
-
 static
 void reset_stats() {
    ddc_reset_stats_main();
 }
 
 
-
 static
 void report_stats(DDCA_Stats_Type stats) {
-   ddc_report_stats_main(stats, 0);
+   ddc_report_stats_main(stats, get_output_level() >= DDCA_OL_VERBOSE, 0);
 
    // Report the elapsed time in ddc_report_stats_main().
    // The start time used there is that at the time of stats initialization,
@@ -123,252 +206,6 @@ void report_stats(DDCA_Stats_Type stats) {
 }
 
 
-// TODO: refactor
-//       originally just displayed capabilities, now returns parsed capabilities as well
-//       these actions should be separated
-Parsed_Capabilities *
-perform_get_capabilities_by_display_handle(Display_Handle * dh) {
-   FILE * fout = stdout;
-   FILE * ferr = stderr;
-   bool debug = false;
-   Parsed_Capabilities * pcap = NULL;
-   char * capabilities_string;
-   Error_Info * ddc_excp = get_capabilities_string(dh, &capabilities_string);
-   Public_Status_Code psc =  ERRINFO_STATUS(ddc_excp);
-   assert( (ddc_excp && psc!=0) || (!ddc_excp && psc==0) );
-
-   if (ddc_excp) {
-      switch(psc) {
-      case DDCRC_REPORTED_UNSUPPORTED:       // should not happen
-      case DDCRC_DETERMINED_UNSUPPORTED:
-         f0printf(ferr, "Unsupported request\n");
-         break;
-      case DDCRC_RETRIES:
-         f0printf(ferr, "Unable to get capabilities for monitor on %s.  Maximum DDC retries exceeded.\n",
-                 dh_repr(dh));
-         break;
-      default:
-         f0printf(ferr, "(%s) !!! Unable to get capabilities for monitor on %s\n",
-                __func__, dh_repr(dh));
-         DBGMSG("Unexpected status code: %s", psc_desc(psc));
-      }
-      // errinfo_free(ddc_excp);
-      ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || report_freed_exceptions);
-   }
-   else {
-      assert(capabilities_string);
-      // pcap is always set, but may be damaged if there was a parsing error
-      pcap = parse_capabilities_string(capabilities_string);
-      DDCA_Output_Level output_level = get_output_level();
-      if (output_level <= DDCA_OL_TERSE) {
-         f0printf(fout,
-                  "%s capabilities string: %s\n",
-                  (dh->dref->io_path.io_mode == DDCA_IO_USB) ? "Synthesized unparsed" : "Unparsed",
-                  capabilities_string);
-      }
-      else {
-         if (dh->dref->io_path.io_mode == DDCA_IO_USB)
-            pcap->raw_value_synthesized = true;
-         // report_parsed_capabilities(pcap, dh->dref->io_path.io_mode);    // io_mode no longer needed
-         dyn_report_parsed_capabilities(
-               pcap,
-               dh,
-               NULL,
-               0);
-         // free_parsed_capabilities(pcap);
-      }
-   }
-   DBGMSF(debug, "Returning: %p", pcap);
-   return pcap;
-}
-
-
-void probe_display_by_dh(Display_Handle * dh)
-{
-   FILE * fout = stdout;
-   bool debug = false;
-   DBGMSF(debug, "Starting. dh=%s", dh_repr(dh));
-   Public_Status_Code psc = 0;
-   Error_Info * ddc_excp = NULL;
-
-   f0printf(fout, "\nMfg id: %s, model: %s, sn: %s\n",
-                  dh->dref->pedid->mfg_id, dh->dref->pedid->model_name, dh->dref->pedid->serial_ascii);
-   f0printf(fout, "\nCapabilities for display on %s\n", dref_short_name_t(dh->dref));
-
-   DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
-   // not needed, message causes confusing messages if get_vcp_version fails but get_capabilities succeeds
-   // if (vspec.major < 2) {
-   //    printf("VCP (aka MCCS) version for display is less than 2.0. Output may not be accurate.\n");
-   // }
-
-   // reports capabilities, and if successful returns Parsed_Capabilities
-   DDCA_Output_Level saved_ol = get_output_level();
-   set_output_level(DDCA_OL_VERBOSE);
-   Parsed_Capabilities * pcaps = perform_get_capabilities_by_display_handle(dh);
-   set_output_level(saved_ol);
-
-   // how to pass this information down into app_show_vcp_subset_values_by_display_handle()?
-   bool table_reads_possible = parsed_capabilities_may_support_table_commands(pcaps);
-   f0printf(fout, "\nMay support table reads:   %s\n", bool_repr(table_reads_possible));
-
-   // *** VCP Feature Scan ***
-   // printf("\n\nScanning all VCP feature codes for display %d\n", dispno);
-   f0printf(fout, "\nScanning all VCP feature codes for display %s\n", dh_repr(dh) );
-   Byte_Bit_Flags features_seen = bbf_create();
-   app_show_vcp_subset_values_by_display_handle(
-         dh, VCP_SUBSET_SCAN, FSF_SHOW_UNSUPPORTED, features_seen);
-
-   if (pcaps) {
-      f0printf(fout, "\n\nComparing declared capabilities to observed features...\n");
-      Byte_Bit_Flags features_declared =
-            parsed_capabilities_feature_ids(pcaps, /*readable_only=*/true);
-      char * s0 = bbf_to_string(features_declared, NULL, 0);
-      f0printf(fout, "\nReadable features declared in capabilities string: %s\n", s0);
-      free(s0);
-
-      Byte_Bit_Flags caps_not_seen = bbf_subtract(features_declared, features_seen);
-      Byte_Bit_Flags seen_not_caps = bbf_subtract(features_seen, features_declared);
-
-      f0printf(fout, "\nMCCS (VCP) version reported by capabilities: %s\n",
-               format_vspec(pcaps->parsed_mccs_version));
-      f0printf(fout, "MCCS (VCP) version reported by feature 0xDf: %s\n",
-               format_vspec(vspec));
-      if (!vcp_version_eq(pcaps->parsed_mccs_version, vspec))
-         f0printf(fout, "Versions do not match!!!\n");
-
-      if (bbf_count_set(caps_not_seen) > 0) {
-         f0printf(fout, "\nFeatures declared as readable capabilities but not found by scanning:\n");
-         for (int code = 0; code < 256; code++) {
-            if (bbf_is_set(caps_not_seen, code)) {
-               VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid_w_default(code);
-               Display_Feature_Metadata * dfm =
-                     dyn_get_feature_metadata_by_dh_dfm(
-                        code,
-                         dh,
-                         true);   //  with_default
-               char * feature_name = get_version_sensitive_feature_name(vfte, pcaps->parsed_mccs_version);
-               if (!streq(feature_name, dfm->feature_name)) {
-                  rpt_vstring(1, "VCP_Feature_Table_Entry feature name: %s", feature_name);
-                  rpt_vstring(1, "Display_Feature_Metadata feature name: %s",
-                                 dfm->feature_name);
-               }
-               // assert( streq(feature_name, ifm->external_metadata->feature_name));
-               f0printf(fout, "   Feature x%02x - %s\n", code, feature_name);
-               if (vfte->vcp_global_flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY) {
-                  free_synthetic_vcp_entry(vfte);
-               }
-               // need to free ifm?
-            }
-         }
-      }
-      else
-         f0printf(fout, "\nAll readable features declared in capabilities were found by scanning.\n");
-
-      if (bbf_count_set(seen_not_caps) > 0) {
-         f0printf(fout, "\nFeatures found by scanning but not declared as capabilities:\n");
-         for (int code = 0; code < 256; code++) {
-            if (bbf_is_set(seen_not_caps, code)) {
-               VCP_Feature_Table_Entry * vfte = vcp_find_feature_by_hexid_w_default(code);
-
-               Display_Feature_Metadata * dfm =
-                     dyn_get_feature_metadata_by_dh_dfm(
-                        code,
-                         dh,
-                         true);   //  with_default
-               char * feature_name = get_version_sensitive_feature_name(vfte, vspec);
-               f0printf(fout, "   Feature x%02x - %s\n", code, feature_name);
-               if (!streq(feature_name, dfm->feature_name)) {
-                  rpt_vstring(1, "VCP_Feature_Table_Entry feature name: %s", feature_name);
-                  rpt_vstring(1, "Internal_Feature_Metadata feature name: %s",
-                                 dfm->feature_name);
-               }
-               // assert( streq(feature_name, ifm->external_metadata->feature_name));
-               if (vfte->vcp_global_flags & DDCA_SYNTHETIC_VCP_FEATURE_TABLE_ENTRY) {
-                  free_synthetic_vcp_entry(vfte);
-               }
-               // free ifm
-            }
-         }
-      }
-      else
-         f0printf(fout, "\nAll features found by scanning were declared in capabilities.\n");
-
-      bbf_free(features_declared);
-      bbf_free(caps_not_seen);
-      bbf_free(seen_not_caps);
-      free_parsed_capabilities(pcaps);
-   }
-   else {
-      f0printf(fout, "\n\nUnable to read or parse capabilities.\n");
-      f0printf(fout, "Skipping comparison of declared capabilities to observed features\n");
-   }
-   bbf_free(features_seen);
-
-
-   puts("");
-   // get VCP 0B
-   DDCA_Any_Vcp_Value * valrec;
-   int color_temp_increment = 0;
-   int color_temp_units = 0;
-   ddc_excp = ddc_get_vcp_value(
-                 dh,
-                 0x0b,              // color temperature increment,
-                 DDCA_NON_TABLE_VCP_VALUE,
-                 &valrec);
-   psc = ERRINFO_STATUS(ddc_excp);
-   if (psc == 0) {
-      if (debug)
-         f0printf(fout, "Value returned for feature x0b: %s\n", summarize_single_vcp_value(valrec) );
-      color_temp_increment = valrec->val.c_nc.sl;
-      free_single_vcp_value(valrec);
-
-      ddc_excp = ddc_get_vcp_value(
-                    dh,
-                    0x0c,              // color temperature request
-                    DDCA_NON_TABLE_VCP_VALUE,
-                    &valrec);
-      psc = ERRINFO_STATUS(ddc_excp);
-      if (psc == 0) {
-         if (debug)
-            f0printf(fout, "Value returned for feature x0c: %s\n", summarize_single_vcp_value(valrec) );
-         color_temp_units = valrec->val.c_nc.sl;
-         int color_temp = 3000 + color_temp_units * color_temp_increment;
-         f0printf(fout, "Color temperature increment (x0b) = %d degrees Kelvin\n", color_temp_increment);
-         f0printf(fout, "Color temperature request   (x0c) = %d\n", color_temp_units);
-         f0printf(fout, "Requested color temperature = (3000 deg Kelvin) + %d * (%d degrees Kelvin)"
-               " = %d degrees Kelvin\n",
-               color_temp_units,
-               color_temp_increment,
-               color_temp);
-      }
-   }
-   if (psc != 0) {
-      f0printf(fout, "Unable to calculate color temperature from VCP features x0B and x0C\n");
-      // errinfo_free(ddc_excp);
-      ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || report_freed_exceptions);
-   }
-   // get VCP 14
-   // report color preset
-
-   DBGMSF(debug, "Done.");
-}
-
-
-void probe_display_by_dref(Display_Ref * dref) {
-   FILE * fout = stdout;
-   Display_Handle * dh = NULL;
-   Public_Status_Code psc = ddc_open_display(dref, CALLOPT_ERR_MSG, &dh);
-   if (psc != 0) {
-      f0printf(fout, "Unable to open display %s, status code %s",
-                     dref_short_name_t(dref), psc_desc(psc) );
-   }
-   else {
-      probe_display_by_dh(dh);
-      ddc_close_display(dh);
-   }
-}
-
-
 //
 // Mainline
 //
@@ -426,8 +263,10 @@ int main(int argc, char *argv[]) {
 
    // global variable in dyn_dynamic_features:
    enable_dynamic_features = parsed_cmd->flags & CMD_FLAG_ENABLE_UDF;
+   enable_sleep_suppression( parsed_cmd->flags & CMD_FLAG_REDUCE_SLEEPS );
+   enable_deferred_sleep( parsed_cmd->flags & CMD_FLAG_DEFER_SLEEPS);
 
-   init_ddc_services();  // n. initializes start timestamp
+   init_ddc_services();   // n. initializes start timestamp
    // overrides setting in init_ddc_services():
    i2c_set_io_strategy(DEFAULT_I2C_IO_STRATEGY);
 
@@ -440,6 +279,16 @@ int main(int argc, char *argv[]) {
    }
 #endif
 
+   // HACK FOR TESTING
+   if (parsed_cmd->flags & CMD_FLAG_F6) {
+      fprintf(stdout, "Setting i2c_force_bus\n");
+      if ( !(parsed_cmd->pdid) || parsed_cmd->pdid->id_type != DISP_ID_BUSNO) {
+         fprintf(stdout, "bus number required, use --busno\n");
+         return 1;
+      }
+      i2c_force_bus = true;
+   }
+
    Call_Options callopts = CALLOPT_NONE;
    i2c_force_slave_addr_flag = parsed_cmd->flags & CMD_FLAG_FORCE_SLAVE_ADDR;
    if (parsed_cmd->flags & CMD_FLAG_FORCE)
@@ -451,46 +300,39 @@ int main(int argc, char *argv[]) {
    // if (show_recoverable_errors)
    //    parsed_cmd->stats = true;
 
-   if (parsed_cmd->output_level >= DDCA_OL_VERBOSE) {
-      show_reporting();
-      f0printf( fout, "%.*s%-*s%s\n",
-                0,"",
-                28, "Force I2C slave address:",
-                bool_repr(i2c_force_slave_addr_flag));
-      f0printf( fout, "%.*s%-*s%s\n",
-                0,"",
-                28, "User defined features:",
-                (enable_dynamic_features) ? "enabled" : "disabled" );  // "Enable user defined features" is too long a title
-                // bool_repr(enable_dynamic_features));
-      f0puts("\n", fout);
-   }
 
    // n. MAX_MAX_TRIES checked during command line parsing
    if (parsed_cmd->max_tries[0] > 0) {
-#ifdef USE_API
-      ddca_set_max_tries(DDCA_WRITE_ONLY_TRIES, parsed_cmd->max_tries[0]);
-#else
-      ddc_set_max_write_only_exchange_tries(parsed_cmd->max_tries[0]);
-#endif
+      // ddc_set_max_write_only_exchange_tries(parsed_cmd->max_tries[0]);  // sets in Try_Data
+      try_data_set_maxtries2(WRITE_ONLY_TRIES_OP, parsed_cmd->max_tries[0]);
+
+      // redundant
+      trd_set_default_max_tries(0, parsed_cmd->max_tries[0]);
+      trd_set_initial_thread_max_tries(0, parsed_cmd->max_tries[0]);
    }
 
    if (parsed_cmd->max_tries[1] > 0) {
-#ifdef USE_API
-      ddca_set_max_tries(DDCA_WRITE_READ_TRIES, parsed_cmd->max_tries[1]);
-#else
-      ddc_set_max_write_read_exchange_tries(parsed_cmd->max_tries[1]);
-#endif
+      // ddc_set_max_write_read_exchange_tries(parsed_cmd->max_tries[1]);   // sets in Try_Data
+      try_data_set_maxtries2(WRITE_READ_TRIES_OP, parsed_cmd->max_tries[1]);
+
+      trd_set_default_max_tries(1, parsed_cmd->max_tries[1]);
+      trd_set_initial_thread_max_tries(1, parsed_cmd->max_tries[1]);
    }
 
    if (parsed_cmd->max_tries[2] > 0) {
-#ifdef USE_API
-      ddca_set_max_tries(DDCA_MULTI_PART_TRIES, parsed_cmd->max_tries[2]);
-#else
-      ddc_set_max_multi_part_read_tries(parsed_cmd->max_tries[2]);
-      ddc_set_max_multi_part_write_tries(parsed_cmd->max_tries[2]);
-#endif
+      // ddc_set_max_multi_part_read_tries(parsed_cmd->max_tries[2]);       // sets in Try_Data
+      // ddc_set_max_multi_part_write_tries(parsed_cmd->max_tries[2]);
+      try_data_set_maxtries2(MULTI_PART_READ_OP,  parsed_cmd->max_tries[2]);
+      try_data_set_maxtries2(MULTI_PART_WRITE_OP, parsed_cmd->max_tries[2]);
+
+      trd_set_default_max_tries(2, parsed_cmd->max_tries[2]);
+      trd_set_initial_thread_max_tries(2, parsed_cmd->max_tries[2]);
+      // impedance match
+      trd_set_default_max_tries(3, parsed_cmd->max_tries[2]);
+      trd_set_initial_thread_max_tries(3, parsed_cmd->max_tries[2]);
    }
 
+
 #ifdef USE_USB
    // if ( !(parsed_cmd->flags & CMD_FLAG_ENABLE_USB)) {
       DDCA_Status rc = ddc_enable_usb_display_detection( parsed_cmd->flags & CMD_FLAG_ENABLE_USB );
@@ -504,14 +346,28 @@ int main(int argc, char *argv[]) {
    ddc_set_async_threshold(threshold);
 
    if (parsed_cmd->sleep_multiplier != 0 && parsed_cmd->sleep_multiplier != 1) {
-      set_sleep_multiplier_factor(parsed_cmd->sleep_multiplier);
+      tsd_set_sleep_multiplier_factor(parsed_cmd->sleep_multiplier);         // for current thread
+      tsd_set_default_sleep_multiplier_factor(parsed_cmd->sleep_multiplier); // for new threads
+      if (parsed_cmd->sleep_multiplier > 1.0f && parsed_cmd->flags & CMD_FLAG_DSA )
+      {
+         tsd_dsa_enable_globally(true);
+      }
+   }
+
+   // experimental timeout of i2c read()
+   if (parsed_cmd->flags & CMD_FLAG_TIMEOUT_I2C_IO) {
+      set_i2c_fileio_use_timeout(true);
+   }
+
+   if (parsed_cmd->output_level >= DDCA_OL_VERBOSE) {
+      report_settings(parsed_cmd, 0);
    }
 
    main_rc = EXIT_SUCCESS;     // from now on assume success;
    DBGTRC(main_debug, TRACE_GROUP, "Initialization complete, process commands");
 
    if (parsed_cmd->cmd_id == CMDID_LISTVCP) {
-      vcp_list_feature_codes(stdout);
+      app_listvcp(stdout);
       main_rc = EXIT_SUCCESS;
    }
 
@@ -528,26 +384,10 @@ int main(int argc, char *argv[]) {
       if (parsed_cmd->flags & CMD_FLAG_WO_ONLY)
          flags |= FSF_WO_ONLY;
 
-      VCP_Feature_Set * fset = create_feature_set_from_feature_set_ref(
-                                parsed_cmd->fref,
-                                parsed_cmd->mccs_vspec,
-                                flags);
-      if (!fset) {
-         vcpinfo_ok = false;
-      }
-      else {
-         if (parsed_cmd->output_level <= DDCA_OL_TERSE)
-            report_feature_set(fset, 0);
-         else {
-            int ct =  get_feature_set_size(fset);
-            int ndx = 0;
-            for (;ndx < ct; ndx++) {
-               VCP_Feature_Table_Entry * pentry = get_feature_set_entry(fset, ndx);
-               report_vcp_feature_table_entry(pentry, 0);
-            }
-         }
-         free_vcp_feature_set(fset);
-      }
+
+
+      vcpinfo_ok = app_vcpinfo(parsed_cmd->fref, parsed_cmd->mccs_vspec, flags);
+
 
       main_rc = (vcpinfo_ok) ? EXIT_SUCCESS : EXIT_FAILURE;
    }
@@ -563,8 +403,63 @@ int main(int argc, char *argv[]) {
 
    else if (parsed_cmd->cmd_id == CMDID_DETECT) {
       DBGTRC(main_debug, TRACE_GROUP, "Detecting displays...");
-      ddc_ensure_displays_detected();
-      ddc_report_displays(/*include_invalid_displays=*/ true, 0);
+      if (!(parsed_cmd->flags & CMD_FLAG_F4)) {  // normal case
+         ddc_ensure_displays_detected();
+         ddc_report_displays(/*include_invalid_displays=*/ true, 0);
+      }
+      else {  // temporary
+         typedef struct {
+            I2C_IO_Strategy_Id  i2c_io_strategy_id;
+            bool                i2c_read_bytewise;
+            bool                edid_uses_i2c_layer;
+            bool                edid_read_bytewise;
+         } Choice_Entry;
+
+         Choice_Entry choices[16] =
+         { {I2C_IO_STRATEGY_FILEIO,  false,    false,    false},
+           {I2C_IO_STRATEGY_FILEIO,  false,    false,    true},
+           {I2C_IO_STRATEGY_FILEIO,  false,    true,     false},
+           {I2C_IO_STRATEGY_FILEIO,  false,    true,     true},
+           {I2C_IO_STRATEGY_FILEIO,  true,     false,    false},
+           {I2C_IO_STRATEGY_FILEIO,  true,     false,    true},
+           {I2C_IO_STRATEGY_FILEIO,  true,     true,     false},
+           {I2C_IO_STRATEGY_FILEIO,  true,     true,     true},
+
+           {I2C_IO_STRATEGY_IOCTL,  false,     false,    false},
+           {I2C_IO_STRATEGY_IOCTL,  false,     false,    true},
+           {I2C_IO_STRATEGY_IOCTL,  false,     true,     false},
+           {I2C_IO_STRATEGY_IOCTL,  false,     true,     true},
+           {I2C_IO_STRATEGY_IOCTL,  true,      false,    false},
+           {I2C_IO_STRATEGY_IOCTL,  true,      false,    true},
+           {I2C_IO_STRATEGY_IOCTL,  true,      true,     false},
+           {I2C_IO_STRATEGY_IOCTL,  true,      true,     true},
+         };
+
+         for (int ndx=0; ndx<16; ndx++) {
+            Choice_Entry cur = choices[ndx];
+
+            rpt_nl();
+            rpt_vstring(0, "===========> IO STRATEGY %d:", ndx+1);
+             char * s = (cur.i2c_io_strategy_id == I2C_IO_STRATEGY_FILEIO) ? "FILEIO" : "IOCTL";
+             int d = 1;
+             rpt_vstring(d, "i2c_io_strategy:          %s", s);
+             rpt_vstring(d, "i2c_read_bytewise:        %s", sbool(cur.i2c_read_bytewise));
+             rpt_vstring(d, "EDID read uses I2C layer: %s", sbool(cur.edid_uses_i2c_layer));
+             rpt_vstring(d, "EDID read bytewise:       %s", sbool(cur.edid_read_bytewise));
+
+             i2c_set_io_strategy(       cur.i2c_io_strategy_id);
+             I2C_Read_Bytewise        = cur.i2c_read_bytewise;
+             EDID_Read_Uses_I2C_Layer = cur.edid_uses_i2c_layer;
+             EDID_Read_Bytewise       = cur.edid_read_bytewise;
+
+             rpt_nl();
+             // discard existing detected monitors
+             ddc_discard_detected_displays();
+             ddc_ensure_displays_detected();
+             // will include any USB or ADL displays, but that's ok
+             ddc_report_displays(/*include_invalid_displays=*/ true, 0);
+         }
+      }
       DBGTRC(main_debug, TRACE_GROUP, "Display detection complete");
       main_rc = EXIT_SUCCESS;
    }
@@ -607,8 +502,10 @@ int main(int argc, char *argv[]) {
                loadvcp_ok = false;
          }
       }
-      if (loadvcp_ok)
+      if (loadvcp_ok) {
+         tsd_dsa_enable(parsed_cmd->flags & CMD_FLAG_DSA);
          loadvcp_ok = loadvcp_by_file(fn, dh);
+      }
 
       // if we opened the display, we close it
       if (dh)
@@ -668,8 +565,8 @@ int main(int argc, char *argv[]) {
       f0printf(fout, "Forcing --force-slave-address..\n");
       i2c_force_slave_addr_flag = true;
       f0printf(fout, "This command will take a while to run...\n\n");
-      ddc_set_max_write_read_exchange_tries(MAX_MAX_TRIES);
-      ddc_set_max_multi_part_read_tries(MAX_MAX_TRIES);
+      try_data_set_maxtries2(MULTI_PART_READ_OP, MAX_MAX_TRIES);
+      try_data_set_maxtries2(MULTI_PART_WRITE_OP, MAX_MAX_TRIES);
 
       ddc_ensure_displays_detected();    // *** ???
       DBGTRC(main_debug, TRACE_GROUP, "display detection complete");
@@ -698,6 +595,7 @@ int main(int argc, char *argv[]) {
       f0printf(fout, "Setting output level normal  Table features will be skipped...\n");
       set_output_level(DDCA_OL_NORMAL);
 
+      tsd_dsa_enable_globally(parsed_cmd->flags & CMD_FLAG_DSA);   // should this apply to INTERROGATE?
       GPtrArray * all_displays = ddc_get_all_displays();
       for (int ndx=0; ndx < all_displays->len; ndx++) {
          Display_Ref * dref = g_ptr_array_index(all_displays, ndx);
@@ -707,7 +605,7 @@ int main(int argc, char *argv[]) {
          }
          else {
             f0printf(fout, "\nProbing display %d\n", dref->dispno);
-            probe_display_by_dref(dref);
+            app_probe_display_by_dref(dref);
             f0printf(fout, "\nStatistics for probe of display %d:\n", dref->dispno);
             report_stats(DDCA_STATS_ALL);
          }
@@ -734,10 +632,11 @@ int main(int argc, char *argv[]) {
       // n. useful even if not much speed up, since avoids cluttering stats
       // with all the failures during detect
       Display_Ref * dref = NULL;
-      if (parsed_cmd->pdid->id_type == DISP_ID_BUSNO && (parsed_cmd->flags & CMD_FLAG_NODETECT)) {
+      // if (parsed_cmd->pdid->id_type == DISP_ID_BUSNO && (parsed_cmd->flags & CMD_FLAG_NODETECT)) {
+      if (parsed_cmd->pdid->id_type == DISP_ID_BUSNO) {
          int busno = parsed_cmd->pdid->busno;
          // is this really a monitor?
-         I2C_Bus_Info * businfo = detect_single_bus(busno);
+         I2C_Bus_Info * businfo = i2c_detect_single_bus(busno);
          if ( businfo && (businfo->flags & I2C_BUS_ADDR_0X50) ) {
             dref = create_bus_display_ref(busno);
             dref->dispno = -1;     // should use some other value for unassigned vs invalid
@@ -773,6 +672,8 @@ int main(int argc, char *argv[]) {
          ddc_open_display(dref, callopts, &dh);
 
          if (dh) {
+            // here or per command?  cur thread only or globally?
+            tsd_dsa_enable_globally(parsed_cmd->flags & CMD_FLAG_DSA);
             if (// parsed_cmd->cmd_id == CMDID_CAPABILITIES ||
                 parsed_cmd->cmd_id == CMDID_GETVCP       ||
                 parsed_cmd->cmd_id == CMDID_READCHANGES
@@ -791,7 +692,10 @@ int main(int argc, char *argv[]) {
                {
                   check_dynamic_features(dref);
 
-                  Parsed_Capabilities * pcaps = perform_get_capabilities_by_display_handle(dh);
+                  Parsed_Capabilities * pcaps = app_get_capabilities_by_display_handle(dh);
+                  if (pcaps) {
+                     app_show_parsed_capabilities(pcaps->raw_value,dh,  pcaps);
+                  }
                   main_rc = (pcaps) ? EXIT_SUCCESS : EXIT_FAILURE;
                   if (pcaps)
                      free_parsed_capabilities(pcaps);
@@ -816,8 +720,8 @@ int main(int argc, char *argv[]) {
                   if (parsed_cmd->flags & CMD_FLAG_RO_ONLY)
                      flags |= FSF_RO_ONLY;
 
-                  // this is nonsense, get do getvcp on a WO feature
-                  // should be caught by parser
+                  // this is nonsense, getvcp on a WO feature should be
+                  // caught by parser
                   if (parsed_cmd->flags & CMD_FLAG_WO_ONLY) {
                      flags |= FSF_WO_ONLY;
                      DBGMSG("Invalid: GETVCP for WO features");
@@ -907,7 +811,7 @@ int main(int argc, char *argv[]) {
             case CMDID_PROBE:
                check_dynamic_features(dref);
 
-               probe_display_by_dh(dh);
+               app_probe_display_by_dh(dh);
                main_rc = EXIT_SUCCESS;
                break;
 
