@@ -1,29 +1,11 @@
-/* query_sysenv_access.c
+/* @file query_sysenv_access.c
  *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  Checks on the the existence of and access to /dev/i2c devices
  */
 
- /** \f
-  *  Checks on the the existence of and access to /dev/i2c devices
-  */
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 
 /** \cond */
 #include <assert.h>
@@ -51,9 +33,8 @@
 #include "query_sysenv_access.h"
 
 
-
 /** Perform redundant checks as cross-verification */
-bool redundant_i2c_device_identification_checks = true;
+bool redundant_i2c_device_identification_checks = false;
 
 
 //
@@ -63,7 +44,12 @@ bool redundant_i2c_device_identification_checks = true;
 // Consolidate them here.  (IN PROGRESS)
 //
 
-Byte_Value_Array get_i2c_devices_by_existence_test() {
+/** Gets a list of all /dev/i2c devices by checking the file system
+ *  if devices named /dev/i2c-N exist.
+ *
+ *  @return Byte_Value_Array containing the valid bus numbers
+ */
+static Byte_Value_Array get_i2c_devices_by_existence_test() {
    Byte_Value_Array bva = bva_create();
    for (int busno=0; busno < I2C_BUS_MAX; busno++) {
       if (i2c_device_exists(busno)) {
@@ -75,7 +61,12 @@ Byte_Value_Array get_i2c_devices_by_existence_test() {
 }
 
 
-Byte_Value_Array get_i2c_devices_by_ls() {
+/** Gets a list of all /dev/i2c-n devices by screen-scraping the output
+ *  of "ls /dev/i2c*".
+ *
+ *  @return Byte_Value_Array containing the valid bus numbers
+ */
+static Byte_Value_Array get_i2c_devices_by_ls() {
    Byte_Value_Array bva = bva_create();
 
    int ival;
@@ -117,7 +108,6 @@ bye:
  *
  *  \return #ByteValueArray of bus numbers for detected I2C devices
  */
-// TODO: simplify, no longer need to test with multiple methods
 Byte_Value_Array identify_i2c_devices() {
 
    Byte_Value_Array i2c_device_numbers_result = NULL;   // result
@@ -128,7 +118,7 @@ Byte_Value_Array identify_i2c_devices() {
    Byte_Value_Array bva4 = NULL;
 
    bva1 = get_i2c_devices_by_existence_test();
-   if (redundant_i2c_device_identification_checks) {
+   if (redundant_i2c_device_identification_checks) {     // normally false, set true for testing
       bva2 = get_i2c_devices_by_ls();
       bva3 = get_i2c_device_numbers_using_udev(/* include_smbus= */ true);
       bva4 = get_i2c_device_numbers_using_udev_w_sysattr_name_filter(NULL);
@@ -167,7 +157,7 @@ static char * get_username(Env_Accumulator * accum) {
    char * cmd = "echo $LOGNAME";
    printf("(%s) executing command: %s\n", __func__, cmd);
    bool ok = execute_shell_cmd_rpt(cmd, 0);
-   printf("(%s) execute_shell_cmd() returned %s\n", __func__, bool_repr(ok));
+   printf("(%s) execute_shell_cmd() returned %s\n", __func__, sbool(ok));
 
 #endif
    uid_t uid = getuid();
@@ -185,16 +175,22 @@ static char * get_username(Env_Accumulator * accum) {
    return uname;
 }
 
-/**
+
+/** Checks which /dev/i2c devices are readable and writable.
  *
  *  \param  accum  accumulates environment information
  *
  *  \remark
  *  Sets the following fields in **Env_Accumulator**:
+ *  - cur_user_all_devi2c_rw
+ *  - cur_user_any_devi2c_rw
+ *  - dev.i2c_common_group_name
+ *  - any_dev_i2c_is_group_rw
+ *  - all_dev_i2c_is_group_rw
  */
 static void check_dev_i2c_access(Env_Accumulator * accum) {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
 
    // bool all_i2c_rw = false;
    int busct = bva_length(accum->dev_i2c_device_numbers);
@@ -260,7 +256,7 @@ static void check_dev_i2c_access(Env_Accumulator * accum) {
                      accum->all_dev_i2c_has_group_i2c = false;
 
                   DBGMSF(debug, "file=%s, st_gid=%d, gr_name=%s, cur_file_grp_rw=%s",
-                        fnbuf, fs.st_gid, gr_name, bool_repr(cur_file_grp_rw));
+                        fnbuf, fs.st_gid, gr_name, sbool(cur_file_grp_rw));
 
                   if (fs.st_gid != 0) {    // root group is special case
                      if (cur_file_grp_rw)
@@ -286,7 +282,7 @@ static void check_dev_i2c_access(Env_Accumulator * accum) {
                accum->cur_uname);
    }
 
-   DBGMSF0(debug, "Done");
+   DBGMSF(debug, "Done");
 }
 
 
@@ -303,7 +299,7 @@ static void check_dev_i2c_access(Env_Accumulator * accum) {
  */
 static void check_group_i2c(Env_Accumulator * accum, bool verbose) {
    bool debug = false;
-   DBGMSF(debug, "Starting. verbose=%s", bool_repr(verbose));
+   DBGMSF(debug, "Starting. verbose=%s", sbool(verbose));
    // verbose = true;
 
    if (verbose) {
@@ -336,6 +332,9 @@ static void check_group_i2c(Env_Accumulator * accum, bool verbose) {
          if (accum->cur_user_in_group_i2c) {
             rpt_vstring(1,"Current user %s is a member of group i2c", accum->cur_uname);
          }
+         else if (streq(accum->cur_uname, "root")) {
+            rpt_vstring(1, "Current user is root, membership in group i2c not needed");
+         }
          else {
             rpt_vstring(1, "WARNING: Current user %s is NOT a member of group i2c", accum->cur_uname);
          }
@@ -399,7 +398,7 @@ static void check_group_i2c(Env_Accumulator * accum, bool verbose) {
    }
 #endif
 
-   DBGMSF0(debug, "Done.");
+   DBGMSF(debug, "Done.");
 }
 
 
@@ -451,7 +450,7 @@ static void check_udev() {
  */
 void check_i2c_devices(Env_Accumulator * accum) {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
 
    // Env_Accumulator values already set
    assert(accum->dev_i2c_device_numbers);
@@ -517,7 +516,7 @@ void check_i2c_devices(Env_Accumulator * accum) {
       check_udev();
    }
 
-   DBGMSF0(debug, "Done");
+   DBGMSF(debug, "Done");
 }
 
 
