@@ -3,7 +3,7 @@
  * Base structures and functions for subsystem that diagnoses user configuration
  */
 
-// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef QUERY_SYSENV_BASE_H_
@@ -20,6 +20,7 @@
 char ** get_known_video_driver_module_names();
 char ** get_prefix_match_names();
 char ** get_other_driver_module_names();
+char ** get_all_driver_module_strings();
 
 void sysenv_rpt_file_first_line(char * fn, char * title, int depth);
 bool sysenv_show_one_file(char * dir_name, char * simple_fn, bool verbose, int depth);
@@ -39,6 +40,8 @@ char * driver_name_list_string(Driver_Name_Node * head);
 bool only_fglrx(Driver_Name_Node * driver_list);
 bool only_nvidia_or_fglrx(Driver_Name_Node * driver_list);
 
+int  i2c_path_to_busno(char * path);
+
 
 #define ENV_ACCUMULATOR_MARKER "ENVA"
 /** Collects system environment information */
@@ -108,4 +111,15 @@ int execute_cmd_collect_with_filter(
       bool         ignore_case,
       int          limit,
       GPtrArray ** result_loc);
+
+
+// if defined, some tests with long elapsed times are skipped to shorten time of test runs
+// #define SYSENV_QUICK_TEST_RUN
+
+// #define SYSENV_TEST_IDENTICAL_EDIDS
+#ifdef SYSENV_TEST_IDENTICAL_EDIDS
+// For testing situation where 2 displays have the same EDID, e.g. LG displays
+extern Byte * first_edid;
+#endif
+
 #endif /* QUERY_SYSENV_BASE_H_ */
