@@ -1,8 +1,9 @@
 /** \f query_sysenv_modules.c
+ *
  *  Module checks
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -89,10 +90,10 @@ bool is_module_builtin(char * module_name) {
       builtin2 = (lines->len == 1);
    }
    g_ptr_array_free(lines, true);
-   // DBGMSG("builtin2=%s", bool_repr(builtin2));
+   // DBGMSG("builtin2=%s", sbool(builtin2));
    result = builtin2;
 
-   DBGMSF(debug, "module_name = %s, returning %s", module_name, bool_repr(result));
+   DBGMSF(debug, "module_name = %s, returning %s", module_name, sbool(result));
    return result;
 }
 
@@ -140,7 +141,7 @@ bool is_module_loadable(char * module_name, int depth) {
         closedir(d);
      }
 
-   DBGMSF(debug, "Done. Returning: %s", bool_repr(result));
+   DBGMSF(debug, "Done. Returning: %s", sbool(result));
    return result;
 }
 
@@ -208,3 +209,22 @@ void check_i2c_dev_module(Env_Accumulator * accum, int depth) {
       }
    }
 }
+
+
+/** Reports video related contents of directory /etc/modprobe.d
+ *
+ *  @param depth  logical indentation depth
+ */
+void probe_modules_d(int depth) {
+   rpt_nl();
+   rpt_vstring(depth, "Video related contents of /etc/modprobe.d");
+   char ** strings = get_all_driver_module_strings();
+   char * grep_terms = strjoin((const char **) strings, ntsa_length(strings), "|");
+   int bufsz = strlen(grep_terms) + 100;
+   char * cmd = calloc(1, bufsz);
+   // g_snprintf(cmd, bufsz, "grep -El \"\(%s)\" /etc/modprobe.d/*conf | xargs tail -n +1", grep_terms );
+   g_snprintf(cmd, bufsz, "grep -EH \"\(%s)\" /etc/modprobe.d/*conf", grep_terms );
+   // DBGMSG("cmd: %s", cmd);
+   execute_shell_cmd_rpt(cmd, depth+1);
+   free(cmd);
+}
