@@ -2,7 +2,7 @@
  *  Probe the USB environment
  */
 
-// Copyright (C) 2016-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2016-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -53,7 +53,7 @@ static bool is_hid_monitor_rdesc(const char * fn) {
    if ( first_line && str_starts_with( first_line, "05 80"))
       result = true;
 
-   DBGMSF(debug, "fn=%s, returning: %s", fn, bool_repr(result));
+   DBGMSF(debug, "fn=%s, returning: %s", fn, sbool(result));
    return result;
 }
 
@@ -148,8 +148,23 @@ static void probe_hiddev(int depth) {
    int d1 = depth+1;
    int rc;
 
+   rpt_label(depth, "hiddev device names found in file system:");
+   GPtrArray * hiddev_devices = get_hiddev_device_names_using_filesys();
+   for (int ndx = 0; ndx < hiddev_devices->len; ndx++) {
+      rpt_vstring(d1, "%s", (char *) g_ptr_array_index(hiddev_devices,ndx));
+   }
+   g_ptr_array_free(hiddev_devices, true);
+
+   rpt_label(depth, "hiddev device names found in udev:");
+   hiddev_devices = get_hiddev_device_names_using_udev();
+   for (int ndx = 0; ndx < hiddev_devices->len; ndx++) {
+      rpt_vstring(d1, "%s", (char *) g_ptr_array_index(hiddev_devices,ndx));
+   }
+
+   rpt_vstring(depth, "Examining hiddev devices reported by udev...");
+
    // rpt_vstring(0, "Checking for USB HID devices using hiddev...");
-   GPtrArray * hiddev_devices = get_hiddev_device_names();
+   // GPtrArray * hiddev_devices = get_hiddev_device_names();
    rpt_vstring(depth, "Found %d USB HID devices.", hiddev_devices->len);
    for (int devndx=0; devndx<hiddev_devices->len; devndx++) {
       rpt_nl();
@@ -165,8 +180,8 @@ static void probe_hiddev(int depth) {
              rpt_vstring(depth, "Unable to open device %s: %s", curfn, strerror(errno));
              Usb_Detailed_Device_Summary * devsum = lookup_udev_usb_device_by_devname(curfn, true);
              if (devsum) {
-                rpt_vstring(d1, "Detiled device summary for testing: ");
-                report_usb_detailed_device_summary(devsum, d1+1);
+                // rpt_vstring(d1, "Detailed device summary for testing: ");
+                // report_usb_detailed_device_summary(devsum, d1+1);
 
                 rpt_vstring(d1, "USB bus %s, device %s, vid:pid: %s:%s - %s:%s",
                                 devsum->busnum_s,
@@ -199,15 +214,14 @@ static void probe_hiddev(int depth) {
                 bool b0 = is_hiddev_monitor(fd);
                 if (b0)
                    rpt_vstring(d1, "Identifies as a USB HID monitor");
-                else
+                else {
                    rpt_vstring(d1, "Not a USB HID monitor");
+                   b0 = force_hiddev_monitor(fd);
+                   if (b0)
+                      rpt_vstring(d1, "Device vid/pid matches exception list.  Forcing report for device.\n");
+                }
 
                 if (get_output_level() >= DDCA_OL_VERBOSE) {
-                   if (!b0) {
-                      b0 = force_hiddev_monitor(fd);
-                      if (b0)
-                         rpt_vstring(d1, "Device vid/pid matches exception list.  Forcing report for device.\n");
-                   }
                    if (b0) {
                       char * simple_devname = strstr(curfn, "hiddev");
                       Udev_Usb_Devinfo * dinfo = get_udev_usb_devinfo("usbmisc", simple_devname);
@@ -224,6 +238,27 @@ static void probe_hiddev(int depth) {
                       dbgrpt_hiddev_device_by_fd(fd, d1);
                    }
                 }
+                if (b0) {
+                   Buffer * edid_buffer = hiddev_get_edid(fd);
+                   if (edid_buffer) {
+                      Parsed_Edid * parsed_edid = create_parsed_edid(edid_buffer->bytes);  // copies bytes
+                      if (!parsed_edid) {
+                         rpt_label(d1, "get_hiddev_edid() returned invalid EDID");
+                         // if debug or verbose, dump the bad edid  ??
+                         // if (debug || get_output_level() >= OL_VERBOSE) {
+                         // }
+                      }
+                      else {
+                         report_parsed_edid(parsed_edid, /*verbose*/ true, d1);
+                         free_parsed_edid(parsed_edid);
+                      }
+                   }
+                   else {
+                      rpt_label(d1, "Unable to read EDID using hiddev");
+                   }
+                }
+
+
              }
              free(cgname);
          }
@@ -279,6 +314,14 @@ static void query_usb_monitors() {
    execute_shell_cmd_rpt("ls -l /dev/hidraw*", 2);
    rpt_nl();
 
+   rpt_vstring(1, "Searching /sys for hiddev* entries...");
+   execute_shell_cmd_rpt("find /sys -name \"hiddev*\"", 2);
+   rpt_nl();
+
+   rpt_vstring(1, "Searching /sys for hidraw* entries...");
+   execute_shell_cmd_rpt("find /sys -name \"hidraw*\"", 2);
+   rpt_nl();
+
    if (output_level >= DDCA_OL_VERBOSE) {
       char * subsys_name = "usbmisc";
       rpt_nl();
@@ -291,6 +334,7 @@ static void query_usb_monitors() {
       probe_udev_subsystem(subsys_name, /*show_usb_parent=*/ true, 1);
    }
 
+#ifdef DISABLE
    if (output_level >= DDCA_OL_VERBOSE) {
       // currently an overwhelming amount of information - need to display
       // only possible HID connected monitors
@@ -313,6 +357,7 @@ static void query_usb_monitors() {
        // vendor specific range on the Apple Cinema display
        // probe_hidapi(1);
    }
+#endif
 
    rpt_nl();
    rpt_vstring(0, "Checking for USB HID devices using hiddev...");
