@@ -13,7 +13,7 @@
  * - debug and trace messages
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #define GNU_SOURCE    // for syscall()
@@ -372,20 +372,21 @@ bool dbgtrc_show_thread_id = false;   ///< include thread id in debug/trace outp
 
 static
 Value_Name_Title_Table trace_group_table = {
-      VNT(DDCA_TRC_BASE, "BASE"),
-      VNT(DDCA_TRC_I2C, "I2C"),
+      VNT(DDCA_TRC_BASE,  "BASE"),
+      VNT(DDCA_TRC_I2C,   "I2C"),
 #ifdef HAVE_ADL
       VNT(TRC_ADL, "ADL"),
 #endif
-      VNT(DDCA_TRC_DDC, "DDC"),
-      VNT(DDCA_TRC_USB, "USB"),
-      VNT(DDCA_TRC_TOP, "TOP"),
-      VNT(DDCA_TRC_ENV, "ENV"),
-      VNT(DDCA_TRC_API, "API"),
-      VNT(DDCA_TRC_UDF, "UDF"),
-      VNT(DDCA_TRC_VCP, "VCP"),
+      VNT(DDCA_TRC_DDC,   "DDC"),
+      VNT(DDCA_TRC_USB,   "USB"),
+      VNT(DDCA_TRC_TOP,   "TOP"),
+      VNT(DDCA_TRC_ENV,   "ENV"),
+      VNT(DDCA_TRC_API,   "API"),
+      VNT(DDCA_TRC_UDF,   "UDF"),
+      VNT(DDCA_TRC_VCP,   "VCP"),
       VNT(DDCA_TRC_DDCIO, "DDCIO"),
       VNT(DDCA_TRC_SLEEP, "SLEEP"),
+      VNT(DDCA_TRC_RETRY, "RETRY"),
       VNT_END
 };
 const int trace_group_ct = ARRAY_SIZE(trace_group_table)-1;
@@ -477,7 +478,7 @@ void add_traced_file(const char * filename) {
       g_ptr_array_add(traced_file_table, bname);
    else
       free(bname);
-   // printf("(%s) filename=|%s|, bname=|%s|, found=%s\n", __func__, filename, bname, bool_repr(found));
+   // printf("(%s) filename=|%s|, bname=|%s|, found=%s\n", __func__, filename, bname, sbool(found));
 }
 
 
@@ -488,7 +489,7 @@ void add_traced_file(const char * filename) {
  */
 bool is_traced_function(const char * funcname) {
    bool result = (traced_function_table && gaux_string_ptr_array_find(traced_function_table, funcname) >= 0);
-   // printf("(%s) funcname=|%s|, returning: %s\n", __func__, funcname, bool_repr(result2));
+   // printf("(%s) funcname=|%s|, returning: %s\n", __func__, funcname, sbool(result2));
    return result;
 }
 
@@ -503,7 +504,7 @@ bool is_traced_file(const char * filename) {
    if (filename) {
       char * bname = g_path_get_basename(filename);
       result = (traced_file_table && gaux_string_ptr_array_find(traced_file_table, bname) >= 0);
-      // printf("(%s) filename=|%s|, bname=|%s|, returning: %s\n", __func__, filename, bname, bool_repr(result));
+      // printf("(%s) filename=|%s|, bname=|%s|, returning: %s\n", __func__, filename, bname, sbool(result));
       free(bname);
    }
    return result;
@@ -727,7 +728,7 @@ void show_ddcmsg() {
    print_simple_title_value(SHOW_REPORTING_TITLE_START,
                               "Reporting DDC data errors: ",
                               SHOW_REPORTING_MIN_TITLE_SIZE,
-                              bool_repr(report_ddc_errors));
+                              sbool(report_ddc_errors));
 }
 
 
