@@ -760,6 +760,32 @@ bool dref_eq(Display_Ref* this, Display_Ref* that) {
 }
 
 
+void dbgrpt_dref_flags(Dref_Flags flags, int depth) {
+
+#define RPT_DREF_FLAG(FLAG_NAME) \
+   rpt_vstring(d1, "%s: %s", #FLAG_NAME, sbool(flags&FLAG_NAME) )
+
+   int d0 = depth;
+   int d1 = d0+1;
+   // DBGMSG("d1 = %d", d1);
+   rpt_vstring(d0, "flags:        0x%02x", flags);
+   RPT_DREF_FLAG(DREF_DDC_COMMUNICATION_CHECKED                 );
+   RPT_DREF_FLAG(DREF_DDC_COMMUNICATION_WORKING                 );
+   RPT_DREF_FLAG(DREF_DDC_NULL_RESPONSE_CHECKED                 );
+   RPT_DREF_FLAG(DREF_DDC_IS_MONITOR_CHECKED                    );
+   RPT_DREF_FLAG(DREF_DDC_IS_MONITOR                            );
+   RPT_DREF_FLAG(DREF_TRANSIENT                                 );
+   RPT_DREF_FLAG(DREF_DYNAMIC_FEATURES_CHECKED                  );
+   RPT_DREF_FLAG(DREF_OPEN                                      );
+   RPT_DREF_FLAG(DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED    );
+   RPT_DREF_FLAG(DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED );
+   RPT_DREF_FLAG(DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED         );
+   RPT_DREF_FLAG(DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED         );
+
+#undef RPT_DREF_FLAG
+}
+
+
 /** Reports the contents of a #Display_Ref in a format appropriate for debugging.
  *
  *  \param  dref  pointer to #Display_Ref instance
@@ -806,6 +832,7 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
 
    // rpt_vstring(d1, "vcp_version:  %d.%d\n", dref->vcp_version.major, dref->vcp_version.minor );
    rpt_vstring(d1, "vcp_version:  %s", format_vspec(dref->vcp_version) );
+#ifdef OLD
    rpt_vstring(d1, "flags:        0x%02x", dref->flags);
    rpt_vstring(d2, "DDC communication checked:                  %s", sbool(dref->flags & DREF_DDC_COMMUNICATION_CHECKED) );
    if (dref->flags & DREF_DDC_COMMUNICATION_CHECKED)
@@ -816,6 +843,8 @@ void dbgrpt_display_ref(Display_Ref * dref, int depth) {
    rpt_vstring(d2, "DDC normal all byte 0 response may indicate unsupported: %s", sbool(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
    rpt_vstring(d2, "DDC does not indicate unsupported:          %s", sbool(dref->flags & DREF_DDC_DOES_NOT_INDICATE_UNSUPPORTED));
    rpt_vstring(d2, "Display Ref is open:                        %s", sbool(dref->flags & DREF_OPEN));
+#endif
+   dbgrpt_dref_flags(dref->flags, d1);
    rpt_vstring(d2, "mmid:                                       %s", (dref->mmid) ? mmk_repr(*dref->mmid) : "NULL");}
 
 
@@ -900,25 +929,24 @@ char * dref_repr_t(Display_Ref * dref) {
 
 /** Creates a #Display_Handle for an I2C #Display_Ref.
  *
- *  \param  fh   file handle of open display
+ *  \param  fd   Linux file descriptor of open display
  *  \param  dref pointer to #Display_Ref
  *  \return newly allocated #Display_Handle
  *
  *  \remark
  *  This functions handles the boilerplate of creating a #Display_Handle.
  */
-Display_Handle * create_bus_display_handle_from_display_ref(int fh, Display_Ref * dref) {
+Display_Handle * create_bus_display_handle_from_display_ref(int fd, Display_Ref * dref) {
    // assert(dref->io_mode == DDCA_IO_DEVI2C);
    assert(dref->io_path.io_mode == DDCA_IO_I2C);
    Display_Handle * dh = calloc(1, sizeof(Display_Handle));
    memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
-   dh->fh = fh;
+   dh->fd = fd;
    dh->dref = dref;
    // dref->vcp_version = DDCA_VSPEC_UNQUERIED;
    dh->repr = g_strdup_printf(
-                "[i2c: fh=%d, busno=%d]",
-                // "Display_Handle[i2c: fh=%d, busno=%d]",
-                dh->fh, dh->dref->io_path.path.i2c_busno);
+                "[i2c: fd=%d, busno=%d]",
+                dh->fd, dh->dref->io_path.path.i2c_busno);
    return dh;
 }
 
@@ -949,19 +977,19 @@ Display_Handle * create_adl_display_handle_from_display_ref(Display_Ref * dref)
 #ifdef USE_USB
 /** Creates a #Display_Handle for a USB #Display_Ref.
  *
- *  \param  fh   file handle of open display
- *  \param  dref pointer to #Display_Ref
+ *  \param  fh    Linux file descriptor of open display
+ *  \param  dref  pointer to #Display_Ref
  *  \return newly allocated #Display_Handle
  *
  *  \remark
  *  This functions handles to boilerplate of creating a #Display_Handle.
  */
-Display_Handle * create_usb_display_handle_from_display_ref(int fh, Display_Ref * dref) {
+Display_Handle * create_usb_display_handle_from_display_ref(int fd, Display_Ref * dref) {
    // assert(dref->io_mode == DDCA_IO_USB);
    assert(dref->io_path.io_mode == DDCA_IO_USB);
    Display_Handle * dh = calloc(1, sizeof(Display_Handle));
    memcpy(dh->marker, DISPLAY_HANDLE_MARKER, 4);
-   dh->fh = fh;
+   dh->fd = fd;
    dh->dref = dref;
    dh->repr = g_strdup_printf(
                 "[usb: %d:%d, %s/hiddev%d]",
@@ -996,7 +1024,7 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
          switch (dh->dref->io_path.io_mode) {
          case (DDCA_IO_I2C):
             // rpt_vstring(d1, "ddc_io_mode = DDC_IO_DEVI2C");
-            rpt_vstring(d1, "fh:                  %d", dh->fh);
+            rpt_vstring(d1, "fd:                  %d", dh->fd);
             rpt_vstring(d1, "busno:               %d", dh->dref->io_path.path.i2c_busno);
             break;
          case (DDCA_IO_ADL):
@@ -1006,7 +1034,7 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
             break;
          case (DDCA_IO_USB):
             // rpt_vstring(d1, "ddc_io_mode = USB_IO");
-            rpt_vstring(d1, "fh:                  %d", dh->fh);
+            rpt_vstring(d1, "fd:                  %d", dh->fd);
             rpt_vstring(d1, "usb_bus:             %d", dh->dref->usb_bus);
             rpt_vstring(d1, "usb_device:          %d", dh->dref->usb_device);
             rpt_vstring(d1, "hiddev_device_name:  %s", dh->dref->usb_hiddev_name);
@@ -1025,8 +1053,8 @@ void dbgrpt_display_handle(Display_Handle * dh, const char * msg, int depth) {
  *
  *  This variant of #dh_repr() is thread safe.
  *
- * \param  dh    display handle
- * \return  string representation of handle
+ * \param   dh      display handle
+ * \return  string  representation of handle
  */
 char * dh_repr_t(Display_Handle * dh) {
    static GPrivate  dh_buf_key = G_PRIVATE_INIT(g_free);
@@ -1039,8 +1067,8 @@ char * dh_repr_t(Display_Handle * dh) {
       switch (dh->dref->io_path.io_mode) {
       case DDCA_IO_I2C:
            snprintf(buf, bufsz,
-                    "Display_Handle[i2c: fh=%d, busno=%d]",
-                    dh->fh, dh->dref->io_path.path.i2c_busno);
+                    "Display_Handle[i2c: fd=%d, busno=%d]",
+                    dh->fd, dh->dref->io_path.path.i2c_busno);
            break;
        case DDCA_IO_ADL:
            snprintf(buf, bufsz,
