@@ -1,32 +1,19 @@
-/* dynamic_features.c
+/** @file dynamic_features.c
  *
- * <copyright>
- * Copyright (C) 2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  Dynamic Feature Record definition, creation, destruction, and conversion
  */
 
+// Copyright (C) 2018-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 #define _GNU_SOURCE      // for asprintf()
 
+/** \cond */
 #include <assert.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+/** \endcond */
 
 #include "ddcutil_status_codes.h"
 #include "ddcutil_types.h"
@@ -34,6 +21,8 @@
 #include "util/report_util.h"
 #include "util/string_util.h"
 
+#include "base/feature_metadata.h"
+
 #include "core.h"
 #include "monitor_model_key.h"
 #include "vcp_version.h"
@@ -77,70 +66,6 @@ Tokenized first_word(char * s) {
 }
 
 
-// copied from sample code, more to more generic location
-/* Creates a string representation of DDCA_Feature_Flags bitfield.
- *
- * Arguments:
- *    flags      feature characteristics
- *
- * Returns:      string representation, caller must free
- */
-char * interpret_feature_flags(DDCA_Version_Feature_Flags flags) {
-   char * buffer = NULL;
-   int rc = asprintf(&buffer, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
-       flags & DDCA_RO             ? "Read-Only, "                   : "",
-       flags & DDCA_WO             ? "Write-Only, "                  : "",
-       flags & DDCA_RW             ? "Read-Write, "                  : "",
-       flags & DDCA_STD_CONT       ? "Continuous (standard), "       : "",
-       flags & DDCA_COMPLEX_CONT   ? "Continuous (complex), "        : "",
-       flags & DDCA_SIMPLE_NC      ? "Non-Continuous (simple), "     : "",
-       flags & DDCA_COMPLEX_NC     ? "Non-Continuous (complex), "    : "",
-       flags & DDCA_NC_CONT        ? "Non-Continuous with continuous subrange, " :"",
-       flags & DDCA_WO_NC          ? "Non-Continuous (write-only), " : "",
-       flags & DDCA_NORMAL_TABLE   ? "Table (readable), "            : "",
-       flags & DDCA_WO_TABLE       ? "Table (write-only), "          : "",
-       flags & DDCA_DEPRECATED     ? "Deprecated, "                  : "",
-       flags & DDCA_USER_DEFINED   ? "User-defined, "                : "",
-       flags & DDCA_SYNTHETIC      ? "Synthesized, "                 : ""
-       );
-   assert(rc >= 0);   // real life code would check for malloc() failure in asprintf()
-   // remove final comma and blank
-   if (strlen(buffer) > 0)
-      buffer[strlen(buffer)-2] = '\0';
-
-   return buffer;
-}
-
-
-// eventually move to a more shared location
-void
-dbgrpt_feature_metadata(
-      DDCA_Feature_Metadata * md,
-      int                     depth)
-{
-   int d0 = depth;
-   int d1 = depth+1;
-   int d2 = depth+2;
-   rpt_structure_loc("DDCA_Feature_Metadata", md, depth);
-   rpt_vstring(d0, "Feature code:      0x%02x",  md->feature_code);
-// rpt_vstring(d1, "MCCS version:      %d.%d",  md->vspec.major, md->vspec.minor);
-   rpt_vstring(d1, "Feature name:      %s",     md->feature_name);
-   rpt_vstring(d1, "Description:       %s",     md->feature_desc);
-   char * s = interpret_feature_flags(md->feature_flags);
-   rpt_vstring(d1, "Feature flags:     0x%04x", md->feature_flags);
-   rpt_vstring(d1, "Interpreted flags: %s", s);
-   free(s);
-   if (md->sl_values) {
-      rpt_label(d1, "SL values:");
-      DDCA_Feature_Value_Entry * curval = md->sl_values;
-      while(curval->value_code || curval->value_name) {
-         rpt_vstring(d2, "0x%02x  - %s", curval->value_code, curval->value_name);
-         curval++;
-      }
-   }
-}
-
-
 // TODO: modify parse_vcp_value() in app_setvcp.c to use this function.
 //       Move this function to a more general location.
 
@@ -184,43 +109,108 @@ char * canonicalize_possible_hex_value(char * string_value) {
 // End of generic functions
 
 
+// Dynamic_Features_Rec
+
+void dbgrpt_dynamic_features_rec(
+      Dynamic_Features_Rec*   dfr,
+      int                     depth)
+{
+   assert(dfr && memcmp(dfr->marker, DYNAMIC_FEATURES_REC_MARKER, 4) == 0);
+   int d1 = depth + 1;
+   rpt_structure_loc("Dynamic_Features_Rec", dfr, depth);
+   rpt_vstring(d1, "marker:         %4s", dfr->marker);
+   rpt_vstring(d1, "mfg_id:         %s", dfr->mfg_id);
+   rpt_vstring(d1, "model_name:     %s", dfr->model_name);
+   rpt_vstring(d1, "product_code:   %u", dfr->product_code);
+   rpt_vstring(d1, "filename:       %s", dfr->filename);
+   rpt_vstring(d1, "MCCS vspec:     %d.%d", dfr->vspec.major, dfr->vspec.minor);
+   rpt_vstring(d1, "flags:          0x%02x %s", dfr->flags, interpret_feature_flags_t(dfr->flags));
+   if (dfr->features) {
+      rpt_vstring(d1, "features count: %d", g_hash_table_size(dfr->features));
+      for (int ndx = 1; ndx < 256; ndx++) {
+         DDCA_Feature_Metadata * cur_feature = g_hash_table_lookup(dfr->features,
+               GINT_TO_POINTER(ndx));
+         if (cur_feature)
+            dbgrpt_ddca_feature_metadata(cur_feature, d1);
+      }
+   }
+}
+
+
+
+
+/** Thread safe function that returns a string representation of a #Dynamic_Features_Rec
+ *  suitable for diagnostic messages. The returned value is valid until the
+ *  next call to this function on the current thread.
+ *
+ *  \param  dfr  pointer to #Dynamic_Features_Rec
+ *  \return string representation
+ */
+char * dfr_repr_t(Dynamic_Features_Rec * dfr) {
+   static GPrivate  dfr_repr_key = G_PRIVATE_INIT(g_free);
+   char * buf = get_thread_fixed_buffer(&dfr_repr_key, 100);
+
+   if (dfr)
+      g_snprintf(buf, 100, "Dynamic_Features_Rec[%s,%s,%d]",
+                           dfr->mfg_id, dfr->model_name, dfr->product_code);
+   else
+      g_snprintf(buf, 100, "NULL");
+   return buf;
+}
+
+
+
 
 DDCA_Feature_Metadata *
 get_dynamic_feature_metadata(
       Dynamic_Features_Rec * dfr,
       uint8_t                feature_code)
 {
+   bool debug = false;
+   DBGMSF(debug, "dfr=%s, feature_code=0x%02x", dfr_repr_t(dfr), feature_code);
+
    DDCA_Feature_Metadata * result = NULL;
-   if (dfr->features)
+   if (dfr && dfr->features)
       result = g_hash_table_lookup(dfr->features, GINT_TO_POINTER(feature_code));
+
+   DBGMSF(debug, "Returning %p", result);
    return result;
 }
 
 
 void
 free_feature_metadata(
-      gpointer data)
+      gpointer data)    // i.e. DDCA_Feature_Metadata *
 {
+   bool debug = false;
+   DBGMSF(debug, "Starting. DDCA_Feature_Metadata * data = %p", data);
+
    DDCA_Feature_Metadata * info = (DDCA_Feature_Metadata*) data;
    assert(memcmp(info->marker, DDCA_FEATURE_METADATA_MARKER, 4) == 0);
    // compare vs ddca_free_metadata_contents()
 
+   if (debug)
+      dbgrpt_ddca_feature_metadata(info, 2);
+
    free(info->feature_desc);
    free(info->feature_name);
    if (info->sl_values) {
+      free_sl_value_table(info->sl_values);
+#ifdef OLD
       DDCA_Feature_Value_Entry * cur = info->sl_values;
       for ( ;cur->value_code != 0x00 || cur->value_name; cur++ ) {
          free(cur->value_name);
       }
       free(info->sl_values);      // is this right? freeing a hunk allocated by GPtrArray
+#endif
    }
    info->marker[3] = 'x';
    free(info);
+
+   DBGMSF(debug, "Done");
 }
 
 
-
-
 Dynamic_Features_Rec *
 dfr_new(
       const char * mfg_id,
@@ -228,8 +218,12 @@ dfr_new(
       uint16_t     product_code,
       const char * filename)
 {
+   assert(mfg_id);
+   assert(model_name);
+
    Dynamic_Features_Rec * frec = calloc(1, sizeof(Dynamic_Features_Rec));
    memcpy(frec->marker, DYNAMIC_FEATURES_REC_MARKER, 4);
+
    frec->mfg_id       = strdup(mfg_id);
    frec->model_name   = strdup(model_name);
    frec->product_code = product_code;
@@ -243,43 +237,28 @@ void
 dfr_free(
       Dynamic_Features_Rec * frec)
 {
+   bool debug = false;
+   DBGMSF(debug, "Starting. frec=%p", frec);
+
    if (frec) {
       assert(memcmp(frec->marker, DYNAMIC_FEATURES_REC_MARKER, 4) == 0);
+
+      if (debug)
+         dbgrpt_dynamic_features_rec(frec, 2);
+
       free(frec->mfg_id);
       free(frec->model_name);
       free(frec->filename);
-      if (frec->features)
+      if (frec->features) {
+         DBGMSF(debug, "Calling g_hash_table_destroy() for %p", frec->features);
          g_hash_table_destroy(frec->features); // n. destroy function for values set at creation
+      }
       free(frec);
    }
+
+   DBGMSF(debug, "Done");
 }
 
-
-void
-dbgrpt_dfr(
-      Dynamic_Features_Rec * dfr,
-      int                    depth)
-{
-   int d1 = depth+1;
-   rpt_structure_loc("Dynamic_Features_Rec", dfr, depth);
-   rpt_vstring(d1, "marker:         %4s",   dfr->marker);
-   rpt_vstring(d1, "mfg_id:         %s",    dfr->mfg_id);
-   rpt_vstring(d1, "model_name:     %s",    dfr->model_name);
-   rpt_vstring(d1, "product_code:   %u",    dfr->product_code);
-   rpt_vstring(d1, "filename:       %s",    dfr->filename);
-   rpt_vstring(d1, "MCCS vspec:     %d.%d", dfr->vspec.major, dfr->vspec.minor);
-   if (dfr->features) {
-      rpt_vstring(d1, "features count: %d", g_hash_table_size(dfr->features));
-      for (int ndx = 1; ndx < 256; ndx++) {
-         DDCA_Feature_Metadata * cur_feature =
-               g_hash_table_lookup(dfr->features, GINT_TO_POINTER(ndx));
-         if (cur_feature)
-            dbgrpt_feature_metadata(cur_feature, d1);
-      }
-   }
-}
-
-
 //
 // Functions private to create_monitor_dynamic_featuers()
 //
@@ -301,8 +280,10 @@ add_error(
       va_start(args, fmt);
       vsnprintf(detail, 200, fmt, args);
       if (filename) {
-         if (linectr > 0)
-            snprintf(xdetail, sizeof(xdetail), "%s at line %d of file %s", detail, linectr, filename);
+         if (linectr > 0) {
+            snprintf(xdetail, sizeof(xdetail), "%s at line %d", detail, linectr);
+            // snprintf(xdetail, sizeof(xdetail), "%s at line %d of file %s", detail, linectr, filename);
+         }
          else
             snprintf(xdetail, sizeof(xdetail), "%s in file %s", detail, filename);
          final_detail = xdetail;
@@ -375,7 +356,7 @@ finalize_feature(
       const char *            filename,
       GPtrArray *             errors)
 {
-   DDCA_Feature_Flags * pflags = &cur_feature_metadata->feature_flags;
+   // DDCA_Feature_Flags * pflags = &cur_feature_metadata->feature_flags;
 
    if (cur_feature_values) {
       // add terminating entry
@@ -389,23 +370,23 @@ finalize_feature(
       // cur_feature_values = NULL;
    }
 
-   if ( *pflags & (DDCA_RW | DDCA_RO | DDCA_WO) )
-      *pflags |= DDCA_RW;
+   if ( cur_feature_metadata->feature_flags & (DDCA_RW | DDCA_RO | DDCA_WO) )
+      cur_feature_metadata->feature_flags |= DDCA_RW;
 
    if (cur_feature_metadata->sl_values) {
       if (cur_feature_metadata->feature_flags & DDCA_COMPLEX_NC) {
-         if ( *pflags & DDCA_WO)
-            switch_bits(pflags, DDCA_COMPLEX_NC, DDCA_WO_NC);
+         if ( cur_feature_metadata->feature_flags & DDCA_WO)
+            switch_bits(&cur_feature_metadata->feature_flags, DDCA_COMPLEX_NC, DDCA_WO_NC);
          else
-            switch_bits(pflags, DDCA_COMPLEX_NC, DDCA_SIMPLE_NC);
+            switch_bits(&cur_feature_metadata->feature_flags, DDCA_COMPLEX_NC, DDCA_SIMPLE_NC);
       }
 
-      else if ( *pflags & (DDCA_COMPLEX_CONT | DDCA_STD_CONT | DDCA_TABLE))
+      else if ( cur_feature_metadata->feature_flags & (DDCA_COMPLEX_CONT | DDCA_STD_CONT | DDCA_TABLE))
           ADD_ERROR(-1,  "Feature values specified for Continuous or Table feature");
    }
 
-   if (*pflags & DDCA_NORMAL_TABLE & (*pflags & DDCA_WO))
-      switch_bits(pflags, DDCA_NORMAL_TABLE, DDCA_WO_TABLE);
+   if (cur_feature_metadata->feature_flags & DDCA_NORMAL_TABLE & DDCA_WO)
+      switch_bits(&cur_feature_metadata->feature_flags, DDCA_NORMAL_TABLE, DDCA_WO_TABLE);
 
    // For now, to revisit
   //  cur_feature_metadata->vspec = frec->vspec;
@@ -417,6 +398,18 @@ finalize_feature(
 }
 
 
+/** Parse a set of lines describing a dynamic feature record, returning a
+ *  newly created #Dynamic_Feature_Rec if successful.
+ *
+ *  @param  mfg_id        3 character manufacturer identifier
+ *  @param  model_name    model name
+ *  @param  product_code  product code
+ *  @param  lines         array of input lines
+ *  @param  filename      source file name, for diagnostic messages, may be NULL
+ *  @param  dynamic_features_loc where to return newly allocated #Dynamic_Features_Rec,
+ *                        set to null if an #Error_Info struct is returned
+ *  @return error info struct, NULL if no error
+ */
 Error_Info *
 create_monitor_dynamic_features(
       const char * mfg_id,
@@ -477,9 +470,10 @@ create_monitor_dynamic_features(
             else if (streq(t1.word, "MODEL")) {
                model_name_seen = true;
                if ( !streq(t1.rest, model_name) ) {
+                  // DBGMSG("Expected model_name: \"%s\"", model_name);
                   ADD_ERROR(linectr, "Unexpected model name \"%s\"", t1.rest);
                }
-                  }
+            }
             else if (streq(t1.word, "MCCS_VERSION")) {
                // mccs_version_seen = true;   // not required for now
                // default as set by calloc() is 0.0, which is DDCA_VSPEC_UNKNOWN
@@ -525,7 +519,7 @@ create_monitor_dynamic_features(
 
                cur_feature_metadata = calloc(1, sizeof(DDCA_Feature_Metadata));
                memcpy(cur_feature_metadata->marker, DDCA_FEATURE_METADATA_MARKER, 4);
-               cur_feature_metadata->feature_flags = DDCA_USER_DEFINED;
+               cur_feature_metadata->feature_flags = DDCA_USER_DEFINED | DDCA_PERSISTENT_METADATA;
 
                char * feature_code = t2.word;
                char * feature_name = t2.rest;
@@ -547,7 +541,7 @@ create_monitor_dynamic_features(
                   }
                   else {     // valid opcode
                      cur_feature_metadata->feature_code = feature_id;
-                     cur_feature_metadata->feature_name = feature_name;
+                     cur_feature_metadata->feature_name = strdup(feature_name);
                      cur_feature_metadata->feature_desc = NULL;   // ignore for now
                   }
                }
@@ -639,11 +633,11 @@ create_monitor_dynamic_features(
       g_ptr_array_free(errors, false);
       *dynamic_features_loc = frec;
       if (debug)
-         dbgrpt_dfr(frec, 0);
+         dbgrpt_dynamic_features_rec(frec, 0);
    }
 
-   DBGMSF(debug, "Done. *dynamic_features_loc=%p, returning %p",
-                  *dynamic_features_loc, master_err);
+   DBGMSF(debug, "Done. *dynamic_features_loc=%p, returning %s",
+                  *dynamic_features_loc, errinfo_summary(master_err));
    assert( (master_err && !*dynamic_features_loc) || (!master_err && *dynamic_features_loc));
    return master_err;
 }
