@@ -1,12 +1,12 @@
-// dynamic_features.h
-
-// Copyright (C) 2018 Sanford Rockowitz <rockowitz@minsoft.com>
-// SPDX-License-Identifier: GPL-2.0-or-later
-
-/** \file
+/** @file dynamic_features.h
+ *
  * Dynamic Feature Record definition, creation, destruction, and conversion
  */
 
+// Copyright (C) 2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+
 #ifndef BASE_DYNAMIC_FEATURES_H_
 #define BASE_DYNAMIC_FEATURES_H_
 
@@ -74,6 +74,11 @@ get_dynamic_feature_metadata(
       Dynamic_Features_Rec *  dfr,
       uint8_t                 feature_code);
 
+// satisfies glib signature
+void
+free_feature_metadata(
+      gpointer data);    // i.e. DDCA_Feature_Metadata *
+
 
 void dbgrpt_dynamic_features_rec(
       Dynamic_Features_Rec*   dfr,
