@@ -1,25 +1,30 @@
 /** \file execution_stats.c
- * Record execution statistics, mainly the count and elapsed time of system calls.
+ *  Record execution statistics, mainly the count and elapsed time of system calls.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
+#include <errno.h>
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 /** \endcond */
 
+#include "ddcutil_status_codes.h"
+
 #include "util/glib_util.h"
 #include "util/report_util.h"
+#include "util/string_util.h"
 
 #include "base/core.h"
 #include "base/sleep.h"
 #include "base/parms.h"
 #include "base/ddc_errno.h"
+#include "base/linux_errno.h"
 
 #include "base/execution_stats.h"
 
@@ -53,7 +58,6 @@ typedef struct {
 } Status_Code_Counts;
 
 
-
 //
 // Global Variables
 //
@@ -73,7 +77,6 @@ static bool                 debug_global_stats_mutex = false;
 static bool                 debug_sleep_stats_mutex = false;
 
 
-
 //
 // IO Event Tracking
 //
@@ -118,7 +121,8 @@ const char * io_event_name(IO_Event_Type event_type) {
    return io_event_stats[event_type].name;
 }
 
-// unused
+
+#ifdef UNUSED
 int max_event_name_length() {
    int result = 0;
    int ndx = 0;
@@ -129,6 +133,7 @@ int max_event_name_length() {
    }
    return result;
 }
+#endif
 
 
 static int total_io_event_count() {
@@ -155,15 +160,8 @@ uint64_t normalize_timestamp(uint64_t timestamp) {
 }
 
 
-/** Called immediately after an I2C IO call, this function updates
- * two sets of data:
- *
- * 1) Updates the total number of calls and elapsed time for
- * categories of calls.
- *
- * 2) Updates the timestamp and call type maintained for the
- * most recent I2C call.  This information is used to determine
- * the required time for the next sleep call.
+/** Called immediately after an I2C IO call, this function updates the total
+ *  number of calls and elapsed time for categories of calls.
  *
  *  @param  event_type        e.g. IE_WRITE
  *  @param  location          function name
@@ -191,11 +189,6 @@ void log_io_call(
 
    DBGMSF(debug, "Updated total nanosec = %"PRIu64", as millis=%"PRIu64,
                   io_event_stats[event_type].call_nanosec, io_event_stats[event_type].call_nanosec /(1000*1000) );
-
-   // unused
-   // last_io_event = event_type;
-
-   // last_io_timestamp = normalize_timestamp(end_time_nanos);
 }
 
 
@@ -237,6 +230,31 @@ void report_io_call_stats(int depth) {
 }
 
 
+typedef struct {
+   int      count;
+   uint64_t nanos;
+} Non_Sleep_Call_Totals;
+
+Non_Sleep_Call_Totals
+get_non_sleep_call_totals () {
+   Non_Sleep_Call_Totals totals;
+   totals.count = 0;
+   totals.nanos = 0;
+    int ndx = 0;
+    // int max_name_length = max_event_name_length();
+
+    for (;ndx < IO_EVENT_TYPE_CT; ndx++) {
+       if (io_event_stats[ndx].call_count > 0) {
+          IO_Event_Type_Stats* curstat = &io_event_stats[ndx];
+
+          totals.count += curstat->call_count;
+          totals.nanos += curstat->call_nanosec;
+       }
+    }
+    return totals;
+ }
+
+
 //
 // Status Code Occurrence Tracking
 //
@@ -371,7 +389,7 @@ int compare( const void* a, const void* b)
 
 
 static
-void show_specific_status_counts(Status_Code_Counts * pcounts) {
+void report_specific_status_counts(Status_Code_Counts * pcounts, int depth) {
    bool debug = false;
    DBGMSF(debug, "Starting");
 
@@ -394,38 +412,26 @@ void show_specific_status_counts(Status_Code_Counts * pcounts) {
       }
    }
    int summed_ct = 0;
-   // fprintf(stdout, "DDC packet error status codes with non-zero counts:  %s\n",
-   fprintf(stdout, "%s:  %s\n",
-           title,
-           (keyct == 0) ? "None" : "");
+   rpt_vstring(depth, "%s:  %s\n", title, (keyct == 0) ? "None" : "");
    if (keyct > 0) {
       qsort(keysp, keyct, sizeof(gpointer), compare);    // sort keys
-      fprintf(stdout, "Count   Status Code                          Description\n");
+      rpt_vstring(depth, "Count   Status Code                          Description");
       int ndx;
       for (ndx=0; ndx<keyct; ndx++) {
          gpointer keyp = keysp[ndx];
-         long key = GPOINTER_TO_INT(keyp);                  // Public_Status_Code
-         // DBGMSF(debug, "key:  %d   %p", key, keyp);
-         // wrong hunch about bug
-         // if (key == 0) {
-         //    DBGMSG("=====> Invalid status code key = %d", key);
-         //    break;
-         // }
+         long key = GPOINTER_TO_INT(keyp);                  // DDCA_Status_Code
          assert( GINT_TO_POINTER(key) == keyp);
 
          int ct  = GPOINTER_TO_INT(g_hash_table_lookup(pcounts->error_counts_hash,keyp));
          summed_ct += ct;
-         // fprintf(stdout, "%4d    %6d\n", ct, key);
-
          Status_Code_Info * desc = find_status_code_info(key);
-
          // or consider flags in Status_Code_Info with this information
          char * aux_msg = "";
          if (ddcrc_is_derived_status_code(key))
             aux_msg = " (derived)";
          else if (ddcrc_is_not_error(key))
             aux_msg = " (not an error)";
-         fprintf(stdout, "%5d   %-28s (%5ld) %s %s\n",
+        rpt_vstring(depth, "%5d   %-28s (%5ld) %s %s",
               ct,
               (desc) ? desc->name : "",
               key,
@@ -434,22 +440,22 @@ void show_specific_status_counts(Status_Code_Counts * pcounts) {
              );
       }
    }
-   printf("Total errors: %d\n", pcounts->total_status_counts);
+   rpt_vstring(depth, "Total errors: %d", pcounts->total_status_counts);
    assert(summed_ct == pcounts->total_status_counts);
    g_free(keysp);
-   // fprintf(stdout,"\n");
    DBGMSF(debug, "Done");
 }
 
 
 /** Master function to display status counts
  */
-void show_all_status_counts() {
-   show_specific_status_counts(primary_error_code_counts);
+void report_all_status_counts(int depth) {
+   report_specific_status_counts(primary_error_code_counts, 0);
    // show_specific_status_counts(secondary_status_code_counts);    // not used
 
    rpt_nl();
-   show_specific_status_counts(retryable_error_code_counts);
+   report_specific_status_counts(retryable_error_code_counts, 0);
+   rpt_nl();
 }
 
 
@@ -486,7 +492,8 @@ int get_true_io_error_count(Status_Code_Counts * pcounts) {
 // Sleep Strategy
 //
 
-// names for Sleep_Event enum values
+
+// Names for Sleep_Event enum values
 static const char * sleep_event_names[] = {
       "SE_WRITE_TO_READ",
       "SE_POST_OPEN",
@@ -494,11 +501,30 @@ static const char * sleep_event_names[] = {
       "SE_POST_READ",
       "SE_DDC_NULL",
       "SE_POST_SAVE_SETTINGS",
+      "SE_PRE_EDID",
+      "SE_PRE_MULTI_PART_READ",
+      "SE_MULTI_PART_READ_TO_WRITE",
+      "SE_BETWEEN_CAP_TABLE_SEGMENTS",
+      "SE_POST_CAP_TABLE_COMMAND",
+      "SE_OTHER",
+      "SE_SPECIAL",
      };
 #define SLEEP_EVENT_ID_CT (sizeof(sleep_event_names)/sizeof(char *))
 
+int max_sleep_event_name_size() {
+   int result = 0;
+   for (int ndx = 0; ndx < SLEEP_EVENT_ID_CT; ndx++) {
+      if (strlen(sleep_event_names[ndx]) > result)
+         result = strlen(sleep_event_names[ndx]);
+   }
+   return result;
+}
+
 /** Returns the name of sleep event type */
 const char * sleep_event_name(Sleep_Event_Type event_type) {
+   // ensure sleep_event_names stays in sync with Sync_Event_Type
+   const int sleep_event_type_count = SE_SPECIAL+1;   // relies on values in enum assigned from 0
+   assert( SLEEP_EVENT_ID_CT ==  sleep_event_type_count);
    return sleep_event_names[event_type];
 }
 
@@ -530,12 +556,12 @@ void record_sleep_event(Sleep_Event_Type event_type) {
 }
 
 
-
 /** Reports execution statistics.
  *
  * @param depth logical indentation depth
  */
 void report_execution_stats(int depth) {
+   int sleep_name_field_size = max_sleep_event_name_size();
    int d1 = depth+1;
    rpt_title("IO and Sleep Events:", depth);
    rpt_vstring(d1, "Total IO events:      %5d", total_io_event_count());
@@ -544,7 +570,7 @@ void report_execution_stats(int depth) {
    rpt_nl();
    rpt_title("Sleep Event type      Count", d1);
    for (int id=0; id < SLEEP_EVENT_ID_CT; id++) {
-      rpt_vstring(d1, "%-21s  %4d", sleep_event_names[id], sleep_event_cts_by_id[id]);
+      rpt_vstring(d1, "%-*s  %4d", sleep_name_field_size, sleep_event_names[id], sleep_event_cts_by_id[id]);
    }
 }
 
@@ -607,3 +633,22 @@ void report_elapsed_stats(int depth) {
          elapsed_nanos / (1000*1000),
          elapsed_nanos);
 }
+
+void report_elapsed_summary(int depth) {
+   uint64_t end_nanos = cur_realtime_nanosec();
+   uint64_t elapsed_nanos = end_nanos - program_start_timestamp;
+
+   Non_Sleep_Call_Totals non_sleep_totals =  get_non_sleep_call_totals ();
+   Sleep_Stats sleep_totals =  get_sleep_stats();;
+
+   rpt_vstring(depth,
+               "Total non sleep system call time: %10"PRIu64" milliseconds",
+               non_sleep_totals.nanos / (1000*1000) );
+   rpt_vstring(depth,
+               "Total sleep call time:            %10"PRIu64" milliseconds",
+               sleep_totals.actual_sleep_nanos / (1000*1000) );
+   rpt_vstring(depth,
+               "Elapsed time:                     %10"PRIu64" milliseconds",
+               elapsed_nanos / (1000*1000));
+   rpt_nl();
+}
