@@ -3,7 +3,7 @@
  *  Functions and strings that are independent of the parser package used.
  */
 
-// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -156,6 +156,7 @@ const Feature_Subset_Table_Entry subset_table[] = {
    // special handling
    {VCP_SUBSET_KNOWN,     CMDID_GETVCP|CMDID_VCPINFO, 3, "KNOWN",     "All features known to ddcutil that are valid for the display"},
    {VCP_SUBSET_KNOWN,     CMDID_GETVCP|CMDID_VCPINFO, 3, "ALL",       "Same as KNOWN"},
+   {VCP_SUBSET_KNOWN,     CMDID_GETVCP|CMDID_VCPINFO, 3, "MCCS",      "Same as KNOWN"},
 // {VCP_SUBSET_SUPPORTED, CMDID_GETVCP,               3, "SUPPORTED", "All known features that are valid for the display"},
    {VCP_SUBSET_SCAN,      CMDID_GETVCP,               3, "SCAN",      "All feature codes 00..FF, except those known to be WO"},
    {VCP_SUBSET_MFG,       CMDID_GETVCP,               3, "MANUFACTURER", "Manufacturer specific codes"},
@@ -193,10 +194,10 @@ const int subset_table_ct = sizeof(subset_table)/sizeof(Feature_Subset_Table_Ent
 char * assemble_command_argument_help() {
    // quick and dirty check that tables are in sync
    // +2 for VCP_SUBSET_SINGLE_FEATURE, VCP_SUBSET_NONE
-   // -1 for double VCP_SUBSET_KNOWN
+   // -2 for triple VCP_SUBSET_KNOWN
    // -1 for double VCP_SUBSET_MFG
    // -1 for double VCP_SUBSET_DYNAMIC
-   assert(subset_table_ct+(2-3) == vcp_subset_count);
+   assert(subset_table_ct+(2-4) == vcp_subset_count);
 
    GString * buf = g_string_sized_new(1000);
    g_string_append(buf,
@@ -259,7 +260,7 @@ bool parse_feature_id_or_subset(char * val, int cmd_id, Feature_Set_Ref * fsref)
         fsref->specific_feature = feature_hexid;
      }
    }
-   DBGMSF(debug, "Returning: %s", bool_repr(ok));
+   DBGMSF(debug, "Returning: %s", sbool(ok));
    if (ok && debug)
       dbgrpt_feature_set_ref(fsref, 0);
    return ok;
@@ -398,7 +399,7 @@ char * tracing_comma_separated_option_help =
        "  The argument to --trace is a comma separated list of trace classes, surrounded by \n"
        "  quotation marks if necessary."
        "  e.g. --trace all, --trace \"I2C,ADL\"\n"
-       "  Valid trace classes are:  BASE, I2C, ADL, DDC, DDCIO, VCP, TOP, ENV, API, UDF, ALL.\n"
+       "  Valid trace classes are:  BASE, I2C, ADL, DDC, DDCIO, VCP, TOP, ENV, API, UDF, SLEEP, RETRY, ALL.\n"
        "  Trace class names are not case sensitive.\n"
        "  (Some trace classes are more useful than others.)\n"
   //   "\n"
@@ -408,7 +409,7 @@ char * tracing_multiple_call_option_help =
        "Trace by trace class:\n"
        "  The argument to --trace is a trace class.  Specify the --trace option multiple\n"
        "  times to activate multiple trace classes, e.g. \"--trace i2c --trace ddc\"\n"
-       "  Valid trace classes are:  BASE, I2C, ADL, DDC, DDCIO, VCP, TOP, ENV, API, UDF, ALL.\n"
+       "  Valid trace classes are:  BASE, I2C, ADL, DDC, DDCIO, VCP, TOP, ENV, API, UDF, SLEEP, RETRY, ALL.\n"
        "  Trace class names are not case sensitive.\n"
        "  (Some trace classes are more useful than others.)\n"
   //   "\n"
