@@ -3,7 +3,7 @@
  *  Parse the command line using the glib goption functions.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -177,6 +177,17 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 #ifdef USE_USB
    gboolean enable_usb_flag = true;
 #endif
+   gboolean timeout_i2c_io_flag = false;
+   gboolean reduce_sleeps_flag  = false;
+   gboolean deferred_sleep_flag = false;
+   gboolean dsa_flag       = false;
+   gboolean f1_flag        = false;
+   gboolean f2_flag        = false;
+   gboolean f3_flag        = false;
+   gboolean f4_flag        = false;
+   gboolean f5_flag        = false;
+   gboolean f6_flag        = false;
+   gboolean debug_parse_flag = false;
    char *   mfg_id_work    = NULL;
    char *   modelwork      = NULL;
    char *   snwork         = NULL;
@@ -203,8 +214,10 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       {"dis",    '\0',  0, G_OPTION_ARG_INT,      &dispwork,         "Display number",              "number"},
       {"bus",     'b',  0, G_OPTION_ARG_INT,      &buswork,          "I2C bus number",              "busnum" },
       {"hiddev", '\0',  0, G_OPTION_ARG_INT,      &hidwork,          "hiddev device number",        "number" },
+#ifdef HAVE_ADL
 //    {"adl",     'a',  0, G_OPTION_ARG_CALLBACK, adl_arg_func,      "ADL adapter and display indexes", "adapterIndex.displayIndex"},
       {"adl",     'a',  0, G_OPTION_ARG_STRING,   &adlwork,          "ADL adapter and display indexes", "adapterIndex.displayIndex"},
+#endif
       {"usb",     'u',  0, G_OPTION_ARG_STRING,   &usbwork,          "USB bus and device numbers",  "busnum.devicenum"},
       {"mfg",     'g',  0, G_OPTION_ARG_STRING,   &mfg_id_work,      "Monitor manufacturer code",   "mfg_id"},
       {"model",   'l',  0, G_OPTION_ARG_STRING,   &modelwork,        "Monitor model",               "model name"},
@@ -241,21 +254,36 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       // tuning
       {"maxtries",'\0', 0, G_OPTION_ARG_STRING,   &maxtrywork,       "Max try adjustment",  "comma separated list" },
       {"stats",   's',  G_OPTION_FLAG_OPTIONAL_ARG,
-                           G_OPTION_ARG_CALLBACK, stats_arg_func,    "Show retry statistics",    "stats type"},
+                           G_OPTION_ARG_CALLBACK, stats_arg_func,    "Show performance statistics",  "stats type"},
       {"force-slave-address",
                   '\0', 0, G_OPTION_ARG_NONE,     &force_slave_flag, "Force I2C slave address",         NULL},
       {"force",   'f',  G_OPTION_FLAG_HIDDEN,
                            G_OPTION_ARG_NONE,     &force_flag,       "Ignore certain checks",           NULL},
       {"verify",  '\0', 0, G_OPTION_ARG_NONE,     &verify_flag,      "Read VCP value after setting it", NULL},
       {"noverify",'\0', 0, G_OPTION_ARG_NONE,     &noverify_flag,    "Do not read VCP value after setting it", NULL},
-      {"nodetect",'\0', 0, G_OPTION_ARG_NONE,     &nodetect_flag,    "Skip initial monitor detection",  NULL},
+//    {"nodetect",'\0', 0, G_OPTION_ARG_NONE,     &nodetect_flag,    "Skip initial monitor detection",  NULL},
       {"async",   '\0', 0, G_OPTION_ARG_NONE,     &async_flag,       "Enable asynchronous display detection", NULL},
 
       {"udf",     '\0', 0, G_OPTION_ARG_NONE,     &enable_udf_flag,  "Enable user defined feature support", NULL},
       {"noudf",   '\0', G_OPTION_FLAG_REVERSE,
                            G_OPTION_ARG_NONE,     &enable_udf_flag,  "Disable user defined feature support", NULL},
+
+      // performance and retry
       {"sleep-multiplier", '\0', 0,
                            G_OPTION_ARG_STRING,   &sleep_multiplier_work, "Multiplication factor for DDC sleeps", "number"},
+      {"timeout-i2c-io",'\0', 0, G_OPTION_ARG_NONE, &timeout_i2c_io_flag, "Wrap I2C IO in timeout",  NULL},
+//    {"no-timeout-ddc-io",'\0',G_OPTION_FLAG_REVERSE,
+//                            G_OPTION_ARG_NONE,  &timeout_i2c_io_flag,   "Do not wrap DDC IO in timeout (default)",  NULL},
+      {"less-sleep" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps",  NULL},
+      {"sleep-less" ,'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps",  NULL},
+//    {"reduce-sleeps",'\0', 0, G_OPTION_ARG_NONE, &reduce_sleeps_flag, "Eliminate some sleeps",  NULL},
+//    {"no-reduce-sleeps",'\0',G_OPTION_FLAG_REVERSE,
+//                               G_OPTION_ARG_NONE,  &reduce_sleeps_flag, "Do not eliminate any sleeps (default)",  NULL},
+      {"lazy-sleep", '\0', 0, G_OPTION_ARG_NONE, &deferred_sleep_flag, "Delay sleeps if possible",  NULL},
+//    {"defer-sleeps",'\0', 0, G_OPTION_ARG_NONE, &deferred_sleep_flag, "Delay sleeps if possible",  NULL},
+      {"dynamic-sleep-adjustment",'\0', 0, G_OPTION_ARG_NONE, &dsa_flag, "Enable dynamic sleep adjustment",  NULL},
+      {"dsa",                     '\0', 0, G_OPTION_ARG_NONE, &dsa_flag, "Enable dynamic sleep adjustment",  NULL},
+
 
       // debugging
       {"excp",    '\0', 0, G_OPTION_ARG_NONE,     &report_freed_excp_flag,  "Report freed exceptions", NULL},
@@ -272,6 +300,13 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 //    {"myusage", '\0', 0, G_OPTION_ARG_NONE,     &myusage_flag,     "Show usage", NULL},
 //    {"myhelp",  '\0', 0,  G_OPTION_ARG_NONE,     &myhelp_flag,      "Show usage", NULL},
       {"i1",      '\0', 0,  G_OPTION_ARG_INT,      &i1_work, "special", "non-negative number" },
+      {"f1",      '\0', 0,  G_OPTION_ARG_NONE,     &f1_flag,         "Special flag 1",    NULL},
+      {"f2",      '\0', 0,  G_OPTION_ARG_NONE,     &f2_flag,         "Special flag 2",    NULL},
+      {"f3",      '\0', 0,  G_OPTION_ARG_NONE,     &f3_flag,         "Special flag 3",    NULL},
+      {"f4",      '\0', 0,  G_OPTION_ARG_NONE,     &f4_flag,         "Special flag 4",    NULL},
+      {"f5",      '\0', 0,  G_OPTION_ARG_NONE,     &f5_flag,         "Special flag 5",    NULL},
+      {"f6",      '\0', 0,  G_OPTION_ARG_NONE,     &f6_flag,         "Special flag 6",    NULL},
+      {"debug-parse", '\0', 0,  G_OPTION_ARG_NONE,  &debug_parse_flag,"Report parsed command",    NULL},
       {"failsim", '\0', 0,  G_OPTION_ARG_FILENAME, &failsim_fn_work, "Enable simulation", "control file name"},
 
       // other
@@ -381,6 +416,16 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
 #ifdef USE_USB
    SET_CMDFLAG(CMD_FLAG_ENABLE_USB,        enable_usb_flag);
 #endif
+   SET_CMDFLAG(CMD_FLAG_TIMEOUT_I2C_IO,    timeout_i2c_io_flag);
+   SET_CMDFLAG(CMD_FLAG_REDUCE_SLEEPS,     reduce_sleeps_flag);
+   SET_CMDFLAG(CMD_FLAG_DSA,               dsa_flag);
+   SET_CMDFLAG(CMD_FLAG_DEFER_SLEEPS,      deferred_sleep_flag);
+   SET_CMDFLAG(CMD_FLAG_F1,                f1_flag);
+   SET_CMDFLAG(CMD_FLAG_F2,                f2_flag);
+   SET_CMDFLAG(CMD_FLAG_F3,                f3_flag);
+   SET_CMDFLAG(CMD_FLAG_F4,                f4_flag);
+   SET_CMDFLAG(CMD_FLAG_F5,                f5_flag);
+   SET_CMDFLAG(CMD_FLAG_F6,                f6_flag);
 
    if (failsim_fn_work) {
 #ifdef ENABLE_FAILSIM
@@ -531,7 +576,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
              // DBGMSG("token=|%s|", token);
              if (strlen(token) > 0 && !streq(token,".")) {
                 int ival;
-                int ct = sscanf(token, "%d", &ival);
+                int ct = sscanf(token, "%ud", &ival);
                 if (ct != 1) {
                    fprintf(stderr, "Invalid --maxtries value: %s\n", token);
                    ok = false;
@@ -544,6 +589,9 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
                    parsed_cmd->max_tries[ndx] = ival;
                 }
              }
+             // split max multipart exchange into read and write
+             // not here, do it in main
+             // parsed_cmd->max_tries[3] = parsed_cmd->max_tries[2];
           }
           assert(ndx == ntsal);
        }
@@ -578,7 +626,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       float multiplier = 0.0f;
       bool arg_ok = str_to_float(sleep_multiplier_work, &multiplier);
       if (arg_ok) {
-         if (multiplier <= 0.0f)
+         if (multiplier <= 0.0f || multiplier >= 100.0)
             arg_ok = false;
       }
 
@@ -704,7 +752,7 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
       puts("");
       // if no command specified, include license in version information and terminate
       if (rest_ct == 0) {
-         puts("Copyright (C) 2015-2018 Sanford Rockowitz");
+         puts("Copyright (C) 2015-2020 Sanford Rockowitz");
          puts("License GPLv2: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>");
          puts("This is free software: you are free to change and redistribute it.");
          puts("There is NO WARRANTY, to the extent permitted by law.");
@@ -791,39 +839,36 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
          }
 
          if (ok && parsed_cmd->cmd_id == CMDID_SETVCP) {
-            if (parsed_cmd->argct == 3) {
-               if (streq(parsed_cmd->args[1],"+") || streq(parsed_cmd->args[1], "-")) {
-                  char * a1 = parsed_cmd->args[1];
-                  char * a2 = parsed_cmd->args[2];
-                  char * newarg1 = calloc(1, 1 + strlen(a2) + 1);
-                  strcpy(newarg1, a1);
-                  strcat(newarg1, a2);
-                  parsed_cmd->args[1] = newarg1;
-                  parsed_cmd->argct = 2;
+            for (int argpos = 0; argpos < parsed_cmd->argct; argpos+=2) {
+               // DBGMSG("argpos=%d, argct=%d", argpos, parsed_cmd->argct);
+               if ( (argpos+1) == parsed_cmd->argct) {
+                  fprintf(stderr, "Missing feature value\n");
+                  ok = false;
+                  break;
+               }
+               char * a1 = parsed_cmd->args[argpos+1];
+               if ( streq(a1,"+") || streq(a1,"-") ) {
+                  if ( (argpos+2) == parsed_cmd->argct) {
+                       fprintf(stderr, "Missing relative feature value\n");
+                       ok = false;
+                       break;
+                  }
+                  char * a2 = parsed_cmd->args[argpos+2];
+                  char * newval = calloc(1, 1 + strlen(a2) + 1);
+                  strcpy(newval, a1);
+                  strcat(newval, a2);
                   free(a1);
                   free(a2);
+                  parsed_cmd->args[argpos+1] = newval;
+                  int ndx = 0;
+                  for (ndx = argpos+2; ndx < (parsed_cmd->argct - 1); ndx++) {
+                     parsed_cmd->args[ndx] = parsed_cmd->args[ndx+1];
+                  }
+                  parsed_cmd->args[parsed_cmd->argct - 1] = NULL;
+                  parsed_cmd->argct--;
                }
             }
          }
-
-         // validate options vs commands
-
-#ifdef OLD
-         // unnecessary - validate_output_levels() handles this
-         switch (parsed_cmd->cmd_id) {
-         case (CMDID_PROBE):
-               if (parsed_cmd->output_level == DDCA_OL_TERSE) {
-                  // don't want to deal with how to report errors, write-only features
-                  fprintf(stderr, "probe command: option --terse unsupported");
-                  ok = false;
-               }
-               break;
-
-         default:
-            break;
-         }
-#endif
-
       }  // recognized command
    }
 
@@ -837,8 +882,10 @@ Parsed_Cmd * parse_command(int argc, char * argv[]) {
    //    free(trace_classes);   // trying to avoid valgrind error re g_option_context_parse() - doesn't solve it
    // }
 
-   if (debug)
+   if (debug || debug_parse_flag) {
+      DBGMSG("ok=%s", sbool(ok));
       dbgrpt_parsed_cmd(parsed_cmd, 0);
+   }
 
    if (!ok) {
       free_parsed_cmd(parsed_cmd);
