@@ -57,6 +57,16 @@ typedef enum {
    CMD_FLAG_WO_ONLY           = 0x040000,
    CMD_FLAG_ENABLE_UDF        = 0x100000,
    CMD_FLAG_ENABLE_USB        = 0x200000,
+   CMD_FLAG_TIMEOUT_I2C_IO    = 0x400000,
+   CMD_FLAG_REDUCE_SLEEPS     = 0x800000,
+   CMD_FLAG_F1              = 0x01000000,
+   CMD_FLAG_F2              = 0x02000000,
+   CMD_FLAG_F3              = 0x04000000,
+   CMD_FLAG_F4              = 0x08000000,
+   CMD_FLAG_F5              = 0x10000000,
+   CMD_FLAG_F6              = 0x20000000,
+   CMD_FLAG_DSA             = 0x40000000,
+   CMD_FLAG_DEFER_SLEEPS    = 0x80000000
 } Parsed_Cmd_Flags;
 
 
@@ -75,7 +85,7 @@ struct {
    gchar **            traced_files;
    gchar **            traced_functions;
    DDCA_Output_Level   output_level;
-   int                 max_tries[3];
+   uint16_t            max_tries[3];
    float               sleep_multiplier;
    uint32_t            flags;      // Parsed_Cmd_Flags
    int                 i1;         // available for temporary use
