@@ -2,7 +2,7 @@
  * Access displays, whether DDC, ADL, or USB
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -15,11 +15,13 @@
 #include <string.h>
 #include <time.h>
 
+#include "../i2c/i2c_strategy_dispatcher.h"
 #include "util/debug_util.h"
 #include "util/edid.h"
 #include "util/error_info.h"
 #include "util/failsim.h"
 #include "util/report_util.h"
+#include "util/string_util.h"
 #include "util/udev_usb_util.h"
 #include "util/udev_util.h"
 /** \endcond */
@@ -31,12 +33,11 @@
 #include "base/linux_errno.h"
 #include "base/monitor_model_key.h"
 #include "base/parms.h"
+#include "base/rtti.h"
 
 #include "vcp/vcp_feature_codes.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "i2c/i2c_do_io.h"
-
 #include "adl/adl_shim.h"
 #ifdef HAVE_ADL
 #include "adl/adl_impl/adl_intf.h"
@@ -46,7 +47,9 @@
 #include "usb/usb_displays.h"
 #endif
 
-#include <dynvcp/dyn_dynamic_features.h>
+#include "dynvcp/dyn_dynamic_features.h"
+
+#include "public/ddcutil_types.h"
 
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_vcp.h"
@@ -73,6 +76,15 @@ void ddc_set_async_threshold(int threshold) {
    async_threshold = threshold;
 }
 
+static inline bool
+value_bytes_zero_for_any_value(DDCA_Any_Vcp_Value * pvalrec) {
+   bool result = pvalrec && pvalrec->value_type ==  DDCA_NON_TABLE_VCP_VALUE &&
+                 pvalrec->val.c_nc.mh == 0 &&
+                 pvalrec->val.c_nc.ml == 0 &&
+                 pvalrec->val.c_nc.sh == 0 &&
+                 pvalrec->val.c_nc.sl == 0;
+   return result;
+}
 
 /** Collects initial monitor checks to perform them on a single open of the
  *  monitor device, and to avoid repeating them.
@@ -94,7 +106,7 @@ void ddc_set_async_threshold(int threshold) {
  *  ADL does not notice that a reported display, e.g. Dell 1905FP, does not support
  *  DDC.
  *  \remark
- *  Monitors are supposed to set the unsupported feature bit in a valid DDC
+ *  Monitors are supposed to set the unsupported feFFature bit in a valid DDC
  *  response, but a few monitors (mis)use the Null Response instead to indicate
  *  an unsupported feature. Others return with the unsupported feature bit not
  *  set, but all bytes (mh, ml, sh, sl) zero.
@@ -169,7 +181,6 @@ bool initial_checks_by_dh(Display_Handle * dh) {
             else {
                assert( psc == 0);
                assert(pvalrec);
-
             }
             assert( (psc == 0 && pvalrec) || (psc != 0 && !pvalrec) );
          }
@@ -188,11 +199,7 @@ bool initial_checks_by_dh(Display_Handle * dh) {
                      pvalrec->val.c_nc.ml,
                      pvalrec->val.c_nc.sh,
                      pvalrec->val.c_nc.sl);
-            if ( pvalrec->val.c_nc.mh == 0 &&
-                 pvalrec->val.c_nc.ml == 0 &&
-                 pvalrec->val.c_nc.sh == 0 &&
-                 pvalrec->val.c_nc.sl == 0
-              )
+            if (value_bytes_zero_for_any_value(pvalrec))
             {
                DBGTRC(debug, TRACE_GROUP, "Setting DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED");
                dh->dref->flags |= DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED;
@@ -223,6 +230,18 @@ bye:
       }
    }
 
+
+   if (!communication_working && i2c_force_bus) {
+      dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
+      communication_working = true;
+      DBGTRC(debug || true , TRACE_GROUP, "dh=%s, Forcing DDC communication success.",
+            dh_repr_t(dh) );
+      dh->dref->flags |= DREF_DDC_COMMUNICATION_WORKING;
+      dh->dref->flags |= DREF_DDC_USES_DDC_FLAG_FOR_UNSUPPORTED;   // good_enuf_for_test
+      dh->dref->flags |= DREF_DDC_COMMUNICATION_CHECKED;
+      dh->dref->vcp_version = DDCA_VSPEC_V22;   // good enuf for test
+   }
+
    DBGTRC(debug, TRACE_GROUP, "dh=%s, Returning: %s",
                  dh_repr_t(dh), sbool(communication_working));
    return communication_working;
@@ -481,9 +500,9 @@ ddc_report_display_by_dref(Display_Ref * dref, int depth) {
 
             if (dref->io_path.io_mode != DDCA_IO_USB) {
                rpt_vstring(d1, "Monitor returns DDC Null Response for unsupported features: %s",
-                                  bool_repr(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
+                                  sbool(dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED));
                // rpt_vstring(d1, "Monitor returns success with mh=ml=sh=sl=0 for unsupported features: %s",
-               //                    bool_repr(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
+               //                    sbool(dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED));
             }
          }
       }
@@ -1054,6 +1073,11 @@ ddc_ensure_displays_detected() {
 
 }
 
+void ddc_discard_detected_displays() {
+   all_displays = NULL;
+   i2c_discard_buses();
+}
+
 
 /** Indicates whether displays have already been detected
  *
@@ -1103,3 +1127,11 @@ ddc_is_usb_display_detection_enabled() {
    return detect_usb_displays;
 }
 
+
+void
+init_ddc_displays() {
+   rtti_func_name_table_add(initial_checks_by_dh, "initial_checks_by_dh");
+   // dbgrpt_func_name_table(0);
+}
+
+
