@@ -1,31 +1,10 @@
-/* dumpload.c
+/** \file dumpload.c
  *
  * Load/store VCP settings from/to file.
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
  */
 
-/** \file
- *
- */
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
@@ -37,25 +16,29 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-/** \endcond */
 
 #include "util/file_util.h"
 #include "util/glib_util.h"
 #include "util/glib_string_util.h"
 #include "util/report_util.h"
 
+#include "ddcutil_types.h"
+/** \endcond */
+
 #include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/ddc_packets.h"
 #include "base/displays.h"
+#include "base/monitor_model_key.h"
 #include "base/parms.h"
 #include "base/status_code_mgt.h"
 #include "base/vcp_version.h"
 
+#include "dynvcp/dyn_feature_codes.h"
+
 #include "i2c/i2c_bus_core.h"
 
 #include "ddc/ddc_displays.h"
-#include "ddc/ddc_edid.h"
 #include "ddc/ddc_output.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_read_capabilities.h"
@@ -71,21 +54,23 @@
  *                if NULL, do nothing
  */
 void free_dumpload_data(Dumpload_Data * data) {
+   bool debug = false;
+   DBGMSF(debug, "Starting. data=%p", data);
    if (data) {
       if (data->vcp_values)
          free_vcp_value_set(data->vcp_values);
       free(data);
    }
+   DBGMSF(debug, "Done.");
 }
 
 
 /** Reports the contents of a #Dumpload_Data struct
  *
- * Arguments:
- *    data     pointer to #Dumpload_Data struct
- *    depth    logical indentation depth
+ *  @param  data     pointer to #Dumpload_Data struct
+ *  @param  depth    logical indentation depth
  */
-void report_dumpload_data(Dumpload_Data * data, int depth) {
+void dbgrpt_dumpload_data(Dumpload_Data * data, int depth) {
    int d1 = depth+1;
    rpt_structure_loc("Dumpload_Data", data, depth);
    // rptIval("busno", NULL, data->busno, d1);
@@ -93,29 +78,43 @@ void report_dumpload_data(Dumpload_Data * data, int depth) {
    // TODO: show abbreviated edidstr
    rpt_str( "mfg_id",       NULL, data->mfg_id,       d1);
    rpt_str( "model",        NULL, data->model,        d1);
+   rpt_unsigned(
+           " product_code", NULL, data->product_code, d1);
    rpt_str( "serial_ascii", NULL, data->serial_ascii, d1);
    rpt_str( "edid",         NULL, data->edidstr,      d1);
    rpt_str( "vcp_version",  NULL, format_vspec(data->vcp_version), d1);
    rpt_int( "vcp_value_ct", NULL, data->vcp_value_ct, d1);
    rpt_structure_loc("vcp_values", data->vcp_values, d1);
    if (data->vcp_values)
-      report_vcp_value_set(data->vcp_values, d1);
+      dbgrpt_vcp_value_set(data->vcp_values, d1);
 }
 
+#define ADD_DATA_ERROR(_expl) \
+      errinfo_add_cause(  \
+         errs,            \
+         errinfo_new2(    \
+            DDCRC_BAD_DATA, __func__, \
+            _expl " at line %d: %s", linectr, line) );
+
 
 /** Given an array of strings stored in a GPtrArray,
- * convert it a #Dumpload_Data struct.
+ *  convert it a #Dumpload_Data struct.
  *
- * @param   garray      array of strings
- *
- * @return  pointer to newly allocated Dumpload_Data struct, or
+ *  @param   garray      array of strings
+ *  @return  pointer to newly allocated Dumpload_Data struct, or
  *                NULL if the data is not valid.
  *                It is the responsibility of the caller to free this struct.
  */
-Dumpload_Data*
-create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
+Error_Info *
+create_dumpload_data_from_g_ptr_array(
+      GPtrArray * garray,
+      Dumpload_Data ** dumpload_data_loc)
+{
    bool debug = false;
-   DBGMSF0(debug, "Starting.");
+   DBGMSF(debug, "Starting.");
+
+   Error_Info * errs = errinfo_new(DDCRC_BAD_DATA, __func__);
+   *dumpload_data_loc = NULL;
 
    Dumpload_Data * data = calloc(1, sizeof(Dumpload_Data));
    bool valid_data = true;
@@ -142,7 +141,12 @@ create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
       ct = sscanf(head, "%31s %256s %15s", s0, s1, s2);
       if (ct > 0 && *s0 != '*' && *s0 != '#') {
          if (ct == 1) {
-            printf("Invalid data at line %d: %s\n", linectr, line);
+            // printf("Invalid data at line %d: %s\n", linectr, line);
+            // Error_Info * err = errinfo_new2(
+            //                       DDCRC_BAD_DATA, __func__,
+            //                       "Invalid data at line %d: %s", linectr, line);
+            // errinfo_add_cause(errs, err);
+            ADD_DATA_ERROR("Invalid data");
             valid_data = false;
          }
          else {
@@ -163,24 +167,38 @@ create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
                //    valid_data = false;
                // }
             }
+            else if (streq(s0, "PRODUCT_CODE")) {
+               int ival;
+               bool ok = str_to_int(s1, &ival, 10);
+               if (ok && ival >= 0 && ival <= 65535)
+                  data->product_code = (uint16_t) ival;
+               else
+                  valid_data = false;
+            }
             else if (streq(s0, "EDID") || streq(s0, "EDIDSTR")) {
-               strncpy(data->edidstr, s1, sizeof(data->edidstr));
+               g_strlcpy(data->edidstr, s1, sizeof(data->edidstr));
             }
             else if (streq(s0, "MFG_ID")) {
-               strncpy(data->mfg_id, s1, sizeof(data->mfg_id));
+               g_strlcpy(data->mfg_id, s1, sizeof(data->mfg_id));
             }
             else if (streq(s0, "MODEL")) {
-               strncpy(data->model, rest, sizeof(data->model));
+               g_strlcpy(data->model, rest, sizeof(data->model));
             }
             else if (streq(s0, "SN")) {
-               strncpy(data->serial_ascii, rest, sizeof(data->serial_ascii));
+               g_strlcpy(data->serial_ascii, rest, sizeof(data->serial_ascii));
             }
             else if (streq(s0, "VCP_VERSION")) {
                data->vcp_version = parse_vspec(s1);
                // using VCP_SPEC_UNKNOWN as value when invalid,
                // what if monitor had no version, so 0.0 was output?
-               if ( vcp_version_eq( data->vcp_version, VCP_SPEC_UNKNOWN) ) {
-                  f0printf(FERR, "Invalid VCP VERSION at line %d: %s\n", linectr, line);
+               if ( vcp_version_eq( data->vcp_version, DDCA_VSPEC_UNKNOWN) ) {
+                  // f0printf(ferr(), "Invalid VCP VERSION at line %d: %s\n", linectr, line);
+                  // errinfo_add_cause(
+                  //            errs,
+                  //            errinfo_new2(
+                  //                  DDCRC_BAD_DATA, __func__,
+                  //                  "Invalid VCP VERSION at line %d: %s\n", linectr, line) );
+                  ADD_DATA_ERROR("Invalid VCP VERSION");
                   valid_data = false;
                }
             }
@@ -192,18 +210,20 @@ create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
             }
             else if (streq(s0, "VCP")) {
                if (ct != 3) {
-                  f0printf(FERR, "Invalid VCP data at line %d: %s\n", linectr, line);
+                  // f0printf(ferr(), "Invalid VCP data at line %d: %s\n", linectr, line);
+                  ADD_DATA_ERROR("Invalid VCP data");
                   valid_data = false;
                }
                else {   // found feature id and value
                   Byte feature_id;
                   bool ok = hhs_to_byte_in_buf(s1, &feature_id);
                   if (!ok) {
-                     f0printf(FERR, "Invalid opcode at line %d: %s", linectr, s1);
+                     // f0printf(ferr(), "Invalid opcode at line %d: %s", linectr, s1);
+                     ADD_DATA_ERROR("Invalid  opcode");
                      valid_data = false;
                   }
                   else {     // valid opcode
-                     DDCA_Single_Vcp_Value * valrec = NULL;
+                     DDCA_Any_Vcp_Value * valrec = NULL;
                      // look up opcode, is it valid?
 
                      // table values need special handling
@@ -213,20 +233,26 @@ create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
                      // One solution: rework data structures to parse later
                      // second solution: vcp version in dumpload data
 
-                     VCP_Feature_Table_Entry * pvft_entry =
-                         vcp_find_feature_by_hexid_w_default(feature_id);
+                     DDCA_Monitor_Model_Key mmk = monitor_model_key_value(
+                           data->mfg_id, data->model, data->product_code);
 
-                     bool is_table_feature = is_feature_table_by_vcp_version(
-                                                pvft_entry,data->vcp_version);
+                     Display_Feature_Metadata * dfm =
+                                              dyn_get_feature_metadata_by_mmk_and_vspec_dfm(
+                                                   feature_id,
+                                                   mmk,
+                                                   data->vcp_version,
+                                                   /*with_default=*/ true);
+                     bool is_table_feature = dfm->feature_flags & DDCA_NORMAL_TABLE;
 
                      if (is_table_feature) {
                         // s2 is hex string
                         Byte * ba;
                         int bytect =  hhs_to_byte_array(s2, &ba);
                         if (bytect < 0) {
-                           f0printf(FERR,
-                                    "Invalid hex string value for opcode at line %d: %s\n",
-                                    linectr, line);
+                           // f0printf(ferr(),
+                           //          "Invalid hex string value for opcode at line %d: %s\n",
+                           //          linectr, line);
+                           ADD_DATA_ERROR("Invalid hex string value for opcode");
                            valid_data = false;
                         }
                         else {
@@ -241,7 +267,8 @@ create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
                         ushort feature_value;
                         ct = sscanf(s2, "%hu", &feature_value);
                         if (ct == 0) {
-                           f0printf(FERR, "Invalid value for opcode at line %d: %s\n", linectr, line);
+                           // f0printf(ferr(), "Invalid value for opcode at line %d: %s\n", linectr, line);
+                           ADD_DATA_ERROR("Invalid value for opcode");
                            valid_data = false;
                         }
                         else {
@@ -257,28 +284,43 @@ create_dumpload_data_from_g_ptr_array(GPtrArray * garray) {
                         data->vcp_value_ct++;
                         vcp_value_set_add(data->vcp_values, valrec);
                      }
+                     dfm_free(dfm);
                   } // valid opcode
 
                } // found feature id and value
             }  // VCP
 
             else {
-               f0printf(FERR, "Unexpected field \"%s\" at line %d: %s\n", s0, linectr, line );
+               // f0printf(ferr(), "Unexpected field \"%s\" at line %d: %s\n", s0, linectr, line );
+               errinfo_add_cause(
+                          errs,
+                          errinfo_new2(
+                                DDCRC_BAD_DATA, __func__,
+                                "Unexpected field \"%s\" at line %d: %s", s0, linectr, line) );
                valid_data = false;
             }
          }    // more than 1 field on line
       }       // non-comment line
    }          // one line of file
 
-   if (!valid_data) {
+   assert( ( valid_data && errs->cause_ct == 0 ) ||
+           (!valid_data && errs->cause_ct >  0) );
+   if (errs->cause_ct == 0) {
+      errinfo_free(errs);
+      errs = NULL;
+   }
+   else {
       if (data) {
          free_dumpload_data(data);
          data = NULL;
       }
    }
-   return data;
+   *dumpload_data_loc = data;
+   return errs;
 }
 
+#undef ADD_DATA_ERROR
+
 
 /** Sets multiple VCP values.
  *
@@ -300,35 +342,24 @@ ddc_set_multiple(
 
    int ndx;
    for (ndx=0; ndx < value_ct; ndx++) {
-      DDCA_Single_Vcp_Value * vrec = vcp_value_set_get(vset, ndx);
+      DDCA_Any_Vcp_Value * vrec
+      = vcp_value_set_get(vset, ndx);
       Byte   feature_code = vrec->opcode;
 
-#ifdef OLD
-      assert(vrec->value_type == DDCA_NON_TABLE_VCP_VALUE);     // Table not yet implemented
-      ushort new_value    = vrec->val.c.cur_val;
-      psc = set_nontable_vcp_value(dh, feature_code, new_value);
-      if (psc != 0) {
-         f0printf(FERR, "Error setting value %d for VCP feature code 0x%02x: %s\n",
-                         new_value, feature_code, psc_desc(psc) );
-         f0printf(FERR, "Terminating.");
-         break;
-      }
-#endif
-
       // HACK: will this affect intermittent error of silently failing sets?
       // pointless, ddc_it2_write_only) calls call_tuned_sleep() after write
       // if (ndx > 0) {
       //    sleep_millis_with_trace(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, "before set_vcp_value()");
       // }
 
-      ddc_excp = set_vcp_value(dh, vrec);
+      ddc_excp = ddc_set_vcp_value(dh, vrec, NULL);
       psc = (ddc_excp) ? ddc_excp->status_code : 0;
       if (ddc_excp) {
-         f0printf(FERR, "Error setting value for VCP feature code 0x%02x: %s\n",
+         f0printf(ferr(), "Error setting value for VCP feature code 0x%02x: %s\n",
                          feature_code, psc_desc(psc) );
          if (psc == DDCRC_RETRIES)
-            f0printf(FERR, "    Try errors: %s\n", errinfo_causes_string(ddc_excp));
-         f0printf(FERR, "Terminating.");
+            f0printf(ferr(), "    Try errors: %s\n", errinfo_causes_string(ddc_excp));
+         f0printf(ferr(), "Terminating.");
          break;
       }
 
@@ -354,12 +385,13 @@ loadvcp_by_dumpload_data(
       Display_Handle *  dh)
 {
    assert(pdata);
+   FILE * errf = ferr();
 
    bool debug = false;
    if (debug) {
         DBGMSG("Loading VCP settings for monitor \"%s\", sn \"%s\", dh=%p \n",
                pdata->model, pdata->serial_ascii, dh);
-        report_dumpload_data(pdata, 0);
+        dbgrpt_dumpload_data(pdata, 0);
    }
 
    Public_Status_Code psc = 0;
@@ -371,13 +403,13 @@ loadvcp_by_dumpload_data(
       assert(dh->dref->pedid);
       bool ok = true;
       if ( !streq(dh->dref->pedid->model_name, pdata->model) ) {
-         f0printf(FERR,
+         f0printf(errf,
             "Monitor model in data (%s) does not match that for specified device (%s)\n",
             pdata->model, dh->dref->pedid->model_name);
          ok = false;
       }
       if (!streq(dh->dref->pedid->serial_ascii, pdata->serial_ascii) ) {
-         f0printf(FERR,
+         f0printf(errf,
             "Monitor serial number in data (%s) does not match that for specified device (%s)\n",
             pdata->serial_ascii, dh->dref->pedid->serial_ascii);
          ok = false;
@@ -390,7 +422,7 @@ loadvcp_by_dumpload_data(
 
    else if ( strlen(pdata->mfg_id) + strlen(pdata->model) + strlen(pdata->serial_ascii) == 0) {
       // Pathological.  Someone's been messing with the VCP file.
-      f0printf(FERR, "Monitor manufacturer id, model, and serial number all missing from input.\n");
+      f0printf(errf, "Monitor manufacturer id, model, and serial number all missing from input.\n");
       psc = DDCRC_INVALID_DISPLAY;
       goto bye;
    }
@@ -406,7 +438,7 @@ loadvcp_by_dumpload_data(
                               did, CALLOPT_NONE);
       free_display_identifier(did);
       if (!dref) {
-         f0printf(FERR, "Monitor not connected: %s - %s   \n", pdata->model, pdata->serial_ascii );
+         f0printf(errf, "Monitor not connected: %s - %s   \n", pdata->model, pdata->serial_ascii );
          psc = DDCRC_INVALID_DISPLAY;
          goto bye;
       }
@@ -455,24 +487,27 @@ loadvcp_by_ntsa(
       DBGMSG("Starting.  ntsa=%p", ntsa);
       verbose = true;
    }
-   Public_Status_Code psc = 0;
+   // Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
 
    GPtrArray * garray = ntsa_to_g_ptr_array(ntsa);
 
-   Dumpload_Data * pdata = create_dumpload_data_from_g_ptr_array(garray);
+   Dumpload_Data * pdata = NULL;
+   ddc_excp = create_dumpload_data_from_g_ptr_array(garray, &pdata);
    DBGMSF(debug, "create_dumpload_data_from_g_ptr_array() returned %p", pdata);
+   assert( (ddc_excp == NULL && pdata != NULL) ||
+           (ddc_excp != NULL && pdata == NULL) );
    if (!pdata) {
-      f0printf(FERR, "Unable to load VCP data from string\n");
-      psc = DDCRC_INVALID_DATA;
-      ddc_excp = errinfo_new(psc, __func__);
+      // f0printf(ferr(), "Unable to load VCP data from string\n");
+      // psc = DDCRC_ARG;     // was DDCRC_INVALID_DATA;
+      // ddc_excp = errinfo_new(psc, __func__);
    }
    else {
       if (verbose) {
-           f0printf(FOUT, "Loading VCP settings for monitor \"%s\", sn \"%s\" \n",
+           f0printf(fout(), "Loading VCP settings for monitor \"%s\", sn \"%s\" \n",
                            pdata->model, pdata->serial_ascii);
-           rpt_push_output_dest(FOUT);
-           report_dumpload_data(pdata, 0);
+           rpt_push_output_dest(fout());
+           dbgrpt_dumpload_data(pdata, 0);
            rpt_pop_output_dest();
       }
       ddc_excp = loadvcp_by_dumpload_data(pdata, dh);
@@ -594,8 +629,11 @@ collect_machine_readable_timestamp(time_t time_millis, GPtrArray* vals) {
    snprintf(buf, bufsz, "TIMESTAMP_TEXT %s", timestamp_buf );
    g_ptr_array_add(vals, strdup(buf));
 
-   snprintf(buf, bufsz, "TIMESTAMP_MILLIS %ld", time_millis);
-   g_ptr_array_add(vals, strdup(buf));
+   // time_t is problematic.  %jd handles x32_abi where time_t is of type long long
+   // but per the c11 spec time_t can be a real type
+   // Value is ignored on input, so just don't output it and avoid the issues. (11/2018)
+   // snprintf(buf, bufsz, "TIMESTAMP_MILLIS %jd", time_millis);
+   // g_ptr_array_add(vals, strdup(buf));
 }
 
 #ifdef UNUSED
@@ -621,7 +659,7 @@ collect_profile_related_values(
       GPtrArray**      pvals)
 {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
    assert( get_output_level() == OL_PROGRAM);
    Public_Status_Code psc = 0;
    GPtrArray * vals = g_ptr_array_sized_new(50);
@@ -662,7 +700,7 @@ dumpvcp_as_dumpload_data(
       Dumpload_Data** pdumpload_data)
 {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
    Public_Status_Code psc = 0;
    Dumpload_Data * dumped_data = calloc(1, sizeof(Dumpload_Data));
 
@@ -676,6 +714,7 @@ dumpvcp_as_dumpload_data(
    Parsed_Edid * edid = dh->dref->pedid;
    assert(edid);
 
+   dumped_data->product_code = edid->product_code;
    memcpy(dumped_data->mfg_id, edid->mfg_id, sizeof(dumped_data->mfg_id));
    memcpy(dumped_data->model,  edid->model_name, sizeof(dumped_data->model));
    memcpy(dumped_data->serial_ascii, edid->serial_ascii, sizeof(dumped_data->serial_ascii));
@@ -688,12 +727,12 @@ dumpvcp_as_dumpload_data(
 
    // VCP values
    Vcp_Value_Set vset = vcp_value_set_new(50);
-   psc = collect_raw_subset_values(
+   psc = ddc_collect_raw_subset_values(
              dh,
              VCP_SUBSET_PROFILE,
              vset,
              true,               //  ignore_unsupported
-             FERR);
+             ferr());
    if (psc == 0) {
       dumped_data->vcp_values = vset;             // NOTE REFERENCE, BE CAREFUL WHEN FREE
       dumped_data->vcp_value_ct = vcp_value_set_size(vset);
@@ -705,7 +744,7 @@ dumpvcp_as_dumpload_data(
       *pdumpload_data = dumped_data;
    if (debug) {
       DBGMSG("Returning: %s, *pdumpload_data=%p", psc_desc(psc), *pdumpload_data);
-      report_dumpload_data(*pdumpload_data, 1);
+      dbgrpt_dumpload_data(*pdumpload_data, 1);
    }
    return psc;
 }
@@ -725,9 +764,10 @@ convert_dumpload_data_to_string_array(Dumpload_Data * data) {
    DBGMSF(debug, "Starting. data=%p", data);
    assert(data);
    if (debug)
-      report_dumpload_data(data, 1);
+      dbgrpt_dumpload_data(data, 1);
 
    GPtrArray * strings = g_ptr_array_sized_new(30);
+   g_ptr_array_set_free_func(strings, g_free);
 
    collect_machine_readable_timestamp(data->timestamp_millis, strings);
 
@@ -737,6 +777,8 @@ convert_dumpload_data_to_string_array(Dumpload_Data * data) {
    g_ptr_array_add(strings, strdup(buf));
    snprintf(buf, bufsz, "MODEL   %s",  data->model);
    g_ptr_array_add(strings, strdup(buf));
+   snprintf(buf, bufsz, "PRODUCT_CODE  %d", data->product_code);
+   g_ptr_array_add(strings, strdup(buf));
    snprintf(buf, bufsz, "SN      %s",  data->serial_ascii);
    g_ptr_array_add(strings, strdup(buf));
 
@@ -748,17 +790,18 @@ convert_dumpload_data_to_string_array(Dumpload_Data * data) {
    snprintf(buf, bufsz, "EDID    %s", hexbuf);
    g_ptr_array_add(strings, strdup(buf));
 
-   if (!vcp_version_eq(data->vcp_version, VCP_SPEC_UNKNOWN)) {
+   if (!vcp_version_eq(data->vcp_version, DDCA_VSPEC_UNKNOWN)) {
       snprintf(buf, bufsz, "VCP_VERSION %d.%d", data->vcp_version.major, data->vcp_version.minor);
       g_ptr_array_add(strings, strdup(buf));
    }
 
    for (int ndx=0; ndx < data->vcp_values->len; ndx++) {
       // n. get_formatted_value_for_feature_table_entry() also has code for table type values
-      DDCA_Single_Vcp_Value * vrec = vcp_value_set_get(data->vcp_values,ndx);
+      DDCA_Any_Vcp_Value * vrec = vcp_value_set_get(data->vcp_values,ndx);
       char buf[200];
       snprintf(buf, 200, "VCP %02X %5d",
-                         vrec->opcode, vrec->val.c.cur_val);
+                         vrec->opcode,
+                         VALREC_CUR_VAL(vrec));
       g_ptr_array_add(strings, strdup(buf));
    }
    return strings;
@@ -779,7 +822,7 @@ convert_dumpload_data_to_string_array(Dumpload_Data * data) {
 Public_Status_Code
 dumpvcp_as_string(Display_Handle * dh, char ** pstring) {
    bool debug = false;
-   DBGMSF0(debug, "Starting");
+   DBGMSF(debug, "Starting");
 
    Public_Status_Code psc    = 0;
    Dumpload_Data *    data   = NULL;
