@@ -1,41 +1,22 @@
-/* ddc_multi_part_io.c
+/** \file ddc_multi_part_io.c
  *
- * Handles multi-part reads and writes used for Table features and
+ * Handle multi-part DDC reads and writes used for Table features and
  * Capabilities.
- *
- * <copyright>
- * Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
  */
 
-/** \file
- * Multi-part DDC reads and writes used for Table features and
- * Capabilities retrieval.
- */
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include "public/ddcutil_types.h"
+
 #include "util/string_util.h"
 /** \endcond */
 
@@ -45,6 +26,7 @@
 #include "base/execution_stats.h"
 #include "base/parms.h"
 #include "base/rtti.h"
+#include "base/tuned_sleep.h"
 
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_try_stats.h"
@@ -53,94 +35,9 @@
 
 
 // Trace management
-
 static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_DDC;
 
 
-// Retry management and statistics
-
-/** \def MAX_MAX_MULTI_EXCHANGE_TRIES
- * Maximum value to which maximum number of capabilities exchange tries can be set
- */
-#define MAX_MAX_MULTI_EXCHANGE_TRIES  MAX_MAX_TRIES     /* from parms.h */
-
-/** \def Maximum number of capabilities exchange tries allowed. Can be adjusted. */
-static int max_multi_part_read_tries = MAX_MULTI_EXCHANGE_TRIES;
-
-static int max_multi_part_write_tries = MAX_MULTI_EXCHANGE_TRIES;
-
-static void * multi_part_read_stats_rec = NULL;
-static void * multi_part_write_stats_rec = NULL;
-
-/** Resets the statistics for multi-part reads */
-void ddc_reset_multi_part_read_stats() {
-   if (multi_part_read_stats_rec)
-      try_data_reset(multi_part_read_stats_rec);
-   else
-      multi_part_read_stats_rec = try_data_create("multi-part read exchange", max_multi_part_read_tries);
-}
-
-
-/** Resets the statistics for multi-part writes */
-void ddc_reset_multi_part_write_stats() {
-   if (multi_part_write_stats_rec)
-      try_data_reset(multi_part_write_stats_rec);
-   else
-      multi_part_write_stats_rec = try_data_create("multi-part write exchange", max_multi_part_write_tries);
-}
-
-
-/** Reports the statistics for multi-part reads */
-void ddc_report_multi_part_read_stats(int depth) {
-   assert(multi_part_read_stats_rec);
-   try_data_report(multi_part_read_stats_rec, depth);
-}
-
-
-/** Reports the statistics for multi-part writes */
-void ddc_report_multi_part_write_stats(int depth) {
-   assert(multi_part_write_stats_rec);
-   try_data_report(multi_part_write_stats_rec, depth);
-}
-
-
-/** Resets the maximum number of multi-part read exchange tries allowed.
- *  @param ct new maximum number, must be <= #MAX_MAX_MULTI_EXCHANGE_TRIES
- */
-void ddc_set_max_multi_part_read_tries(int ct) {
-   assert(ct > 0 && ct <= MAX_MAX_MULTI_EXCHANGE_TRIES);
-   max_multi_part_read_tries = ct;
-   if (multi_part_read_stats_rec)
-         try_data_set_max_tries(multi_part_read_stats_rec, ct);
-}
-
-/** Resets the maximum number of multi-part write exchange tries allowed.
- *  @param ct new maximum number, must be <= #MAX_MAX_MULTI_EXCHANGE_TRIES
- */
-void ddc_set_max_multi_part_write_tries(int ct) {
-   assert(ct > 0 && ct <= MAX_MAX_MULTI_EXCHANGE_TRIES);
-   max_multi_part_write_tries = ct;
-   if (multi_part_write_stats_rec)
-         try_data_set_max_tries(multi_part_write_stats_rec, ct);
-}
-
-
-/** Gets the current maximum number of multi-part read exchange tries allowed
-  * @return maximum number of tries
-  */
-int ddc_get_max_multi_part_read_tries() {
-   return max_multi_part_read_tries;
-}
-
-
-/** Gets the current maximum number of multi-part write exchange tries allowed
-  * @return maximum number of tries
-  */
-int ddc_get_max_multi_part_write_tries() {
-   return max_multi_part_write_tries;
-}
-
-
 /** Makes one attempt to read the entire capabilities string or table feature value
 *
 * @param  dh             display handle for open i2c or adl device
@@ -148,9 +45,9 @@ int ddc_get_max_multi_part_write_tries() {
 * @param  request_subtype  VCP feature code for table read, ignore for capabilities
 * @param  all_zero_response_ok  if true, an all zero response is not regarded
 *         as an error
-* @param  accumulator    buffer in which to return result
+* @param  accumulator    buffer in which to return result (already allocated)
 *
-* @return status code
+* @return @Error_Info struct with error detail, NULL if no error
 */
 static Error_Info *
 try_multi_part_read(
@@ -163,7 +60,7 @@ try_multi_part_read(
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP,
           "Starting. request_type=0x%02x, request_subtype=x%02x, all_zero_response_ok=%s, accumulator=%p",
-          request_type, request_subtype, bool_repr(all_zero_response_ok), accumulator);
+          request_type, request_subtype, sbool(all_zero_response_ok), accumulator);
 
    const int MAX_FRAGMENT_SIZE = 32;
    const int readbuf_size = 6 + MAX_FRAGMENT_SIZE + 1;
@@ -174,7 +71,10 @@ try_multi_part_read(
    DDC_Packet * request_packet_ptr  = NULL;
    DDC_Packet * response_packet_ptr = NULL;
    request_packet_ptr = create_ddc_multi_part_read_request_packet(
-                           request_type, request_subtype, 0, "try_multi_part_read");
+                           request_type,
+                           request_subtype,
+                           0,
+                           "try_multi_part_read");
    buffer_set_length(accumulator,0);
    int  cur_offset = 0;
    bool complete   = false;
@@ -201,6 +101,7 @@ try_multi_part_read(
       DBGTRC(debug, DDCA_TRC_NONE,
              "ddc_write_read_with_retry() request_type=0x%02x, request_subtype=0x%02x, returned %s",
              request_type, request_subtype, errinfo_summary(excp));
+      TUNED_SLEEP_WITH_TRACE(dh, SE_AFTER_EACH_CAP_TABLE_SEGMENT, NULL);
 
       if (excp) {
       // if (psc != 0) {
@@ -212,7 +113,7 @@ try_multi_part_read(
       assert(!excp && psc == 0);
 
       if ( IS_TRACING_BY_FUNC_OR_FILE() || debug ) {
-         DBGMSG0("After try_write_read():");
+         DBGMSG("After try_write_read():");
          dbgrpt_interpreted_multi_read_fragment(response_packet_ptr->parsed.multi_part_read_fragment, 0);
       }
 
@@ -257,13 +158,14 @@ try_multi_part_read(
 
 
 /** Gets the DDC capabilities string for a monitor, performing retries if necessary.
+ *  Also used for VCP features of type Table.
 *
-*  @param  dh handle of open display
+*  @param  dh                    handle of open display
 *  @param  request_type
-*  @param  request_subtype  VCP function code for table read, ignore for capabilities
-*  @param  all_zero_response_ok   if true, zero response is not an error
-*  @param  pp_buffer  address at which to return newly allocated #Buffer in which
-*                   result is returned
+*  @param  request_subtype       VCP function code for table read, ignore for capabilities
+*  @param  all_zero_response_ok  if true, zero response is not an error
+*  @param  buffer_loc            address at which to return newly allocated #Buffer in which
+*                                result is returned
 *
 *  @retval  NULL    success
 *  @retval  #Ddc_Error containing status DDCRC_UNSUPPORTED does not support Capabilities Request
@@ -275,13 +177,15 @@ multi_part_read_with_retry(
       Byte             request_type,
       Byte             request_subtype,   // VCP feature code for table read, ignore for capabilities
       bool             all_zero_response_ok,
-      Buffer**         pp_buffer)
+      Buffer**         buffer_loc)
 {
    bool debug = false;
+   Retry_Op_Value max_multi_part_read_tries = try_data_get_maxtries2(MULTI_PART_READ_OP);
    DBGTRC(debug, TRACE_GROUP,
           "Starting.  request_type=0x%02x, request_subtype=0x%02x, all_zero_response_ok=%s"
           ", max_multi_part_read_tries=%d",
-          request_type, request_subtype, bool_repr(all_zero_response_ok), max_multi_part_read_tries);
+          request_type, request_subtype, sbool(all_zero_response_ok), max_multi_part_read_tries);
+
 
    Public_Status_Code rc = -1;   // dummy value for first call of while loop
    Error_Info * ddc_excp = NULL;
@@ -330,6 +234,11 @@ multi_part_read_with_retry(
          // rc = DDCRC_DETERMINED_UNSUPPORTED;    // ??
          // COUNT_STATUS_CODE(rc);   // double counting?
       }
+      else if (rc == -EBADF) {
+         // DBGMSG("EBADF");
+         can_retry = false;
+      }
+
       tryctr++;
    }
    assert( (rc<0 && ddc_excp) || (rc==0 && !ddc_excp) );
@@ -361,9 +270,9 @@ multi_part_read_with_retry(
    }
 
    // if counts for DDCRC_ALL_TRIES_ZERO?
-   try_data_record_tries(multi_part_read_stats_rec, rc, tryctr);
+   try_data_record_tries2(MULTI_PART_READ_OP, rc, tryctr);
 
-   *pp_buffer = accumulator;
+   *buffer_loc = accumulator;
    DBGTRC(debug, TRACE_GROUP, "Returning: %s", errinfo_summary(ddc_excp));
    return ddc_excp;
 }
@@ -442,6 +351,7 @@ multi_part_write_with_retry(
      Byte             vcp_code,
      Buffer *         value_to_set)
 {
+   Retry_Op_Value max_multi_part_write_tries = try_data_get_maxtries2(MULTI_PART_WRITE_OP);
    bool debug = false;
    if (IS_TRACING())
       puts("");
@@ -495,7 +405,7 @@ multi_part_write_with_retry(
 }
 
 
-static void init_ddc_multi_part_io_func_name_table() {
+static inline void init_ddc_multi_part_io_func_name_table() {
 #define ADD_FUNC(_NAME) rtti_func_name_table_add(_NAME, #_NAME);
    ADD_FUNC(try_multi_part_read);
    ADD_FUNC(multi_part_read_with_retry);
