@@ -1,7 +1,7 @@
 /** \file ddc_output.c
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -14,6 +14,7 @@
 #include <string.h>
 #include <time.h>
 
+#include "../i2c/i2c_strategy_dispatcher.h"
 #include "util/error_info.h"
 #include "util/report_util.h"
 /** \endcond */
@@ -23,11 +24,10 @@
 #include "base/ddc_packets.h"
 #include "base/linux_errno.h"
 #include "base/parms.h"
+#include "base/rtti.h"
 #include "base/sleep.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "i2c/i2c_do_io.h"
-
 #include "adl/adl_shim.h"
 
 #ifdef USE_USB
@@ -150,21 +150,22 @@ check_valid_operation_by_feature_id_and_dh(
  * Returns:
  *    status code
  */
-static Public_Status_Code
+static Error_Info *
 get_raw_value_for_feature_metadata(
       Display_Handle *           dh,
-      Display_Feature_Metadata *    frec,
+      Display_Feature_Metadata * frec,
       bool                       ignore_unsupported,
-      DDCA_Any_Vcp_Value ** pvalrec,
+      DDCA_Any_Vcp_Value **      pvalrec,
       FILE *                     msg_fh)
 {
-   bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting");
-
+   assert(frec);
    assert(dh);
    assert(dh->dref);
 
-   Public_Status_Code psc = 0;
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "Starting. frec=%p");
+
+   // Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
 
    // DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
@@ -179,6 +180,7 @@ get_raw_value_for_feature_metadata(
    DDCA_Any_Vcp_Value * valrec = NULL;
    if (dh->dref->io_path.io_mode == DDCA_IO_USB) {
 #ifdef USE_USB
+     Public_Status_Code
      psc = usb_get_vcp_value(
               dh,
               feature_code,
@@ -196,13 +198,14 @@ get_raw_value_for_feature_metadata(
               feature_code,
               feature_type,
               &valrec);
-      psc = ERRINFO_STATUS(ddc_excp);
+      // psc = ERRINFO_STATUS(ddc_excp);
    }
-   assert ( (psc==0 && valrec) || (psc!=0 && !valrec) );
+   ASSERT_IFF( ddc_excp, !valrec);
+   // assert ( (psc==0 && valrec) || (psc!=0 && !valrec) );
 
    // For now, only regard -EIO as unsupported feature for the
    // single model on which this has been observed
-   if (psc ==  -EIO &&
+   if (ERRINFO_STATUS(ddc_excp) ==  -EIO &&
        dh->dref->io_path.io_mode == DDCA_IO_I2C &&
        streq(dh->dref->pedid->mfg_id, "DEL")    &&
        streq(dh->dref->pedid->model_name, "AW3418DW") )
@@ -213,12 +216,15 @@ get_raw_value_for_feature_metadata(
          f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
                         feature_code, feature_name, "Unsupported feature code (EIO)");
       }
-      psc = DDCRC_DETERMINED_UNSUPPORTED;
+      // psc = DDCRC_DETERMINED_UNSUPPORTED;
       COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
+      ddc_excp = errinfo_new_with_cause2(
+                   DDCRC_DETERMINED_UNSUPPORTED, ddc_excp, __func__, "EIO");
    }
 
    else {
-      switch(psc) {
+      Public_Status_Code psc = ERRINFO_STATUS(ddc_excp);
+      switch( psc ) {
       case 0:
          break;
 
@@ -236,7 +242,9 @@ get_raw_value_for_feature_metadata(
                            feature_code, feature_name, "Unsupported feature code (Null response)");
          }
          COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
-         psc = DDCRC_DETERMINED_UNSUPPORTED;
+         // psc = DDCRC_DETERMINED_UNSUPPORTED;
+         ddc_excp = errinfo_new_with_cause2(
+                     DDCRC_DETERMINED_UNSUPPORTED, ddc_excp, __func__, "DDC NULL Response");
          break;
 
       case DDCRC_READ_ALL_ZERO:
@@ -245,8 +253,10 @@ get_raw_value_for_feature_metadata(
             f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
                            feature_code, feature_name, "Unsupported feature code (All zero response)");
          }
-         psc = DDCRC_DETERMINED_UNSUPPORTED;
+         // psc = DDCRC_DETERMINED_UNSUPPORTED;
          COUNT_STATUS_CODE(DDCRC_DETERMINED_UNSUPPORTED);
+         ddc_excp = errinfo_new_with_cause2(
+                     DDCRC_DETERMINED_UNSUPPORTED, ddc_excp, __func__, "MH=ML=SH=SL=0");
          break;
 
 #ifdef FUTURE
@@ -268,36 +278,47 @@ get_raw_value_for_feature_metadata(
          break;
 
       case DDCRC_REPORTED_UNSUPPORTED:
-      case DDCRC_DETERMINED_UNSUPPORTED:
          if (!ignore_unsupported) {
             f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
                             feature_code, feature_name, "Unsupported feature code");
          }
          break;
 
-      default:
-      {
-         char buf[200];
-         snprintf(buf, 200, "Invalid response. status code=%s, %s", psc_desc(psc), dh_repr_t(dh));
-         f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
-                          feature_code, feature_name, buf);
-      }
-      }
+      case DDCRC_DETERMINED_UNSUPPORTED:
+         if (!ignore_unsupported) {
+            char text[100];
+            g_snprintf(text, 100, "Unsupported feature code (%s)", ddc_excp->detail);
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                            feature_code, feature_name, text);
+         }
+         break;
 
+      default:
+         {
+            char buf[200];
+            snprintf(buf, 200, "Invalid response. status code=%d, %s", psc, dh_repr_t(dh));
+            f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
+                             feature_code, feature_name, buf);
+         }
+      }
    }
 
    *pvalrec = valrec;
-   DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s, *pvalrec=%p", psc_desc(psc), *pvalrec);
-   assert( (psc == 0 && *pvalrec) || (psc != 0 && !*pvalrec) );
-   if (*pvalrec && (debug || IS_TRACING())) {
-      dbgrpt_single_vcp_value(*pvalrec, 1);
+   ASSERT_IFF(!ddc_excp, *pvalrec);;
+
+   if (debug || IS_TRACING()) {
+      if (ddc_excp) {
+         DBGMSG("Done.     Returning %s", errinfo_summary(ddc_excp));
+      }
+      else {
+         DBGMSG("Done.     Returning NULL, *pvalrec -> ");
+         dbgrpt_single_vcp_value(*pvalrec, 3);
+      }
    }
-   if (ddc_excp) {
-      ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
-   }
-   return psc;
+   return ddc_excp;
 }
 
+
 #ifdef IN_PROGREESS
 Public_Status_Code
 get_raw_value_for_feature_metadata_dfm(
@@ -459,7 +480,7 @@ collect_raw_feature_set_values2_dfm(
       DDCA_Any_Vcp_Value *  pvalrec;
       // DDCA_Feature_Metadata * ddca_meta = dfm_to_ddca_feature_metadata(dfm);
 
-      Public_Status_Code cur_status_code =
+      Error_Info *  cur_ddc_excp =
             get_raw_value_for_feature_metadata(
                   dh,
                   dfm,    // ddca_meta,
@@ -467,8 +488,9 @@ collect_raw_feature_set_values2_dfm(
                   &pvalrec,
                    msg_fh);
       // todo: free ddca_meta
+      Public_Status_Code cur_status_code = ERRINFO_STATUS(cur_ddc_excp);
 
-      if (cur_status_code == 0) {
+      if (!cur_ddc_excp) {   // changed from (cur_status_code == 0) to avoid coverity complaint re resource leak
          vcp_value_set_add(vset, pvalrec);
       }
       else if ( (cur_status_code == DDCRC_REPORTED_UNSUPPORTED ||
@@ -477,8 +499,10 @@ collect_raw_feature_set_values2_dfm(
               )
       {
          // no problem
+         ERRINFO_FREE_WITH_REPORT(cur_ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
       }
       else {
+         ERRINFO_FREE_WITH_REPORT(cur_ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
          master_status_code = cur_status_code;
          break;
       }
@@ -578,6 +602,7 @@ ddc_get_formatted_value_for_display_feature_metadata(
    DBGTRC(debug, TRACE_GROUP, "Starting. suppress_unsupported=%s", sbool(suppress_unsupported));
 
    Public_Status_Code psc = 0;
+   Error_Info * ddc_excp;
    *formatted_value_loc = NULL;
 
    DDCA_MCCS_Version_Spec vspec = get_vcp_version_by_display_handle(dh);
@@ -599,15 +624,16 @@ ddc_get_formatted_value_for_display_feature_metadata(
    // bool ignore_unsupported = !(output_level >= DDCA_OL_NORMAL && !suppress_unsupported);
    bool ignore_unsupported = suppress_unsupported;
 
-   psc = get_raw_value_for_feature_metadata(
+   ddc_excp = get_raw_value_for_feature_metadata(
             dh,
             dfm,    // extmeta,
             ignore_unsupported,
             &pvalrec,
             (output_level == DDCA_OL_TERSE) ? NULL : msg_fh);
             // msg_fh);
+   psc = ERRINFO_STATUS(ddc_excp);
    assert( (psc==0 && (feature_type == pvalrec->value_type)) || (psc!=0 && !pvalrec) );
-   if (psc == 0) {
+   if (!ddc_excp) {      // changed from (psc == 0) to avoid avoid coverity complaint re resource leak
       // if (!is_table_feature && output_level >= OL_VERBOSE) {
       // if (!is_table_feature && debug) {
       if (output_level >= DDCA_OL_VERBOSE || debug) {
@@ -659,7 +685,7 @@ ddc_get_formatted_value_for_display_feature_metadata(
          }
       }
 
-      else  {
+      else  {    // output_level >= DDCA_OL_NORMAL
          bool ok;
          char * formatted_data = NULL;
 
@@ -673,6 +699,7 @@ ddc_get_formatted_value_for_display_feature_metadata(
             f0printf(msg_fh, FMT_CODE_NAME_DETAIL_W_NL,
                             feature_code, feature_name, "!!! UNABLE TO FORMAT OUTPUT");
             psc = DDCRC_INTERPRETATION_FAILED;
+            ddc_excp = errinfo_new(DDCRC_INTERPRETATION_FAILED, __func__);
             // TODO: retry with default output function
          }
 
@@ -702,8 +729,11 @@ ddc_get_formatted_value_for_display_feature_metadata(
       free_single_vcp_value(pvalrec);
 
    DBGTRC(debug, TRACE_GROUP,
-          "Done.  Returning: %s, *formatted_value_loc=%p",
+          "Done.      Returning: %s, *formatted_value_loc=%p",
           psc_desc(psc), formatted_value_loc);
+
+   ASSERT_IFF(psc == 0, !ddc_excp);
+   ERRINFO_FREE_WITH_REPORT(ddc_excp, debug || IS_TRACING() || report_freed_exceptions);
    return psc;
 }
 
@@ -884,3 +914,15 @@ ddc_show_vcp_values(
    DBGTRC(debug, TRACE_GROUP, "Done. Returning %s", psc_desc(psc));
    return psc;
 }
+
+static void init_ddc_output_func_name_table() {
+#define ADD_FUNC(_NAME) rtti_func_name_table_add(_NAME, #_NAME);
+   ADD_FUNC(get_raw_value_for_feature_metadata);
+   ADD_FUNC(ddc_get_formatted_value_for_display_feature_metadata);
+#undef ADD_FUNC
+}
+
+void init_ddc_output() {
+   init_ddc_output_func_name_table();
+}
+
