@@ -4,7 +4,7 @@
  *  or the ADL API, as appropriate.  Handles I2C bus retry.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 // N. ddc_open_display() and ddc_close_display() handle case USB, but the
@@ -16,6 +16,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <i2c/i2c_strategy_dispatcher.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,6 +24,8 @@
 // #include <sys/ioctl.h>
 #include <unistd.h>
 
+#include "public/ddcutil_types.h"
+
 #include "util/debug_util.h"
 #include "util/string_util.h"
 #include "util/utilrpt.h"
@@ -30,15 +33,15 @@
 
 #include "base/ddc_errno.h"
 #include "base/displays.h"
+#include "base/dynamic_sleep.h"
 #include "base/execution_stats.h"
 #include "base/parms.h"
 #include "base/rtti.h"
 #include "base/status_code_mgt.h"
 #include "base/tuned_sleep.h"
+#include "base/thread_sleep_data.h"
 
 #include "i2c/i2c_bus_core.h"
-#include "i2c/i2c_do_io.h"
-
 #include "adl/adl_shim.h"
 
 #ifdef USE_USB
@@ -206,7 +209,7 @@ ddc_open_display(
 
    if (ddcrc == 0) {
       if (dref->io_path.io_mode != DDCA_IO_USB)
-         TUNED_SLEEP_WITH_TRACE(dref->io_path.io_mode, SE_POST_OPEN, NULL);
+         TUNED_SLEEP_WITH_TRACE(dh, SE_POST_OPEN, NULL);
       dref->flags |= DREF_OPEN;
    }
    else {
@@ -237,118 +240,56 @@ bye:
 Status_Errno
 ddc_close_display(Display_Handle * dh) {
    bool debug = false;
-   if (debug) {
-      DBGMSG0("Starting.");
-      dbgrpt_display_handle(dh, __func__, 1);
-   }
+   DBGMSF(debug, "Starting. dh=%s, dref=%s, fd=%d, dpath=%s",
+              dh_repr_t(dh), dref_repr_t(dh->dref), dh->fd, dpath_short_name_t(&dh->dref->io_path) ) ;
    Status_Errno rc = 0;
-
-   switch(dh->dref->io_path.io_mode) {
-   case DDCA_IO_I2C:
-      {
-         rc = i2c_close_bus(dh->fd, dh->dref->io_path.path.i2c_busno,  CALLOPT_NONE);    // return error if failure
-         if (rc != 0) {
-            assert(rc < 0);
-            DBGMSG("i2c_close_bus returned %d", rc);
-            COUNT_STATUS_CODE(rc);
+   if (dh->fd == -1) {
+      rc = DDCRC_INVALID_OPERATION;    // or DDCRC_ARG?
+   }
+   else {
+      switch(dh->dref->io_path.io_mode) {
+      case DDCA_IO_I2C:
+         {
+            rc = i2c_close_bus(dh->fd, dh->dref->io_path.path.i2c_busno,  CALLOPT_NONE);    // return error if failure
+            if (rc != 0) {
+               assert(rc < 0);
+               DBGMSG("i2c_close_bus returned %d, errno=%s", rc, psc_desc(errno) );
+               COUNT_STATUS_CODE(rc);
+            }
+            dh->fd = -1;    // indicate invalid, in case we try to continue using dh
+            break;
          }
-         dh->fd = -1;    // indicate invalid, in case we try to continue using dh
-         break;
-      }
-   case DDCA_IO_ADL:
-      break;           // nothing to do
+      case DDCA_IO_ADL:
+         break;           // nothing to do
 
-   case DDCA_IO_USB:
+      case DDCA_IO_USB:
 #ifdef USE_USB
-      {
-         rc = usb_close_device(dh->fd, dh->dref->usb_hiddev_name, CALLOPT_NONE); // return error if failure
-         if (rc != 0) {
-            assert(rc < 0);
-            DBGMSG("usb_close_device returned %d", rc);
-            COUNT_STATUS_CODE(rc);
+         {
+            rc = usb_close_device(dh->fd, dh->dref->usb_hiddev_name, CALLOPT_NONE); // return error if failure
+            if (rc != 0) {
+               assert(rc < 0);
+               DBGMSG("usb_close_device returned %d", rc);
+               COUNT_STATUS_CODE(rc);
+            }
+            dh->fd = -1;
+            break;
          }
-         dh->fd = -1;
-         break;
-      }
 #else
-      PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
+         PROGRAM_LOGIC_ERROR("ddcutil not built with USB support");
 #endif
-   } //switch
+      } //switch
+   }
 
    dh->dref->flags &= (~DREF_OPEN);
    Distinct_Display_Ref display_id = get_distinct_display_ref(dh->dref);
    unlock_distinct_display(display_id);
 
    free_display_handle(dh);
+   DBGMSF(debug, "Done.     Returning: %s", psc_desc(rc));
    return rc;
 }
 
 
-//
-// Retry Management and Statistics
-//
-
-// constants in parms.h:
-static int max_write_only_exchange_tries =  MAX_WRITE_ONLY_EXCHANGE_TRIES;
-static int max_write_read_exchange_tries =  MAX_WRITE_READ_EXCHANGE_TRIES;
-
-
-static void * write_read_stats_rec = NULL;
-static void * write_only_stats_rec = NULL;
-
-
-void ddc_reset_write_read_stats() {
-   if (write_read_stats_rec)
-      try_data_reset(write_read_stats_rec);
-   else
-      write_read_stats_rec = try_data_create("ddc write/read", max_write_read_exchange_tries);
-}
-
-
-void ddc_report_write_read_stats(int depth) {
-   assert(write_read_stats_rec);
-   try_data_report(write_read_stats_rec, depth);
-}
-
-
-void ddc_reset_write_only_stats() {
-   if (write_only_stats_rec)
-      try_data_reset(write_only_stats_rec);
-   else
-      write_only_stats_rec = try_data_create("ddc write only", max_write_only_exchange_tries);
-}
-
-
-void ddc_report_write_only_stats(int depth) {
-   assert(write_only_stats_rec);
-   try_data_report(write_only_stats_rec, depth);
-}
-
-
-void ddc_set_max_write_only_exchange_tries(int ct) {
-   assert(ct > 0 && ct <= MAX_MAX_TRIES);
-   max_write_only_exchange_tries = ct;
-   if (write_only_stats_rec)
-      try_data_set_max_tries(write_only_stats_rec, ct);
-}
-
-
-int ddc_get_max_write_only_exchange_tries() {
-   return max_write_only_exchange_tries;
-}
-
-
-void ddc_set_max_write_read_exchange_tries(int ct) {
-   assert(ct > 0 && ct <= MAX_MAX_TRIES);
-   max_write_read_exchange_tries = ct;
-   if (write_read_stats_rec)
-      try_data_set_max_tries(write_read_stats_rec, ct);
-}
-
-int ddc_get_max_write_read_exchange_tries() {
-   return max_write_read_exchange_tries;
-}
-
 
 
 // work in progress
@@ -386,9 +327,11 @@ DDCA_Status (*Write_Read_Raw_Function)(
  *   -errno if error in write
  *   DDCRC_READ_ALL_ZERO
  */
-static DDCA_Status ddc_i2c_write_read_raw(
+// static  // allow function to appear in backtrace
+DDCA_Status ddc_i2c_write_read_raw(
          Display_Handle * dh,
          DDC_Packet *     request_packet_ptr,
+         bool             read_bytewise,
          int              max_read_bytes,
          Byte *           readbuf,
          int *            pbytes_received
@@ -408,14 +351,19 @@ static DDCA_Status ddc_i2c_write_read_raw(
    bool single_byte_reads = false;   // doesn't work
 #endif
 
+   Byte slave_addr = request_packet_ptr->raw_bytes->bytes[0];      // 0x6e
+   assert(slave_addr >> 1 == 0x37);
+
+   CHECK_DEFERRED_SLEEP(dh);
    Status_Errno_DDC rc =
          invoke_i2c_writer(
                            dh->fd,
+                           0x37,
                            get_packet_len(request_packet_ptr)-1,
                            get_packet_start(request_packet_ptr)+1 );
    DBGMSF(debug, "invoke_i2c_writer() returned %d\n", rc);
    if (rc == 0) {
-      TUNED_SLEEP_I2C_WITH_TRACE(SE_WRITE_TO_READ, NULL);
+      TUNED_SLEEP_WITH_TRACE(dh, SE_WRITE_TO_READ, NULL);
       // tuned_sleep_i2c_with_trace(SE_WRITE_TO_READ, __func__, NULL);
 
       // ALTERNATIVE_THAT_DIDNT_WORK:
@@ -423,10 +371,11 @@ static DDCA_Status ddc_i2c_write_read_raw(
       //    rc = invoke_single_byte_i2c_reader(dh->fd, max_read_bytes, readbuf);
       // else
 
-      rc = invoke_i2c_reader(dh->fd, max_read_bytes, readbuf);
+      CHECK_DEFERRED_SLEEP(dh);
+      rc = invoke_i2c_reader(dh->fd, 0x37, read_bytewise, max_read_bytes, readbuf);
       // try adding sleep to see if improves capabilities read for P2411H
       // tuned_sleep_i2c_with_trace(SE_POST_READ, __func__, NULL);
-      TUNED_SLEEP_I2C_WITH_TRACE(SE_POST_READ, NULL);
+      TUNED_SLEEP_WITH_TRACE(dh, SE_POST_READ, NULL);
 
       if (rc == 0 && all_bytes_zero(readbuf, max_read_bytes)) {
          DDCMSG(debug, "All zero response detected in %s", __func__);
@@ -489,7 +438,7 @@ static DDCA_Status ddc_adl_write_read_raw(
    }
    else {
       // tuned_sleep(DDCA_IO_ADL, SE_WRITE_TO_READ);
-      TUNED_SLEEP_WITH_TRACE(DDCA_IO_ADL, SE_WRITE_TO_READ, NULL);
+      TUNED_SLEEP_WITH_TRACE(dh, SE_WRITE_TO_READ, NULL);
       psc = adlshim_ddc_read_only(
             dh,
             readbuf,
@@ -522,9 +471,11 @@ static DDCA_Status ddc_adl_write_read_raw(
 }
 
 
-static DDCA_Status ddc_write_read_raw(
+// static  // allow function to appear in backtrace
+DDCA_Status ddc_write_read_raw(
       Display_Handle * dh,
       DDC_Packet *     request_packet_ptr,
+      bool             read_bytewise,
       int              max_read_bytes,
       Byte *           readbuf,
       int *            p_rcvd_bytes_ct
@@ -534,9 +485,13 @@ static DDCA_Status ddc_write_read_raw(
    DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, readbuf=%p, max_read_bytes=%d",
                               dh_repr_t(dh), readbuf, max_read_bytes);
    if (debug) {
-      DBGMSG0("request_packer_ptr->raw_bytes:");
-      dbgrpt_buffer(request_packet_ptr->raw_bytes, 1);
+      // DBGMSG0("request_packet_ptr->raw_bytes:");
+      // dbgrpt_buffer(request_packet_ptr->raw_bytes, 1);
+      char * s =  hexstring3_t(request_packet_ptr->raw_bytes->bytes,
+                              request_packet_ptr->raw_bytes->len, " ", 1, false );
+      DBGMSG("request_packet_ptr->raw_bytes: %s", s);
    }
+
    DDCA_Status psc;
 
    // This function should not be called for USB
@@ -546,6 +501,7 @@ static DDCA_Status ddc_write_read_raw(
         psc =  ddc_i2c_write_read_raw(
               dh,
               request_packet_ptr,
+              read_bytewise,
               max_read_bytes,
               readbuf,
               p_rcvd_bytes_ct
@@ -589,6 +545,7 @@ Error_Info *
 ddc_write_read(
       Display_Handle * dh,
       DDC_Packet *     request_packet_ptr,
+      bool             read_bytewise,
       int              max_read_bytes,
       Byte             expected_response_type,
       Byte             expected_subtype,
@@ -606,6 +563,7 @@ ddc_write_read(
    psc =  ddc_write_read_raw(
             dh,
             request_packet_ptr,
+            read_bytewise,
             max_read_bytes,
             readbuf,
             &bytes_received
@@ -629,24 +587,28 @@ ddc_write_read(
           *response_packet_ptr_loc = NULL;
        }
    }
+   dsa_record_ddcrw_status_code(psc);
 
    free(readbuf);    // or does response_packet_ptr_loc point into here?
 
    // already done:
    // if (rc != 0)
-   //    COUNT_STATUS_CODE(rc);
-
-   // If a DDC status code, has already been counted when set.  what about RR_ADL?
-   // if (psc < 0  && get_modulation(psc) != RR_DDC)
    //    COUNT_STATUS_CODE(psc);
 
+   // Convert status code to Error_Info *
    Error_Info * excp = NULL;
    if (psc < 0)
       excp = errinfo_new(psc, __func__);
 
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(excp)  );
-   if (psc == 0 && (IS_TRACING() || debug) )
-      dbgrpt_packet(*response_packet_ptr_loc, 1);
+   if (debug || IS_TRACING()) {
+      if (excp) {
+         DBGMSG("Done.     Returning: %s", errinfo_summary(excp)  );
+      }
+      else {
+         DBGMSG("Done.     Returning: NULL, *response_packet_ptr_loc ->");
+         dbgrpt_packet(*response_packet_ptr_loc, 3);
+      }
+   }
 
    return excp;
 }
@@ -687,47 +649,59 @@ ddc_write_read_with_retry(
 {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, all_zero_response_ok=%s",
-          dh_repr_t(dh), bool_repr(all_zero_response_ok)  );
+          dh_repr_t(dh), sbool(all_zero_response_ok)  );
    assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
    // show_backtrace(1);
 
-   if (debug)
-      dbgrpt_display_ref(dh->dref, 1);
+   // if (debug)
+   //     dbgrpt_display_ref(dh->dref, 1);
 
    bool retry_null_response = !(dh->dref->flags & DREF_DDC_USES_NULL_RESPONSE_FOR_UNSUPPORTED);
 
    DDCA_Status  psc;
+   bool read_bytewise = I2C_Read_Bytewise;   // normally set to DEFAULT_I2C_READ_BYTEWISE
    int  tryctr;
    bool retryable;
    int  ddcrc_read_all_zero_ct = 0;
    int  ddcrc_null_response_ct = 0;
    int  ddcrc_null_response_max = (retry_null_response) ? 3 : 0;
+   bool sleep_multiplier_incremented = false;
    // ddcrc_null_response_max = 6;  // *** TEMP *** for testing
    DBGMSF(debug, "retry_null_response = %s, ddcrc_null_response_max = %d",
           sbool(retry_null_response), ddcrc_null_response_max);
    Error_Info * try_errors[MAX_MAX_TRIES];
 
-   assert(max_write_read_exchange_tries > 0);   // to avoid clang warning
+   // assert(max_write_read_exchange_tries > 0);   // to avoid clang warning
+   int max_tries = try_data_get_maxtries2(WRITE_READ_TRIES_OP);
+   assert(max_tries >= 0);
    for (tryctr=0, psc=-999, retryable=true;
-        tryctr < max_write_read_exchange_tries && psc < 0 && retryable;
+        tryctr < max_tries && psc < 0 && retryable;
         tryctr++)
    {
       DBGMSF(debug,
-           "Start of try loop, tryctr=%d, max_write_read_echange_tries=%d, rc=%d, retryable=%d",
-           tryctr, max_write_read_exchange_tries, psc, retryable );
+           "Start of try loop, tryctr=%d, max_tries=%d, rc=%d, retryable=%s, read_bytewise=%s",
+           tryctr, max_tries, psc, sbool(retryable), sbool(read_bytewise) );
 
       Error_Info * cur_excp = ddc_write_read(
                 dh,
                 request_packet_ptr,
+                read_bytewise,
                 max_read_bytes,
                 expected_response_type,
                 expected_subtype,
                 response_packet_ptr_loc);
+
+      // TESTCASES:
+      // if (tryctr < 2)
+      // cur_excp = errinfo_new(DDCRC_NULL_RESPONSE, "dummy");
+      // cur_excp = errinfo_new(-EIO, "dummy");
+
       psc = (cur_excp) ? cur_excp->status_code : 0;
       try_errors[tryctr] = cur_excp;
 
       if (psc == 0 && ddcrc_null_response_ct > 0) {
-         DBGMSG("%s, ddc_write_read() succeeded after %d sleep and retry for DDC Null Response",
+         DBGTRC(debug, TRACE_GROUP | DDCA_TRC_RETRY,
+                "%s, ddc_write_read() succeeded after %d sleep and retry for DDC Null Response",
                 dh_repr_t(dh),
                 ddcrc_null_response_ct);
       }
@@ -741,40 +715,49 @@ ddc_write_read_with_retry(
             // is the monitor using NULL response to indicate unsupported?
             // Acer monitor uses NULL response instead of setting the unsupported
             // flag in a valid response
+            switch (psc) {
+            case DDCRC_NULL_RESPONSE:
+                  {
+                     retryable = (++ddcrc_null_response_ct < ddcrc_null_response_max);
+                     DBGMSF(debug, "DDCRC_NULL_RESPONSE, retryable=%s", sbool(retryable));
+                     if (retryable) {
+                        if (ddcrc_null_response_ct == 1 && get_output_level() >= DDCA_OL_VERBOSE)
+                           f0printf(fout(), "Extended delay as recovery from DDC Null Response...\n");
+                        tsd_set_sleep_multiplier_ct(ddcrc_null_response_ct+1);
+                        sleep_multiplier_incremented = true;
+                        // replaces: call_dynamic_tuned_sleep_i2c(SE_DDC_NULL, ddcrc_null_response_ct);
+                     }
+                  }
+                  break;
 
-            if (psc == DDCRC_NULL_RESPONSE) {
-               retryable = (ddcrc_null_response_ct++ < ddcrc_null_response_max);
-               DBGMSF(debug, "DDCRC_NULL_RESPONSE, retryable=%s", bool_repr(retryable));
-               if (retryable) {
-                  if (ddcrc_null_response_ct == 1 && get_output_level() >= DDCA_OL_VERBOSE)
-                     f0printf(fout(), "Extended delay as recovery from DDC Null Response...\n");
-                  set_sleep_multiplier_ct(ddcrc_null_response_ct+1);
-                  // replaces: call_dynamic_tuned_sleep_i2c(SE_DDC_NULL, ddcrc_null_response_ct);
-               }
-            }
-            // when is DDCRC_READ_ALL_ZERO actually an error vs the response of the monitor instead of NULL response?
-            // On Dell monitors (P2411, U3011) all zero response occurs on unsupported Table features
-            // But also seen as a bad response
-            else if ( psc == DDCRC_READ_ALL_ZERO)
-               retryable = (all_zero_response_ok) ? false : true;
+            case (DDCRC_READ_ALL_ZERO):
+                 // when is DDCRC_READ_ALL_ZERO actually an error vs the response of the monitor instead of NULL response?
+                 // On Dell monitors (P2411, U3011) all zero response occurs on unsupported Table features
+                 // But also seen as a bad response
+                 retryable = (all_zero_response_ok) ? false : true;
+                 break;
 
-            else if (psc == -EIO)
-                retryable = false;     // ??
+            case (-EIO):
+                 // retryable = false;     // ??
+                 break;
 
-            else if (psc == -EBADF)
-               retryable = false;
+            case (-EBADF):
+                 // DBGMSG("EBADF");
+                 retryable = false;
+                 break;
 
-            else if (psc == -ENXIO)    // no such device or address, i915 driver
-               retryable = false;
-
-            else
-               retryable = true;     // for now
+            case (-ENXIO):    // no such device or address, i915 driver
+                 retryable = false;  // have seen success after 7 retries of errors including ENXIO, DDCRC_DATA, make retryable?
+                 break;
 
+            default:
+                 retryable = true;     // for now
 
             // try exponential backoff on all errors, not just SE_DDC_NULL
             // if (retryable)
             //    call_dynamic_tuned_sleep_i2c(SE_DDC_NULL, tryctr+1);
-         }
+                  }
+         }  // DDCA_IO_I2C
 
          else {   // DDC_IO_ADL
             // TODO more detailed tests
@@ -789,26 +772,46 @@ ddc_write_read_with_retry(
          if (psc == DDCRC_READ_ALL_ZERO)
             ddcrc_read_all_zero_ct++;
       }    // rc < 0
+      // DBGMSG("Bottom of try loop. psc=%d, tryctr=%d, retryable=%s", psc, tryctr, sbool(retryable));
    }
-   DBGTRC(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, psc=%d, retryable=%s",
-         tryctr, psc, bool_repr(retryable));
+   DBGTRC(debug, DDCA_TRC_NONE, "After try loop. tryctr=%d, psc=%d, retryable=%s, read_bytewise=%s",
+         tryctr, psc, sbool(retryable), sbool(read_bytewise));
+
+   int errct = (psc == 0) ? tryctr-1 : tryctr;
+   // DBGMSG("psc=%d, tryctr=%d, errct=%d", psc, tryctr, errct);
+
+   // read_bytewise = !read_bytewise;
+#ifdef OLD
    if (debug) {
       for (int ndx = 0; ndx < tryctr; ndx++) {
+         DBGMSG("try_errors[ndx] = %p", try_errors[ndx]);
          DBGMSG("try_errors[%d] = %s", ndx, errinfo_summary(try_errors[ndx]));
       }
    }
+#endif
+   // DBGMSG("try_errors = %p, &try_errors=%p", try_errors, &try_errors);
+   if (errct > 0) {
+         char * s0 = (psc == 0) ? "Succeeded" : "Failed";
+         char * s1 = (errct == 1) ? "" : "s";
+         char * s = errinfo_array_summary(try_errors, errct);
+         DBGTRC(debug, TRACE_GROUP | DDCA_TRC_RETRY, "%s after %d error%s: %s", s0, errct, s1, s);
+         free(s);
 
-   set_sleep_multiplier_ct(1);   // in case we changed it
+   }
+   if (sleep_multiplier_incremented) {
+      tsd_set_sleep_multiplier_ct(1);   // in case we changed it
+      tsd_bump_sleep_multiplier_changer_ct();
+   }
 
    Error_Info * ddc_excp = NULL;
 
    if (psc < 0) {
       // int last_try_index = tryctr-1;
-      DBGTRC(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, bool_repr(retryable));
+      DBGTRC(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, sbool(retryable));
 
       if (retryable)
          psc = DDCRC_RETRIES;
-      else if (ddcrc_read_all_zero_ct == max_write_read_exchange_tries)
+      else if (ddcrc_read_all_zero_ct == max_tries)
          psc = DDCRC_ALL_TRIES_ZERO;
       else if (ddcrc_null_response_ct > ddcrc_null_response_max)
          psc = DDCRC_ALL_RESPONSES_NULL;
@@ -825,7 +828,7 @@ ddc_write_read_with_retry(
       }
    }
 
-   try_data_record_tries(write_read_stats_rec, psc, tryctr);
+   try_data_record_tries2(WRITE_READ_TRIES_OP, psc, tryctr);
 
    DBGTRC(debug, TRACE_GROUP, "Done.  Total Tries (tryctr): %d. Returning: %s", tryctr, errinfo_summary(ddc_excp));
    return ddc_excp;
@@ -844,17 +847,24 @@ ddc_write_read_with_retry(
  */
 static Status_Errno_DDC
 ddc_i2c_write_only(
-         int           fh,
+         Display_Handle * dh,
          DDC_Packet *  request_packet_ptr
         )
 {
    bool debug = false;
+   int fh = dh->fd;
    DBGTRC0(debug, TRACE_GROUP, "Starting.");
    if (debug)
       dbgrpt_packet(request_packet_ptr, 1);
 
+   // Byte slave_address = request_packet_ptr[0];
+   // assert(slave_address == 0x37);
+   Byte slave_address = 0x37;
+
+   CHECK_DEFERRED_SLEEP(dh);
    Status_Errno_DDC rc =
          invoke_i2c_writer(fh,
+                           slave_address,
                            get_packet_len(request_packet_ptr)-1,
                            get_packet_start(request_packet_ptr)+1 );
    if (rc < 0)
@@ -864,7 +874,7 @@ ddc_i2c_write_only(
             ? SE_POST_SAVE_SETTINGS
             : SE_POST_WRITE;
    // tuned_sleep_i2c_with_trace(sleep_type, __func__, NULL);
-   TUNED_SLEEP_I2C_WITH_TRACE(sleep_type, NULL);
+   TUNED_SLEEP_WITH_TRACE(dh, sleep_type, NULL);
    DBGTRC(debug, TRACE_GROUP, "Done. rc=%s", psc_desc(rc) );
    return rc;
 }
@@ -887,7 +897,7 @@ ddc_write_only(
    DDCA_Status psc = 0;
    assert(dh->dref->io_path.io_mode != DDCA_IO_USB);
    if (dh->dref->io_path.io_mode == DDCA_IO_I2C) {
-      psc = ddc_i2c_write_only(dh->fd, request_packet_ptr);
+      psc = ddc_i2c_write_only(dh, request_packet_ptr);
    }
    else {
       psc = adlshim_ddc_write_only(
@@ -936,14 +946,15 @@ ddc_write_only_with_retry(
    bool               retryable;
    Error_Info *       try_errors[MAX_MAX_TRIES];
 
-   assert(max_write_only_exchange_tries > 0);
+   int max_tries = try_data_get_maxtries2(WRITE_ONLY_TRIES_OP);
+   assert(max_tries > 0);
    for (tryctr=0, psc=-999, retryable=true;
-       tryctr < max_write_only_exchange_tries && psc < 0 && retryable;
+       tryctr < max_tries && psc < 0 && retryable;
        tryctr++)
    {
       DBGMSF(debug,
-             "Start of try loop, tryctr=%d, max_write_only_exchange_tries=%d, rc=%d, retryable=%d",
-             tryctr, max_write_only_exchange_tries, psc, retryable );
+             "Start of try loop, tryctr=%d, max_tries=%d, rc=%d, retryable=%d",
+             tryctr, max_tries, psc, retryable );
 
       Error_Info * cur_excp = ddc_write_only(dh, request_packet_ptr);
       psc = (cur_excp) ? cur_excp->status_code : 0;
@@ -975,12 +986,12 @@ ddc_write_only_with_retry(
       // now:
       //   tryctr = number of tries
       //   tryctr-1 = index of last try
-      //   tryctr == max_write_only_exchange_tries &&  retryable
-      //   tryctr <  max_write_only_exchange_tries && !retryable
-      //   tryctr == max_write_only_exchange_tries && !retryable
+      //   tryctr == max_tries &&  retryable
+      //   tryctr <  max_tries && !retryable
+      //   tryctr == max_tries && !retryable
 
       // int last_try_index = tryctr-1;
-      DBGTRC(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, bool_repr(retryable));
+      DBGTRC(debug, TRACE_GROUP, "After try loop. tryctr=%d, retryable=%s", tryctr, sbool(retryable));
 
       if (retryable)
          psc = DDCRC_RETRIES;
@@ -1000,7 +1011,7 @@ ddc_write_only_with_retry(
       }
    }
 
-   try_data_record_tries(write_only_stats_rec, psc, tryctr);
+   try_data_record_tries2(WRITE_ONLY_TRIES_OP, psc, tryctr);
 
    DBGTRC(debug, TRACE_GROUP, "Done.  Returning: %s", errinfo_summary(ddc_excp));
    return ddc_excp;
@@ -1013,6 +1024,8 @@ init_ddc_packet_io_func_name_table() {
    ADD_FUNC(ddc_write_read_raw);
    ADD_FUNC(ddc_write_read);
    ADD_FUNC(ddc_write_read_with_retry);
+   ADD_FUNC(ddc_write_only);
+   ADD_FUNC(ddc_write_only_with_retry);
 #undef ADD_FUNC
 }
 
