@@ -19,6 +19,7 @@
 #include "base/core.h"
 #include "base/ddc_errno.h"
 #include "base/sleep.h"
+#include "base/tuned_sleep.h"
 
 #ifdef USE_USB
 #include "usb/usb_displays.h"
@@ -36,40 +37,40 @@
 // Capabilities Related Functions
 //
 
-/* Executes the VCP Get Capabilities command to obtain the
- * capabilities string.  The string is returned in null terminated
- * form in a Buffer struct.  It is the responsibility of the caller to
- * free this struct.
+/** Executes the VCP Get Capabilities command to obtain the
+ *  capabilities string.  The string is returned in null terminated
+ *  form in a Buffer struct.  It is the responsibility of the caller to
+ *  free this struct.
  *
- * Arguments:
- *    dh                    pointer to display handle
- *    ppCapabilitiesBuffer  address at which to return pointer to allocated Buffer
- *    retry_history         if non-null, collects retryable errors
- *
- * Returns:
- *   status code
+ * @param dh                       display handle
+ * @param capabilities_buffer_loc  address at which to return pointer to allocated Buffer
+ * @return                         status code
  */
 static Error_Info *
-get_capabilities_buffer(
+get_capabilities_into_buffer(
       Display_Handle * dh,
-      Buffer**         ppCapabilitiesBuffer)
+      Buffer**         capbilities_buffer_loc)
 {
+   bool debug = false;
+   DBGMSF(debug, "Starting. dh=%s", dh_repr_t(dh));
    Public_Status_Code psc;
    Error_Info * ddc_excp = NULL;
 
+   TUNED_SLEEP_WITH_TRACE(dh, SE_PRE_MULTI_PART_READ, "Before reading capabilities");
+
    ddc_excp = multi_part_read_with_retry(
                dh,
                DDC_PACKET_TYPE_CAPABILITIES_REQUEST,
                0x00,                       // no subtype for capabilities
                false,                      // !all_zero_response_ok
-               ppCapabilitiesBuffer);
-   Buffer * cap_buffer = *ppCapabilitiesBuffer;
+               capbilities_buffer_loc);
+   Buffer * cap_buffer = *capbilities_buffer_loc;
    // psc = (ddc_excp) ? ddc_excp->psc: 0;
    psc = ERRINFO_STATUS(ddc_excp);
    assert(psc <= 0);
    if (psc == 0) {
       // trim trailing blanks and nulls
-      int len = buffer_length(*ppCapabilitiesBuffer);
+      int len = buffer_length(*capbilities_buffer_loc);
       while ( len > 0 ) {
          Byte ch = cap_buffer->bytes[len-1];
          if (ch == ' ' || ch == '\0')
@@ -81,6 +82,7 @@ get_capabilities_buffer(
       buffer_set_byte(cap_buffer, len, '\0');
       buffer_set_length(cap_buffer, len+1);
    }
+   DBGMSF(debug, "Done.     dh=%s, Returning: %s", dh_repr_t(dh), errinfo_summary(ddc_excp));
    return ddc_excp;
 }
 
@@ -121,9 +123,9 @@ get_capabilities_string(
 #endif
       }
       else {
-         SLEEP_MILLIS_WITH_TRACE(200, "Before reading capabilities");
+
          Buffer * pcaps_buffer;
-         ddc_excp = get_capabilities_buffer(dh, &pcaps_buffer);
+         ddc_excp = get_capabilities_into_buffer(dh, &pcaps_buffer);
          // psc = (ddc_excp) ? ddc_excp->psc : 0;
          psc = ERRINFO_STATUS(ddc_excp);
          if (psc == 0) {
@@ -142,6 +144,8 @@ get_capabilities_string(
 Error_Info *
 get_capabilities_string_by_dref(Display_Ref * dref, char **pcaps) {
    assert(dref);
+   bool debug = true;
+   DBGMSF(debug, "Starting. dref=%s", dref_repr_t(dref));
 
    Public_Status_Code psc = 0;
    Error_Info * ddc_excp = NULL;
@@ -150,12 +154,20 @@ get_capabilities_string_by_dref(Display_Ref * dref, char **pcaps) {
       psc = ddc_open_display(dref, CALLOPT_NONE, &dh);
       if (psc == 0) {
          ddc_excp = get_capabilities_string(dh, &dref->capabilities_string);
-         ddc_close_display(dh);
+         int rc = ERRINFO_STATUS(ddc_excp);
+         if (rc == -EBADF) {
+            DBGMSF(debug, "EBADF, skipping ddc_close_display()");
+         }
+         else
+            ddc_close_display(dh);
       }
       else
          ddc_excp = errinfo_new(psc, __func__);
    }
    *pcaps = dref->capabilities_string;
+   DBGMSF(debug, "Done.    Returning: %p", ddc_excp);
+   DBGMSF(debug, "Done.     dref = %s, error_info = %s, capabilities: %s", dref_repr_t(dref),
+          errinfo_summary(ddc_excp), *pcaps);
    return ddc_excp;
 }
 
