@@ -2,7 +2,7 @@
  *  Virtual Control Panel access
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -18,6 +18,7 @@
 
 #include "util/error_info.h"
 #include "util/report_util.h"
+#include "util/string_util.h"
 #include "util/utilrpt.h"
 /** \endcond */
 
@@ -230,7 +231,7 @@ set_table_vcp_value(
  */
 bool ddc_set_verify_setvcp(bool onoff) {
    bool debug = false;
-   DBGMSF(debug, "Setting verify_setvcp = %s", bool_repr(onoff));
+   DBGMSF(debug, "Setting verify_setvcp = %s", sbool(onoff));
    Thread_Vcp_Settings * settings = get_thread_vcp_settings();
    bool old_value = settings->verify_setvcp;
    settings->verify_setvcp = onoff;
@@ -261,7 +262,8 @@ is_rereadable_feature(
    DDCA_Vcp_Feature_Code unrereadable_features[] = {
          0x02,        // new control value
          0x03,        // soft controls
-         0x60,        // input source ???
+         0x60,        // input source - for some monitors it is meaningful to read the new 
+                      //                value, others won't respond if set to a different input
    };
 
    bool result = true;
@@ -286,7 +288,30 @@ is_rereadable_feature(
       }
    }
 
-   DBGMSF(debug, "Returning: %s", bool_repr(result));
+   DBGMSF(debug, "Returning: %s", sbool(result));
+   return result;
+}
+
+
+static bool
+is_unreadable_sl_value(
+      DDCA_Vcp_Feature_Code opcode,
+      Byte                  sl_value)
+{
+   bool debug = false;
+   DBGMSF(debug, "opcode=0x%02x, sl_value=0x%02x", opcode, sl_value);
+
+   bool result = false;
+   switch(opcode)
+   {
+   case 0xd6:
+      if (sl_value == 5)        // turn off display, trying to read from it will fail
+         result = true;
+      break;
+   default:
+      break;
+   }
+   DBGMSF(debug, "Done.     Returning: %s", sbool(result));
    return result;
 }
 
@@ -313,12 +338,11 @@ single_vcp_value_equal(
                      (memcmp(vrec1->val.t.bytes, vrec2->val.t.bytes, vrec1->val.t.bytect) == 0 );
       }
    }
-   DBGMSF(debug, "Returning: %s", bool_repr(result));
+   DBGMSF(debug, "Returning: %s", sbool(result));
    return result;
 }
 
 
-
 // TODO: Consider wrapping set_vcp_value() in set_vcp_value_with_retry(), which would
 // retry in case verification fails
 
@@ -364,7 +388,12 @@ ddc_set_vcp_value(
    }
 
    if (!ddc_excp && ddc_get_verify_setvcp()) {
-      if (is_rereadable_feature(dh, vrec->opcode) ) {
+      if ( is_rereadable_feature(dh, vrec->opcode) &&
+           ( vrec->value_type != DDCA_NON_TABLE_VCP_VALUE ||
+             !is_unreadable_sl_value(vrec->opcode, vrec->val.c_nc.sl)
+           )
+         )
+      {
          f0printf(verbose_msg_dest, "Verifying that value of feature 0x%02x successfully set...\n", vrec->opcode);
          DDCA_Any_Vcp_Value * newval = NULL;
          ddc_excp = ddc_get_vcp_value(
@@ -400,7 +429,12 @@ ddc_set_vcp_value(
          }
       }
       else {
-         f0printf(verbose_msg_dest, "Feature 0x%02x does not support verification\n", vrec->opcode);
+         if (!is_rereadable_feature(dh, vrec->opcode) )
+            f0printf(verbose_msg_dest, "Feature 0x%02x does not support verification\n", vrec->opcode);
+         else
+            f0printf(verbose_msg_dest, "Feature 0x%02x, value 0x%02x does not support verification\n",
+                                       vrec->opcode,
+                                       vrec->val.c_nc.sl);
       }
    }
 
@@ -467,6 +501,18 @@ mock_get_nontable_vcp_value(
 // Get VCP values
 //
 
+
+#ifdef MOVED
+static inline bool
+value_bytes_zero(Parsed_Nontable_Vcp_Response * parsed_val) {
+   return (parsed_val->mh == 0 &&
+           parsed_val->ml == 0 &&
+           parsed_val->sh == 0 &&
+           parsed_val->sl == 0);
+}
+#endif
+
+
 /** Gets the value for a non-table feature.
  *
  *  \param  dh                 handle for open display
@@ -485,7 +531,7 @@ ddc_get_nontable_vcp_value(
        Parsed_Nontable_Vcp_Response** ppInterpretedCode)
 {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "dh=%s, Reading feature 0x%02x", dh_repr_t(dh), feature_code);
+   DBGTRC(debug, TRACE_GROUP, "Starting. dh=%s, Reading feature 0x%02x", dh_repr_t(dh), feature_code);
 
    Public_Status_Code psc = 0;
    Error_Info * excp = NULL;
@@ -508,8 +554,6 @@ ddc_get_nontable_vcp_value(
    Byte expected_subtype = feature_code;
    int max_read_bytes  = 20;    // actually 3 + 8 + 1, or is it 2 + 8 + 1?
 
-   // retry:
-   // psc = ddc_write_read_with_retry(
    excp = ddc_write_read_with_retry(
            dh,
            request_packet_ptr,
@@ -526,7 +570,6 @@ ddc_get_nontable_vcp_value(
          DBGMSG("ddc_write_read_with_retry() returned %s, reponse_packet_ptr=%p",
                 psc_desc(psc), response_packet_ptr);
    }
-   // psc = (excp) ? excp->psc : 0;
 
    if (!excp) {
       assert(response_packet_ptr);
@@ -549,6 +592,18 @@ ddc_get_nontable_vcp_value(
          else if (!parsed_response->supported_opcode) {
             psc = DDCRC_REPORTED_UNSUPPORTED;
             excp = errinfo_new(DDCRC_REPORTED_UNSUPPORTED, __func__);
+            if (!value_bytes_zero(parsed_response)) {
+               // for exploring
+               DBGMSG("supported_opcode == false, but not all value bytes 0");
+            }
+         }
+         else if (value_bytes_zero(parsed_response) &&
+               (dh->dref->flags & DREF_DDC_USES_MH_ML_SH_SL_ZERO_FOR_UNSUPPORTED) )
+         {
+            // just a messages for now
+            DBGMSG("all value bytes 0, supported_opcode == true, setting DDCRC_DETERMINED_UNSUPPORTED)");
+            psc = DDCRC_DETERMINED_UNSUPPORTED;
+            excp = errinfo_new2(psc, __func__, "MH=ML=SH=SL=0");
          }
 
          if (psc != 0) {
@@ -572,13 +627,12 @@ ddc_get_nontable_vcp_value(
    assert( (!excp && parsed_response) || (excp && !parsed_response)); // needed to avoid clang warning
    if (debug || IS_TRACING() ) {
       if (excp) {
-         DBGMSG("Error reading feature x%02x.  Returning exception: %s", feature_code, errinfo_summary(excp));
+         DBGMSG("Done.     Error reading feature x%02x.  Returning exception: %s", feature_code, errinfo_summary(excp));
          // errinfo_report(excp, 1);
-         DBGMSG("Done");
       }
       else {
-         DBGMSG("Success reading feature x%02x. *ppinterpreted_code=%p", feature_code, parsed_response);
-         DBGMSG("  mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x, max value=%d, cur value=%d",
+         DBGMSG("Done.     Success reading feature x%02x. *ppinterpreted_code=%p", feature_code, parsed_response);
+         DBGMSG("          mh=0x%02x, ml=0x%02x, sh=0x%02x, sl=0x%02x, max value=%d, cur value=%d",
                 parsed_response->mh, parsed_response->ml,
                 parsed_response->sh, parsed_response->sl,
                 (parsed_response->mh<<8) | parsed_response->ml,
@@ -590,6 +644,7 @@ ddc_get_nontable_vcp_value(
    return excp;
 }
 
+
 /** Gets the value of a table feature in a newly allocated Buffer struct.
  *  It is the responsibility of the caller to free the Buffer.
  *
@@ -637,7 +692,9 @@ ddc_get_table_vcp_value(
             ddc_excp->status_code == DDCRC_ALL_RESPONSES_NULL)
    {
       Error_Info * wrapped_exception = ddc_excp;
-      ddc_excp = errinfo_new_with_cause(DDCRC_DETERMINED_UNSUPPORTED, wrapped_exception, __func__);
+      ddc_excp = errinfo_new_with_cause2(
+            DDCRC_DETERMINED_UNSUPPORTED, wrapped_exception, __func__, "DDC NULL Message");
+
    }
 
    DBGTRC(debug, TRACE_GROUP,
@@ -746,19 +803,22 @@ ddc_get_vcp_value(
    } // non USB
 
    *valrec_loc = valrec;
+   ASSERT_IFF(psc == 0,*valrec_loc);
 
-   DBGTRC(debug, TRACE_GROUP, "Done. psc=%s", psc_desc(psc) );
-   assert( (psc == 0 && valrec) || (psc != 0 && !valrec) );
-
-   if (psc == 0 && ( debug || IS_TRACING_GROUP(TRACE_GROUP)))
-      dbgrpt_single_vcp_value(valrec,1);
-   assert( (psc == 0 && *valrec_loc) || (psc != 0 && !*valrec_loc) );
-   DBGTRC(debug, TRACE_GROUP, "Done. Returning: %s", errinfo_summary(ddc_excp));
+   if (debug || IS_TRACING() ) {
+      if (psc == 0)  {
+         DBGMSG("Done.     Returning: %s, *valrec ->", errinfo_summary(ddc_excp));
+         dbgrpt_single_vcp_value(valrec,3);
+      }
+      else
+         DBGMSG("Done.     Returning: %s", errinfo_summary(ddc_excp));
+   }
    return ddc_excp;
 }
 
 static void init_ddc_vcp_func_name_table() {
 #define ADD_FUNC(_NAME) rtti_func_name_table_add(_NAME, #_NAME);
+   ADD_FUNC(ddc_get_nontable_vcp_value);
    ADD_FUNC(ddc_get_table_vcp_value);
    ADD_FUNC(ddc_get_vcp_value);
 #undef ADD_FUNC
