@@ -100,8 +100,8 @@ format_relative_gamma(Byte relative_gamma)
  *
  *  \remark
  *  The bytes parm needs to be Byte_Value_Arrary, not a Bit_Bytes_Flag
- *  because the former returns the bytes in the order specified, whereas
- *  the latter effectively sorts them.
+ *  because the former returns the bytes in the order specified in the
+ *  capabilities string, whereas the latter effectively sorts them.
  */
 static void
 report_gamma_capabilities(
@@ -328,52 +328,60 @@ report_capabilities_feature(
       rpt_vstring(d1, "Values (unparsed): %s", vfr->value_string);
    }
 
-#ifdef OLD_BVA
+   // only BVA variant works because of feature x72 (gamma)
+
+#ifdef CFR_BVA
    // hex_dump((Byte*) vfr, sizeof(VCP_Feature_Record));
    // if (vfr->values)
    //    report_id_array(vfr->values, "Feature values:");
    char * buf0 = NULL;
    DBGMSF(debug, "vfr->values=%p", vfr->values);
    if (vfr->values) {
-      // Get the descriptions of the documented values for the feature
-      DDCA_Feature_Value_Entry * feature_values =
-            find_feature_values_for_capabilities(vfr->feature_id, vcp_version);
-
-      DBGMSF(debug, "Feature values %sfound for feature 0x%02x",
-                    (feature_values) ? "" : "NOT ",
-                    vfr->feature_id);
-      int ct = bva_length(vfr->values);
-      if (feature_values) {  // did we find descriptions for the features?
-         if (ol >= DDCA_OL_VERBOSE)
-            f0printf(FOUT, "    Values (  parsed):\n");
-         else
-            f0printf(FOUT, "    Values:\n");
-         int ndx = 0;
-         for (; ndx < ct; ndx++) {
-            Byte hval = bva_get(vfr->values, ndx);
-            char *  value_name = sl_value_table_lookup(feature_values, hval);
-            if (!value_name)
-               value_name = "Unrecognized value";
-            f0printf(FOUT, "       %02x: %s\n", hval, value_name);
-         }
+      if (vfr->feature_id == 0x72) { // special handling for feature x72 (gamma)
+         report_gamma_capabilities(vfr->values, d1);
       }
-      else {          // no interpretation available, just show the values
-         int required_size = 3 * ct;
-         buf0 = malloc(required_size);
-         char * bufend = buf0+required_size;
+      else {
+         // Get the descriptions of the documented values for the feature
+         DDCA_Feature_Value_Entry * feature_values =
+               find_feature_values_for_capabilities(vfr->feature_id, vcp_version);
 
-         char * pos = buf0;
-         int ndx = 0;
-         for (; ndx < ct; ndx++) {
-            Byte hval = bva_get(vfr->values, ndx);
-            snprintf(pos, bufend-pos, "%02X ", hval);
-            pos = pos+3;
+         DBGMSF(debug, "Feature values %sfound for feature 0x%02x",
+                       (feature_values) ? "" : "NOT ",
+                       vfr->feature_id);
+         int ct = bva_length(vfr->values);
+
+         if (feature_values) {  // did we find descriptions for the features?
+            if (ol >= DDCA_OL_VERBOSE)
+               rpt_label(d1, "Values (  parsed):");
+            else
+               rpt_label(d1 , "Values:");
+            int ndx = 0;
+            for (; ndx < ct; ndx++) {
+               Byte hval = bva_get(vfr->values, ndx);
+               char *  value_name = sl_value_table_lookup(feature_values, hval);
+               if (!value_name)
+                  value_name = "Unrecognized value";
+               rpt_vstring(d2, "%02x: %s", hval, value_name);
+            }
+         }
+         else {          // no interpretation available, just show the values
+            int required_size = 3 * ct;
+            buf0 = malloc(required_size);
+            char * bufend = buf0+required_size;
+
+            char * pos = buf0;
+            int ndx = 0;
+            for (; ndx < ct; ndx++) {
+               Byte hval = bva_get(vfr->values, ndx);
+               snprintf(pos, bufend-pos, "%02X ", hval);
+               pos = pos+3;
+            }
+            *(pos-1) = '\0';
+            if (ol >= DDCA_OL_VERBOSE)
+               rpt_vstring(d1, "Values (  parsed): %s (interpretation unavailable)", buf0);
+            else
+               rpt_vstring(d1, "Values: %s (interpretation unavailable)", buf0);
          }
-         *(pos-1) = '\0';
-         if (ol >= DDCA_OL_VERBOSE)
-            f0printf(FOUT, "    Values (  parsed): %s (interpretation unavailable)\n", buf0);
-         else
-            f0printf(FOUT, "    Values: %s (interpretation unavailable)\n", buf0);
       }
    }
 
@@ -382,10 +390,12 @@ report_capabilities_feature(
       free(buf0);
 #endif
 
-// #ifdef NEW_BBF
+#ifdef CFR_BBF
 
+   // WRONG: report_gamma_capabilities requires value bytes in order given in the
+   // capabilities string, no way to get that from BitByteFlags
    DBGMSF(debug, "vfr->bbflags=%p", vfr->bbflags);
-   if (vfr->bbflags) {
+   if (vfr->bbflags) {    // WRONG TEST, NOT A POINTER
 
       // Get the descriptions of the documented values for the feature
       DDCA_Feature_Value_Entry * feature_values = NULL;
@@ -417,6 +427,8 @@ report_capabilities_feature(
             dynamic_disclaimer = " (from user defined feature definition)";
 
          if (vfr->feature_id == 0x72) {    // special handling for gamma
+            // WRONG: report_gamma_capabilities requires value bytes in order given in the
+            // capabilities string, no way to get that from BitByteFlags
             report_gamma_capabilities(vfr->values, d2);
          }
          else {
@@ -441,7 +453,7 @@ report_capabilities_feature(
          free(buf1);
       }
    }
-// #endif
+#endif
 
    DBGMSF(debug, "Done.");
 }
@@ -508,6 +520,8 @@ void dyn_report_parsed_capabilities(
       Display_Ref *            dref,
       int                      depth)
 {
+   int d1 = depth+1;
+   int d2 = depth+2;
    bool debug = false;
    assert(pcaps && memcmp(pcaps->marker, PARSED_CAPABILITIES_MARKER, 4) == 0);
    DBGMSF(debug, "Starting. dh-%s, dref=%s, pcaps->raw_cmds_segment_seen=%s, "
@@ -526,9 +540,18 @@ void dyn_report_parsed_capabilities(
                       (pcaps->raw_value_synthesized) ? "Synthesized unparsed" : "Unparsed",
                       pcaps->raw_value);
    }
+   rpt_vstring(d0, "Model: %s", (pcaps->model) ? pcaps->model : "Not specified");
+
    bool damaged = false;
-   rpt_vstring(d0, "MCCS version: %s",
-                   (pcaps->mccs_version_string) ? pcaps->mccs_version_string : "not present");
+   assert( ( pcaps->mccs_version_string && !vcp_version_eq(pcaps->parsed_mccs_version, DDCA_VSPEC_UNQUERIED) ) ||
+           (!pcaps->mccs_version_string &&  vcp_version_eq(pcaps->parsed_mccs_version, DDCA_VSPEC_UNQUERIED) ) );
+   if (pcaps->mccs_version_string) {
+      char * s = (vcp_version_eq(pcaps->parsed_mccs_version, DDCA_VSPEC_UNKNOWN)) ? " (invalid)" : "";
+      rpt_vstring(d0, "MCCS version: %s%s", pcaps->mccs_version_string, s);
+   }
+   else {
+      rpt_vstring(d0, "MCCS version: Not specified");
+   }
 
    if (pcaps->commands)
       report_commands(pcaps->commands, d0);
@@ -552,7 +575,7 @@ void dyn_report_parsed_capabilities(
 
    if (pcaps->vcp_features) {
 
-#ifdef CAPABILITIES_TEST_CASES
+#ifdef X72_CAPABILITIES_TEST_CASES
       // for testing feature x72 gamma
       char * vstring = NULL;
       Capabilities_Feature_Record * cfr = NULL;
@@ -586,7 +609,19 @@ void dyn_report_parsed_capabilities(
       if (pcaps->raw_vcp_features_seen)
          damaged = true;
    }
-   if (damaged)
+
+   if (damaged)   { // should use pcaps->caps_validity
+      rpt_nl();
       rpt_label(d0, "Capabilities string not completely parsed");
+
+      if (pcaps->messages && pcaps->messages->len > 0) {
+         rpt_label(d1, "Errors:");
+         for (int ndx = 0; ndx < pcaps->messages->len; ndx++) {
+            char * s = g_ptr_array_index(pcaps->messages, ndx);
+            rpt_label(d2, s);
+         }
+      }
+   }
 }
 
+
