@@ -1,18 +1,16 @@
-// i2c_bus_core.c
-
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
-// SPDX-License-Identifier: GPL-2.0-or-later
-
-/** \file
+/** \file i2c_bus_core.c
+ *
  * I2C bus detection and inspection
  */
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <glib-2.0/glib.h>
-// #include <glib.h>
+#include <i2c/i2c_strategy_dispatcher.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/ioctl.h>
@@ -25,6 +23,7 @@
 #include "util/failsim.h"
 #include "util/file_util.h"
 #include "util/glib_string_util.h"
+#include "util/i2c_util.h"
 #include "util/report_util.h"
 #include "util/edid.h"
 #include "util/string_util.h"
@@ -38,10 +37,12 @@
 #include "base/last_io_event.h"
 #include "base/linux_errno.h"
 #include "base/parms.h"
+#include "base/rtti.h"
 #include "base/sleep.h"
 #include "base/status_code_mgt.h"
+#include "base/tuned_sleep.h"
+#include "base/per_thread_data.h"
 
-#include "i2c/i2c_do_io.h"
 #include "i2c/wrap_i2c-dev.h"
 
 #include "i2c/i2c_bus_core.h"
@@ -59,6 +60,10 @@ static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_I2C;
 bool i2c_force_slave_addr_flag = false;
 
 
+// Another ugly global variable for testing purposes
+
+bool i2c_force_bus = false;
+
 //
 // Basic I2C bus operations
 //
@@ -92,23 +97,15 @@ int i2c_open_bus(int busno, Byte callopts) {
    int errsv = errno;
 
    if (fd < 0) {
-#ifdef OLD
-      if (callopts & CALLOPT_ERR_ABORT) {
-         TERMINATE_EXECUTION_ON_ERROR("Open failed for %s. errno=%s\n",
-                                      filename, linux_errno_desc(errsv));
-      }
-#endif
-     //  if (callopts & CALLOPT_ERR_MSG) {
-         f0printf(ferr(), "Open failed for %s: errno=%s\n",
-                        filename, linux_errno_desc(errsv));
-     // }
+      f0printf(ferr(), "Open failed for %s: errno=%s\n", filename, linux_errno_desc(errsv));
       fd = -errsv;
    }
    else {
       RECORD_IO_FINISH_NOW(fd, IE_OPEN);
+      ptd_append_thread_description(filename);
    }
 
-   DBGTRC(debug, TRACE_GROUP, "Returning file descriptor: %d", fd);
+   DBGTRC(debug, TRACE_GROUP, "Done.     busno=%d, Returning file descriptor: %d", busno, fd);
    return fd;
 }
 
@@ -125,8 +122,8 @@ int i2c_open_bus(int busno, Byte callopts) {
 Status_Errno i2c_close_bus(int fd, int busno, Call_Options callopts) {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP,
-          "Starting. fd=%d, busno=%d, callopts=%s",
-          fd, busno,  interpret_call_options_t(callopts));
+          "Starting. busno=%d, fd=%d, callopts=%s",
+          busno,  fd, interpret_call_options_t(callopts));
 
    Status_Errno result = 0;
    int rc = 0;
@@ -166,7 +163,7 @@ Status_Errno i2c_close_bus(int fd, int busno, Call_Options callopts) {
       result = -errsv;
    }
    assert(result <= 0);
-   DBGTRC(debug, TRACE_GROUP, "Returning: %d", result);
+   DBGTRC(debug, TRACE_GROUP, "Done.     fd=%d, busno=%d, Returning: %s", busno, fd, psc_desc(result));
    return result;
 }
 
@@ -176,11 +173,11 @@ Status_Errno i2c_close_bus(int fd, int busno, Call_Options callopts) {
  * @param  fd        Linux file descriptor for open /dev/i2c-n
  * @param  addr      slave address
  * @param  callopts  call option flags, controlling failure action\n
- *                   if CALLOPT_FORCE set, use IOCTL op I2C_SLAVE_FORCE\n
+ *                   if CALLOPT_FORCE set, use IOCTL op I2C_SLAVE_FORCE
  *                   to take control even if address is in use by another driver
  *
  * @retval  0 if success
- * @retval <0 negative Linux errno, if ioctl call fails and CALLOPT_ERR_ABORT not set in callopts
+ * @retval <0 negative Linux errno, if ioctl call fails
  *
  * \remark
  * Errors which are recovered are counted here using COUNT_STATUS_CODE().
@@ -249,125 +246,10 @@ retry:
 }
 
 
-//
-// I2C Bus Inspection - Functionality Flags
-//
-
-
-// Separate table for interpreting functionality flags.
-
-Value_Name_Table functionality_flag_table = {
-      VN(I2C_FUNC_I2C                    ),   //0x00000001
-      VN(I2C_FUNC_10BIT_ADDR             ),   //0x00000002
-      VN(I2C_FUNC_PROTOCOL_MANGLING      ),   //0x00000004 /* I2C_M_IGNORE_NAK etc. */
-      VN(I2C_FUNC_SMBUS_PEC              ),   //0x00000008
-   // VN(I2C_FUNC_NOSTART                ),   //0x00000010 /* I2C_M_NOSTART */  // i2c-tools 4.0
-   // VN(I2C_FUNC_SLAVE                  ),   //0x00000020                      // i2c-tools 4.0
-      {0x00000010, "I2C_FUNC_NOSTART", NULL  },
-      {0x00000020, "I2C_FUNC_SLAVE",   NULL    },
-      VN(I2C_FUNC_SMBUS_BLOCK_PROC_CALL  ),   //0x00008000 /* SMBus 2.0 */
-      VN(I2C_FUNC_SMBUS_QUICK            ),   //0x00010000
-      VN(I2C_FUNC_SMBUS_READ_BYTE        ),   //0x00020000
-      VN(I2C_FUNC_SMBUS_WRITE_BYTE       ),   //0x00040000
-      VN(I2C_FUNC_SMBUS_READ_BYTE_DATA   ),   //0x00080000
-      VN(I2C_FUNC_SMBUS_WRITE_BYTE_DATA  ),   //0x00100000
-      VN(I2C_FUNC_SMBUS_READ_WORD_DATA   ),   //0x00200000
-      VN(I2C_FUNC_SMBUS_WRITE_WORD_DATA  ),   //0x00400000
-      VN(I2C_FUNC_SMBUS_PROC_CALL        ),   //0x00800000
-      VN(I2C_FUNC_SMBUS_READ_BLOCK_DATA  ),   //0x01000000
-      VN(I2C_FUNC_SMBUS_WRITE_BLOCK_DATA ),   //0x02000000
-      VN(I2C_FUNC_SMBUS_READ_I2C_BLOCK   ),   //0x04000000 /* I2C-like block xfer  */
-      VN(I2C_FUNC_SMBUS_WRITE_I2C_BLOCK  ),   //0x08000000 /* w/ 1-byte reg. addr. */
-   // VN(I2C_FUNC_SMBUS_HOST_NOTIFY      ),   //0x10000000               // i2c-tools 4.0
-      {0x10000000, "I2C_FUNC_SMBUS_HOST_NOTIFY", NULL},
-      VN_END
-};
-
-
-/** Gets the I2C functionality flags for an open I2C bus,
- *  specified by its file descriptor.
- *
- *  @param fd  Linux file descriptor
- *  @return functionality flags
- */
-unsigned long i2c_get_functionality_flags_by_fd(int fd) {
-   bool debug = false;
-   DBGMSF(debug, "Starting.");
-
-   unsigned long funcs;
-   int rc;
-
-   RECORD_IO_EVENT(IE_OTHER, ( rc = ioctl(fd, I2C_FUNCS, &funcs) ) );
-   // int errsv = errno;
-   if (rc < 0) {
-      REPORT_IOCTL_ERROR("I2C_FUNCS", errno);
-      funcs = 0;
-   }
-
-   DBGMSF(debug, "Functionality for file descriptor %d: %lu, 0x%0lx", fd, funcs, funcs);
-   return funcs;
-}
-
-
-/** Returns a string representation of functionality flags.
- *
- * @param functionality  long int of flags
- * @return string representation of flags
- */
-char * i2c_interpret_functionality_flags(unsigned long functionality) {
-   // HACK ALERT: There are 2 entries for bit I2C_FUNC_I2C in functionality_table,
-   // one for function name ioctl_read and another for function name ioctl_write
-   // These are at indexes 0 and 1.   For our purposes here we only want to check
-   // each bit once, so we start at index 1 instead of 0.
-   // return vnt_interpret_flags(functionality, functionality_table2+1, false, ", ");
-   return vnt_interpret_flags(functionality, functionality_flag_table, false, ", ");
-}
-
-
-/** Reports functionality flags.
- *
- *  The output is multiline.
- *
- *  @param  functionality  flags to report
- *  @param  maxline        maximum length of 1 line
- *  @param  depth          logical indentation depth
- */
-void i2c_report_functionality_flags(long functionality, int maxline, int depth) {
-   bool debug = false;
-   DBGMSF(debug, "Starting.  functionality=0x%lx, maxline=%d", functionality, maxline);
-
-   char * buf0 = i2c_interpret_functionality_flags(functionality);
-   DBGMSF(debug, "buf0=|%s|", buf0);
-
-   char * header = "Functionality: ";
-   int hdrlen = strlen(header);
-   int maxpiece = maxline - ( rpt_get_indent(depth) + hdrlen);
-
-   Null_Terminated_String_Array ntsa = strsplit_maxlength( buf0, maxpiece, " ");
-   int ntsa_ndx = 0;
-   while (true) {
-      char * s = ntsa[ntsa_ndx++];
-      if (!s)
-         break;
-      // printf("(%s) header=|%s|, s=|%s|\n", __func__, header, s);
-      rpt_vstring(depth, "%-*s%s", hdrlen, header, s);
-      // printf("(%s) s = %p\n", __func__, s);
-      if (strlen(header) > 0)
-         header = "";
-
-   }
-   free(buf0);
-   ntsa_free(ntsa, /* free_strings */ true);
-
-   DBGMSF(debug, "Done");
-}
-
-
 //
 // I2C Bus Inspection - Slave Addresses
 //
 
-
 /** Checks whether a /dev/i2c-n device represents an dDP device,
  *  i.e. a laptop display.
  *
@@ -473,94 +355,103 @@ bool * i2c_detect_all_slave_addrs(int busno) {
 }
 #endif
 
-#ifdef DETECT_SLAVE_ADDRS
-
-/** Checks DDC related addresses on an I2C bus to see if the addresses are active.
- *  The bus device has already been opened.
- *
- * \param  fd       file descriptor for open i2c device
- * \param  presult  where to return result byte
- * \return status code, 0 if success
- *
- * Sets:
- *   Returns byte with flags possibly set:
- *    I2C_BUS_ADDR_0x30        true if addr x30 responds (EDID block selection)
- *    I2C_BUS_ADDR_0x50        true if addr x50 responds (EDID)
- *    I2C_BUS_ADDR_0x37        true if addr x37 responds (DDC commands)
- */
-// static
-Status_Errno_DDC i2c_detect_ddc_addrs_by_fd(int fd, Byte * presult) {
-   bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Starting. fd=%d", fd);
-   assert(fd >= 0);
-   unsigned char result = 0x00;
-
-   Byte    readbuf;  //  1 byte buffer
-   // Byte    writebuf = {0x00};
-   Status_Errno_DDC base_rc = 0;
-
-#ifdef DISABLE
-   // Causes screen corruption on Dell XPS 13, which has a QHD+ eDP screen
-   base_rc = i2c_set_addr(fd, 0x30, CALLOPT_NONE);
-   if (base_rc < 0) {
-      goto bye;
-   }
-   base_rc = invoke_i2c_writer(fd, 1, &writebuf);
-   // DBGMSG("invoke_i2c_writer() returned %s", psc_desc(rc));
-   if (base_rc == 0)
-      result |= I2C_BUS_ADDR_0X30;
-#else
-   // DBGMSG("Skipping probe of slave address x30.");
-#endif
-
-   base_rc = i2c_set_addr(fd, 0x50, CALLOPT_NONE);
-   if (base_rc < 0) {
-      goto bye;
-   }
-   base_rc = invoke_i2c_reader(fd, 1, &readbuf);
-   if (base_rc == 0)
-      result |= I2C_BUS_ADDR_0X50;
-
-#ifdef DISABLE
-   // 10/2018: Causes temporary screen corruption on Dell XPS 13, which has a QHD+ eDP screen
-   base_rc = i2c_set_addr(fd, 0x37, CALLOPT_NONE);
-   if (base_rc < 0) {
-      goto bye;
-   }
-   // 7/2018: changed from invoke_i2c_reader() to invoke_i2c_writer()
-   //         Dell P2715Q does not always respond to read of single byte
-   base_rc = invoke_i2c_writer(fd, 1, &writebuf);
-   DBGTRC(debug, TRACE_GROUP,"invoke_i2c_writer() for slave address x37 returned %s", psc_desc(base_rc));
-   if (base_rc == 0) {
-      base_rc = invoke_i2c_reader(fd, 1, &readbuf);
-      DBGTRC(debug, TRACE_GROUP,"invoke_i2c_reader() for slave address x37 returned %s", psc_desc(base_rc));
-   }
-
-   if (base_rc == 0)
-      result |= I2C_BUS_ADDR_0X37;
-#else
-   // DBGMSG("Skipping probe of slave address x37.");
-#endif
-
-   base_rc = 0;
-
-bye:
-   if (base_rc != 0)
-      result = 0x00;
-
-   *presult = result;
-
-   DBGTRC(debug, TRACE_GROUP,
-          "Done.  Returning base_rc=%s, *presult = 0x%02x", psc_desc(base_rc), *presult);
-   return base_rc;
-}
-
-#endif
 
 //
 // I2C Bus Inspection - EDID Retrieval
 //
 
+static Status_Errno_DDC
+i2c_get_edid_bytes_directly(int fd, Buffer* rawedid, bool read_bytewise)
+{
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "Getting EDID. File descriptor = %d, filename=%s, read_bytewise=%s",
+                 fd, filename_for_fd_t(fd), sbool(read_bytewise));
+
+   Byte byte_to_write = 0x00;
+   int rc = 0;
+   RECORD_IO_EVENTX(
+          fd,
+          IE_WRITE,
+          ( rc = write(fd, &byte_to_write, 1) )
+         );
+   if (rc < 0) {
+      rc = -errno;
+      DBGMSF(debug, "write() failed.  rc = %s", psc_desc(rc));
+   }
+   else {
+      rc = 0;
+      if (read_bytewise) {
+         int ndx = 0;
+         for (; ndx < 128 && rc == 0; ndx++) {
+            RECORD_IO_EVENTX(
+                fd,
+                IE_READ,
+                ( rc = read(fd, &rawedid->bytes[ndx], 1) )
+               );
+            if (rc < 0)
+               rc = -errno;
+            else  {
+               assert(rc == 1);
+               rc = 0;
+            }
+          }
+          DBGMSF(debug, "Final single byte read returned %d, ndx=%d", rc, ndx);
+      }
+      else {
+         RECORD_IO_EVENTX(
+             fd,
+             IE_READ,
+             ( rc = read(fd, rawedid->bytes, 128) )
+            );
+         if (rc >= 0) {
+            assert(rc == 128);
+            rc = 0;
+         }
+         else {
+            rc = -errno;
+         }
+         DBGMSF(debug, "read() returned %s", psc_desc(rc) );
+      }
+   } // write succeeded
+
+   DBGTRC(debug, TRACE_GROUP, "Returning: %s", psc_desc(rc));
+   return rc;
+}
+
+
+static Status_Errno_DDC
+i2c_get_edid_bytes_using_i2c_layer(int fd, Buffer* rawedid, bool read_bytewise)
+{
+   bool debug = false;
+   DBGTRC(debug, TRACE_GROUP, "Getting EDID. File descriptor=%d, filename=%s, read_bytewise=%s",
+                 fd, filename_for_fd_t(fd), sbool(read_bytewise));
+
+   Byte byte_to_write = 0x00;
+   int rc = invoke_i2c_writer(fd, 0x50, 1, &byte_to_write);
+   DBGMSF(debug, "invoke_i2c_writer returned %s", psc_desc(rc));
+   if (rc == 0) {   // write succeeded
+      if (read_bytewise) {
+         int ndx = 0;
+         for (; ndx < 128 && rc == 0; ndx++) {
+            // DBGMSG("Before invoke_i2c_reader() call");
+            rc = invoke_i2c_reader(fd, 0x50, false, 1, &rawedid->bytes[ndx] );
+         }
+         DBGMSF(debug, "Final single byte read returned %d, ndx=%d", rc, ndx);
+      } // read_bytewise == true
+      else {
+         rc = invoke_i2c_reader(fd, 0x50, read_bytewise, 128, rawedid->bytes);
+         DBGMSF(debug, "invoke_i2c_reader returned %s", psc_desc(rc));
+
+      }
+      if (rc == 0) {
+         rawedid->len = 128;
+      }
+   }  // write succeeded
+   DBGTRC(debug, TRACE_GROUP, "Returning: %s", psc_desc(rc));
+   return rc;
+}
+
+
 /** Gets EDID bytes of a monitor on an open I2C device.
  *
  * @param  fd        file descriptor for open /dev/i2c-n
@@ -571,9 +462,11 @@ bye:
  */
 Status_Errno_DDC i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid) {
    bool debug = false;
-   DBGTRC(debug, TRACE_GROUP, "Getting EDID for file %d", fd);
-
-   bool conservative = true;
+   DBGTRC(debug, TRACE_GROUP, "Getting EDID. File descriptor = %d, filename=%s",
+                              fd, filename_for_fd_t(fd));
+#ifdef OLD
+   bool conservative = false;
+#endif
 
    assert(rawedid->buffer_size >= 128);
    Status_Errno_DDC rc;
@@ -582,50 +475,74 @@ Status_Errno_DDC i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid) {
    if (rc < 0) {
       goto bye;
    }
+
+#ifdef OLD
+   // need a different call since tuned_sleep_with_tracex() now takes Display_Handle *, not DDCA_IO_Type
    // 10/23/15, try disabling sleep before write
-   if (conservative)
-      sleep_millis_with_tracex(DDC_TIMEOUT_MILLIS_DEFAULT, __func__, __LINE__, __FILE__, "Before write");
+   if (conservative) {
+      TUNED_SLEEP_WITH_TRACE(DDCA_IO_I2C, SE_PRE_EDID, "Before write");
+   }
+#endif
 
-   Byte byte_to_write = 0x00;
+   int max_tries = 4;  // 2 each read_bytewise == true/false
+   bool read_bytewise = EDID_Read_Bytewise;
+   rc = -1;
+   DBGMSF(debug, "EDID read performed using %s,read_bytewise=%s",
+                 (EDID_Read_Uses_I2C_Layer) ? "I2C layer" : "local io", sbool(read_bytewise));
 
-   int max_tries = 3;
-   for (int tryctr = 0; tryctr < max_tries; tryctr++) {
-      rc = invoke_i2c_writer(fd, 1, &byte_to_write);
-      if (rc == 0) {
-         rc = invoke_i2c_reader(fd, 128, rawedid->bytes);
-         assert(rc <= 0);
-         if (rc == 0) {
-            rawedid->len = 128;
-            if (debug) {
-               DBGMSG("call_read returned:");
-               dbgrpt_buffer(rawedid, 1);
-               DBGMSG("edid checksum = %d", edid_checksum(rawedid->bytes) );
-            }
-            Byte checksum = edid_checksum(rawedid->bytes);
-            if (checksum != 0) {
-               // possible if successfully read bytes from i2c bus with no monitor
-               // attached - the bytes will be junk.
-               // e.g. nouveau driver, Quadro card, on blackrock
-               DBGTRC(debug, TRACE_GROUP, "Invalid EDID checksum %d, expected 0.", checksum);
-               rawedid->len = 0;
-               rc = DDCRC_INVALID_EDID;    // was DDCRC_EDID
-            }
-         }
-         if (rc == 0)
-            break;
+   for (int tryctr = 0; tryctr < max_tries && rc != 0; tryctr++) {
+      DBGMSF(debug, "Trying EDID read, tryctr=%d, max_tries=%d, read_bytewise=%s",
+                    tryctr, max_tries, sbool(read_bytewise));
+
+      if (EDID_Read_Uses_I2C_Layer) {
+         rc = i2c_get_edid_bytes_using_i2c_layer(fd, rawedid, read_bytewise);
       }
-      if (tryctr < max_tries)
-         DBGTRC(debug, TRACE_GROUP, "Retrying EDID read.  tryctr=%d, max_tries=%d", tryctr, max_tries);
+      else {
+         rc = i2c_get_edid_bytes_directly(fd, rawedid, read_bytewise);
+      }
+      if (rc == -ENXIO || rc == -EIO) {
+         // DBGMSG("breaking");
+         break;
+      }
+      assert(rc <= 0);
+      if (rc == 0) {
+         rawedid->len = 128;
+         if (debug || IS_TRACING() ) {
+            DBGMSG("get bytes returned:");
+            dbgrpt_buffer(rawedid, 1);
+            DBGMSG("edid checksum = %d", edid_checksum(rawedid->bytes) );
+         }
+         Byte checksum = edid_checksum(rawedid->bytes);
+         if (checksum != 0) {
+            // possible if successfully read bytes from i2c bus with no monitor
+            // attached - the bytes will be junk.
+            // e.g. nouveau driver, Quadro card, on blackrock
+            DBGTRC(debug, TRACE_GROUP, "Invalid EDID checksum %d, expected 0.", checksum);
+            rawedid->len = 0;
+            rc = DDCRC_INVALID_EDID;    // was DDCRC_EDID
+         }
+      }  // get bytes succeeded
+
+      // try the alternative
+      // read_bytewise = !read_bytewise;
    }
 
 bye:
    if (rc < 0)
       rawedid->len = 0;
 
+#ifdef NO
    if (debug || IS_TRACING()) {
-      DBGMSG("Returning %s.  edidbuf contents:", psc_desc(rc));
-      buffer_dump(rawedid);
+      if (rc == 0) {
+         DBGMSG("Returning %s.  edidbuf contents:", psc_desc(rc));
+         buffer_dump(rawedid);
+      }
+      else {
+         DBGMSG("Returning %s", psc_desc(rc));
+      }
    }
+#endif
+   DBGTRC(debug, TRACE_GROUP, "Returning %s", psc_desc(rc));
    return rc;
 }
 
@@ -684,6 +601,58 @@ static I2C_Bus_Info * i2c_new_bus_info(int busno) {
 }
 
 
+bool i2c_detect_x37(int fd) {
+   bool debug = false;
+   bool result = false;
+
+   // Quirks
+   // - i2c_set_addr() Causes screen corruption on Dell XPS 13, which has a QHD+ eDP screen
+   //   avoided by never calling this function for an eDP screen
+   // - Dell P2715Q does not respond to single byte read, but does respond to
+   //   a write (7/2018), so this function checks both
+   Status_Errno_DDC rc = i2c_set_addr(fd, 0x37, CALLOPT_NONE);
+   if (rc == 0)  {
+      // regard either a successful write() or a read() as indication slave address is valid
+      Byte    writebuf = {0x00};
+      rc = write(fd, &writebuf, 1);
+      DBGTRC(debug, TRACE_GROUP,"write() for slave address x37 returned %s", psc_desc(rc));
+      if (rc == 1)
+         result = true;
+
+      Byte    readbuf[4];  //  4 byte buffer
+      rc = read(fd, readbuf, 4);
+      DBGTRC(debug, TRACE_GROUP,"read() for slave address x37 returned %s", psc_desc(rc));
+      // test doesn't work, buffer contains random bytes (but same random bytes for every
+      // display in a single call to i2cdetect)
+      // Byte ddc_null_msg[4] = {0x6f, 0x6e, 0x80, 0xbe};
+      // if (rc == 4) {
+      //    DBGMSG("read x37 returned: 0x%08x", readbuf);
+      //    result = (memcmp( readbuf, ddc_null_msg, 4) == 0);
+      // }
+      if (rc >= 0)
+         result = true;
+   }
+
+   DBGMSF(debug, "Returning %s", SBOOL(result));
+   return result;
+}
+
+
+// Factored out of i2c_check_bus().  Not needed, since i2c_check_bus() is called
+// only when the bus name is valid
+void i2c_bus_check_valid_name(I2C_Bus_Info * bus_info) {
+  assert(bus_info && ( memcmp(bus_info->marker, I2C_BUS_INFO_MARKER, 4) == 0) );
+
+  if ( !(bus_info->flags & I2C_BUS_VALID_NAME_CHECKED) ) {
+      bus_info->flags |= I2C_BUS_VALID_NAME_CHECKED;
+      if ( !sysfs_is_ignorable_i2c_device(bus_info->busno) )
+         bus_info->flags |= I2C_BUS_HAS_VALID_NAME;
+   }
+
+   bus_info->flags |= I2C_BUS_HAS_VALID_NAME;
+}
+
+
 /** Inspects an I2C bus.
  *
  *  Takes the number of the bus to be inspected from the #I2C_Bus_Info struct passed
@@ -691,104 +660,45 @@ static I2C_Bus_Info * i2c_new_bus_info(int busno) {
  *
  *  @param  bus_info  pointer to #I2C_Bus_Info struct in which information will be set
  */
-static void i2c_check_bus(I2C_Bus_Info * bus_info) {
+void i2c_check_bus(I2C_Bus_Info * bus_info) {
    bool debug = false;
    DBGTRC(debug, TRACE_GROUP, "Starting. busno=%d, buf_info=%p", bus_info->busno, bus_info );
 
-   assert(bus_info);
-   assert( memcmp(bus_info->marker, I2C_BUS_INFO_MARKER, 4) == 0);
+   assert(bus_info && ( memcmp(bus_info->marker, I2C_BUS_INFO_MARKER, 4) == 0) );
 
-   int fd = 0; // Linux file descriptor
+   // void i2c_bus_check_valid_name(bus_info);  // unnecessary
+   assert( (bus_info->flags & I2C_BUS_EXISTS) &&
+           (bus_info->flags & I2C_BUS_VALID_NAME_CHECKED) &&
+           (bus_info->flags & I2C_BUS_HAS_VALID_NAME)
+         );
 
    if (!(bus_info->flags & I2C_BUS_PROBED)) {
       DBGMSF(debug, "Probing");
       bus_info->flags |= I2C_BUS_PROBED;
+      int fd = i2c_open_bus(bus_info->busno, CALLOPT_ERR_MSG);
+      if (fd >= 0) {
+          DBGMSF(debug, "Opened bus /dev/i2c-%d", bus_info->busno);
+          bus_info->flags |= I2C_BUS_ACCESSIBLE;
 
-      if ( is_edp_device(bus_info->busno) ) {
-         DBGMSF(debug, "eDP device detected");
-         bus_info->flags |= I2C_BUS_EDP;
-         // goto bye;
+          bus_info->functionality = i2c_get_functionality_flags_by_fd(fd);
+
+          DDCA_Status ddcrc = i2c_get_parsed_edid_by_fd(fd, &bus_info->edid);
+          DBGMSF(debug, "i2c_get_parsed_edid_by_fd() returned %d", ddcrc);
+          if (ddcrc == 0) {
+             bus_info->flags |= I2C_BUS_ADDR_0X50;
+
+             if ( is_edp_device(bus_info->busno) ) {
+                DBGMSF(debug, "eDP device detected");
+                bus_info->flags |= I2C_BUS_EDP;
+             }
+             else {
+                if ( i2c_detect_x37(fd) )
+                   bus_info->flags |= I2C_BUS_ADDR_0X37;
+             }
+          }
+          i2c_close_bus(fd, bus_info->busno,  CALLOPT_ERR_MSG);
       }
-
-      // probing hangs on PowerMac if i2c device is SMU
-      // but, bus_info has already been filtered for ignorable devices
-      assert( !is_ignorable_i2c_device(bus_info->busno) );
-
-#ifdef OLD
-      // tests no longer needed, see comment on above assert()
-      if ( !(bus_info->flags & I2C_BUS_VALID_NAME_CHECKED) ) {
-         bus_info->flags |= I2C_BUS_VALID_NAME_CHECKED;
-         if ( !is_ignorable_i2c_device(bus_info->busno) )
-            bus_info->flags |= I2C_BUS_HAS_VALID_NAME;
-      }
-#endif
-      bus_info->flags |= I2C_BUS_VALID_NAME_CHECKED;
-      bus_info->flags |= I2C_BUS_HAS_VALID_NAME;
-
-      if (bus_info->flags & I2C_BUS_VALID_NAME_CHECKED &&
-          bus_info->flags & I2C_BUS_HAS_VALID_NAME )
-      {
-         fd = i2c_open_bus(bus_info->busno, CALLOPT_ERR_MSG);  // returns if failure
-
-         if (fd >= 0) {
-            bus_info->flags |= I2C_BUS_ACCESSIBLE;
-
-            bus_info->functionality = i2c_get_functionality_flags_by_fd(fd);
-            // DBGMSF(debug, "i2c_get_functionality_flags_by_fd() returned %lu", bus_info->functionality);
-
-#ifdef DETECT_SLAVE_ADDRS
-            Byte ddc_addr_flags = 0x00;
-            Status_Errno_DDC psc = i2c_detect_ddc_addrs_by_fd(fd, &ddc_addr_flags);
-            if (psc != 0) {
-               DBGMSF(debug, "detect_ddc_addrs_by_fd() returned %d", psc);
-               f0printf(ferr(), "Failure detecting bus addresses for /dev/i2c-%d: status code=%s\n",
-                              bus_info->busno, psc_desc(psc));
-               goto bye;
-            }
-            bus_info->flags |= ddc_addr_flags;
-
-            if (bus_info->flags & I2C_BUS_ADDR_0X50) {
-               // Have seen case of nouveau driver with Quadro card where
-               // there's a bus that has no monitor but responds to the X50 probe
-               // of detect_ddc_addrs_by_fd() and then returns a garbage EDID
-               // when the bytes are read in i2c_get_parsed_edid_by_fd()
-               // TODO: handle case of i2c_get_parsed_edid_by_fd() returning NULL
-               // but should never fail if detect_ddc_addrs_by_fd() succeeds
-               psc = i2c_get_parsed_edid_by_fd(fd, &bus_info->edid);
-               if (psc != 0) {
-                  DBGMSF(debug, "i2c_get_parsed_edid_by_fd() returned %d", psc);
-                  f0printf(ferr(), "Failure getting EDID for /dev/i2c-%d: status code=%s\n",
-                                 bus_info->busno, psc_desc(psc));
-                  goto bye;
-               }
-               // bus_info->flags |= I2C_BUS_EDID_CHECKED;
-            }
-   #ifdef NO
-            // test is being made in ddc_displays.c
-            if (bus_info->flags & I2C_BUS_ADDR_0X37) {
-               // have seen case where laptop display reports addr 37 active, but
-               // it doesn't respond to DDC
-               // 8/2017: If DDC turned off on U3011 monitor, addr x37 still detected
-               // TODO: sanity check for DDC goes here
-               // or make this check at a higher level, since I2c doesn't understand DDC
-
-            }
-   #endif
-#else
-            Public_Status_Code psc = i2c_get_parsed_edid_by_fd(fd, &bus_info->edid);
-            DBGMSF(debug, "i2c_get_parsed_edid_by_fd() returned %d", psc);
-            if (psc == 0)
-               bus_info->flags |= I2C_BUS_ADDR_0X50;
-            else
-               goto bye;     // not logically necessary, avoids iftesting label bye
-#endif
-         }
-      }
-   }
-
-bye:
-   if (fd >= 0)
-      i2c_close_bus(fd, bus_info->busno,  CALLOPT_ERR_MSG);
+   }   // probing complete
 
    // DBGTRC(debug, TRACE_GROUP, "Done. flags=0x%02x", bus_info->flags );
    if (debug || IS_TRACING() ) {
@@ -860,8 +770,8 @@ void i2c_dbgrpt_bus_info(I2C_Bus_Info * bus_info, int depth) {
       rpt_vstring(depth, "I2C bus has valid name:  %s", sbool(bus_info->flags & I2C_BUS_HAS_VALID_NAME));
 #ifdef DETECT_SLAVE_ADDRS
       rpt_vstring(depth, "Address 0x30 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X30));
-      rpt_vstring(depth, "Address 0x37 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X37));
 #endif
+      rpt_vstring(depth, "Address 0x37 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X37));
       rpt_vstring(depth, "Address 0x50 present:    %s", sbool(bus_info->flags & I2C_BUS_ADDR_0X50));
       // not useful and clutters the output
       // i2c_report_functionality_flags(bus_info->functionality, /* maxline */ 90, depth);
@@ -900,13 +810,15 @@ void i2c_report_active_display(I2C_Bus_Info * businfo, int depth) {
       rpt_vstring(depth+1, "I2C address 0x30 (EDID block#)  present: %-5s", srepr(businfo->flags & I2C_BUS_ADDR_0X30));
       rpt_vstring(depth+1, "I2C address 0x37 (DDC)          present: %-5s", srepr(businfo->flags & I2C_BUS_ADDR_0X37));
 #endif
-      rpt_vstring(depth+1, "I2C address 0x50 (EDID) present: %-5s", sbool(businfo->flags & I2C_BUS_ADDR_0X50));
-      rpt_vstring(depth+1, "Is eDP device:                   %-5s", sbool(businfo->flags & I2C_BUS_EDP));
+      rpt_vstring(depth+1, "I2C address 0x50 (EDID) responsive: %-5s", sbool(businfo->flags & I2C_BUS_ADDR_0X50));
+      rpt_vstring(depth+1, "Is eDP device:                      %-5s", sbool(businfo->flags & I2C_BUS_EDP));
+      if (!(businfo->flags & I2C_BUS_EDP))
+      rpt_vstring(depth+1, "I2C address 0x37 (DDC) responsive:  %-5s", sbool(businfo->flags & I2C_BUS_ADDR_0X37));
 
       char fn[PATH_MAX];     // yes, PATH_MAX is dangerous, but not as used here
       sprintf(fn, "/sys/bus/i2c/devices/i2c-%d/name", businfo->busno);
       char * sysattr_name = file_get_first_line(fn, /* verbose*/ false);
-      rpt_vstring(depth+1, "%s: %s", fn, sysattr_name);
+      rpt_vstring(depth+1, "%s:   %s", fn, sysattr_name);
       free(sysattr_name);
    }
 
@@ -1002,6 +914,7 @@ int i2c_detect_buses() {
          i2c_check_bus(businfo);
          // if (debug || IS_TRACING() )
          //    i2c_dbgrpt_bus_info(businfo, 0);
+         DBGMSF(debug, "Valid bus: /dev/i2c-%d", busno);
          g_ptr_array_add(i2c_buses, businfo);
       }
       bva_free(i2c_bus_bva);
@@ -1012,14 +925,18 @@ int i2c_detect_buses() {
 }
 
 
-I2C_Bus_Info * detect_single_bus(int busno) {
+void i2c_discard_buses() {
+   i2c_buses= NULL;
+}
+
+I2C_Bus_Info * i2c_detect_single_bus(int busno) {
    bool debug = false;
    DBGTRC(debug, DDCA_TRC_I2C, "Starting.  busno = %d", busno);
    I2C_Bus_Info * businfo = NULL;
 
    if (i2c_device_exists(busno) ) {
       businfo = i2c_new_bus_info(busno);
-      businfo->flags = I2C_BUS_EXISTS;
+      businfo->flags = I2C_BUS_EXISTS | I2C_BUS_VALID_NAME_CHECKED | I2C_BUS_HAS_VALID_NAME;
       i2c_check_bus(businfo);
       if (debug)
          i2c_dbgrpt_bus_info(businfo, 0);
@@ -1180,7 +1097,7 @@ int i2c_report_buses(bool report_all, int depth) {
 
    puts("");
    if (report_all)
-      rpt_vstring(depth,"Detected %d I2C buses:", busct);
+      rpt_vstring(depth,"Detected %d non-ignorable I2C buses:", busct);
    else
       rpt_vstring(depth, "I2C buses with monitors detected at address 0x50:");
 
@@ -1208,3 +1125,20 @@ bool is_probably_laptop_display(I2C_Bus_Info * businfo) {
 }
 #endif
 
+static void init_i2c_bus_core_func_name_table() {
+#define ADD_FUNC(_NAME) rtti_func_name_table_add(_NAME, #_NAME);
+   ADD_FUNC(i2c_open_bus);
+   ADD_FUNC(i2c_close_bus);
+   ADD_FUNC(i2c_get_edid_bytes_using_i2c_layer);
+   ADD_FUNC(i2c_get_edid_bytes_directly);
+   ADD_FUNC(i2c_detect_buses);
+   ADD_FUNC(i2c_detect_single_bus);
+   ADD_FUNC(i2c_get_raw_edid_by_fd);
+#undef ADD_FUNC
+}
+
+void init_i2c_bus_core() {
+   init_i2c_bus_core_func_name_table();
+}
+
+
