@@ -1,8 +1,8 @@
 /** \file i2c_bus_core.h
+ *
  *  I2C bus detection and inspection
  */
-
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef I2C_BUS_CORE_H_
@@ -37,17 +37,13 @@ extern GPtrArray * i2c_buses;
 // changing ioctl op I2C_SLAVE to op I2C_SLAVE_FORCE.
 extern bool i2c_force_slave_addr_flag;
 
+extern bool i2c_force_bus;
+
 // Basic I2C bus operations
 int           i2c_open_bus(int busno, Call_Options callopts);
 Status_Errno  i2c_close_bus(int fd, int busno, Call_Options callopts);
 Status_Errno  i2c_set_addr(int fd, int addr, Call_Options callopts);
 
-// Bus functionality flags
-unsigned long i2c_get_functionality_flags_by_fd(int fd);
-char *        i2c_interpret_functionality_flags(unsigned long functionality);
-void          i2c_report_functionality_flags(long functionality, int maxline, int depth);
-
-
 // EDID inspection
 Status_Errno_DDC i2c_get_raw_edid_by_fd(int fd, Buffer * rawedid);
 Status_Errno_DDC i2c_get_parsed_edid_by_fd(int fd, Parsed_Edid ** edid_ptr_loc);
@@ -84,7 +80,8 @@ int  i2c_device_count();           // simple /dev/i2c-n count, no side effects
 
 // Bus inventory - detect and probe buses
 int i2c_detect_buses();            // creates internal array of Bus_Info for I2C buses
-I2C_Bus_Info * detect_single_bus(int busno);
+void i2c_discard_buses();
+I2C_Bus_Info * i2c_detect_single_bus(int busno);
 void i2c_free_bus_info(I2C_Bus_Info * bus_info);
 
 // Simple Bus_Info retrieval
@@ -95,4 +92,6 @@ I2C_Bus_Info * i2c_find_bus_info_by_busno(int busno);
 // Reports all detected i2c buses:
 int  i2c_report_buses(bool report_all, int depth);
 
+void init_i2c_bus_core();
+
 #endif /* I2C_BUS_CORE_H_ */
