@@ -3,7 +3,7 @@
  *  C API base functions.
  */
 
-// Copyright (C) 2015-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2015-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "config.h"
@@ -12,11 +12,16 @@
 #include <errno.h>
 #include <string.h>
  
+#include "public/ddcutil_c_api.h"
+
 #include "base/base_init.h"
 #include "base/build_info.h"
 #include "base/core.h"
 #include "base/parms.h"
+#include "base/thread_retry_data.h"
+#include "base/thread_sleep_data.h"
 #include "base/tuned_sleep.h"
+#include "base/per_thread_data.h"
 
 #include "adl/adl_shim.h"
 
@@ -25,14 +30,17 @@
 #include "ddc/ddc_multi_part_io.h"
 #include "ddc/ddc_packet_io.h"
 #include "ddc/ddc_services.h"
+#include "ddc/ddc_try_stats.h"
 #include "ddc/ddc_vcp.h"
 #include "ddc/ddc_watch_displays.h"
 
-#include "public/ddcutil_c_api.h"
-
 #include "libmain/api_base_internal.h"
 
 
+//
+// Precondition Failure
+//
+
 DDCA_Api_Precondition_Failure_Mode api_failure_mode = DDCA_PRECOND_STDERR_RETURN;
 
 DDCA_Api_Precondition_Failure_Mode
@@ -51,26 +59,6 @@ ddca_get_precondition_failure_mode()
 }
 
 
-
-
-__THROW  __attribute__ ((__noreturn__))
-void __precond_fail (const char *__assertion, const char *__file,
-            unsigned int __line, const char *__function)
-{
-   fprintf(stderr, "Precondition failure in function %s at line %d of file %s: %s\n",
-                   __function, __line, __file, __assertion);
-   abort();
-}
-
-__THROW  __attribute__ ((__noreturn__))
-void __precond_abort()
-{
-   abort();
-}
-
-
-
-
 //
 // Library Build Information
 //
@@ -433,19 +421,32 @@ ddca_max_max_tries(void) {
 
 int
 ddca_get_max_tries(DDCA_Retry_Type retry_type) {
+#ifdef OLD
    int result = 0;
    switch(retry_type) {
-      case (DDCA_WRITE_ONLY_TRIES):
+      case (WRITE_ONLY_TRIES_OP):
          result = ddc_get_max_write_only_exchange_tries();
       break;
-   case (DDCA_WRITE_READ_TRIES):
+   case (WRITE_READ_TRIES_OPE):
       result = ddc_get_max_write_read_exchange_tries();
       break;
-   case (DDCA_MULTI_PART_TRIES):
+   case (MULTI_PART_READ_OP):
       result = ddc_get_max_multi_part_read_tries();
       break;
+   case (MULTI_PART_WRITE_OP):
+      result = ddc_get_max_multi_part_write_tries();
+      break;
    }
-   return result;
+#endif
+
+
+
+   Retry_Op_Value result3 = try_data_get_maxtries2(retry_type);
+   // new way using retry_mgt
+   Retry_Op_Value result2 = trd_get_thread_max_tries(retry_type);
+   // assert(result == result2);
+   assert(result2 == result3);
+   return result2;
 }
 
 
@@ -459,18 +460,35 @@ ddca_set_max_tries(
    if (max_tries < 1 || max_tries > MAX_MAX_TRIES)
       rc = DDCRC_ARG;
    else {
+
+#ifdef OLD
       switch(retry_type) {
-      case (DDCA_WRITE_ONLY_TRIES):
-         ddc_set_max_write_only_exchange_tries(max_tries);
+      case (WRITE_ONLY_TRIES_OP):
+         ddc_set_max_write_only_exchange_tries(max_tries);   // sets in Try_Data
+
          break;
-      case (DDCA_WRITE_READ_TRIES):
-         ddc_set_max_write_read_exchange_tries(max_tries);
+      case (WRITE_READ_TRIES_OPE):
+         ddc_set_max_write_read_exchange_tries(max_tries);   // sets in Try_Data
          break;
-      case (DDCA_MULTI_PART_TRIES):
-         ddc_set_max_multi_part_read_tries(max_tries);
+      case (MULTI_PART_READ_OP):
+         ddc_set_max_multi_part_read_tries(max_tries);       // sets in Try_Data  , sets vars in ddc_muti_part.io
+      case (MULTI_PART_WRITE_OP):
          ddc_set_max_multi_part_write_tries(max_tries);      // TODO: Separate constant
          break;
       }
+#endif
+
+
+
+      try_data_set_maxtries2(retry_type, max_tries);
+      // for DDCA_MULTI_PART_TRIES, set both  MULTI_PART_WRITE_OP and MULTI_PART_READ_OP
+      if (retry_type == DDCA_MULTI_PART_TRIES)
+         try_data_set_maxtries2(MULTI_PART_WRITE_OP, max_tries);
+
+      // new way, set in retry_mgt
+      trd_set_thread_max_tries(retry_type, max_tries);
+      if (retry_type == DDCA_MULTI_PART_TRIES)
+           trd_set_thread_max_tries(MULTI_PART_WRITE_OP, max_tries);
    }
    return rc;
 }
@@ -487,16 +505,72 @@ ddca_is_verify_enabled() {
    return ddc_get_verify_setvcp();
 }
 
+#ifdef NOT_NEEDED
+void ddca_lock_default_sleep_multiplier() {
+   lock_default_sleep_multiplier();
+}
+
+void ddca_unlock_sleep_multiplier() {
+   unlock_default_sleep_multiplier();
+}
+#endif
+
+
+bool
+ddca_enable_sleep_suppression(bool newval) {
+   bool old = is_sleep_suppression_enabled();
+   enable_sleep_suppression(newval);
+   return old;
+}
+
+bool
+ddca_is_sleep_suppression_enabled() {
+   return is_sleep_suppression_enabled();
+}
+
+
+void
+ddca_set_default_sleep_multiplier(double multiplier)
+{
+   tsd_set_default_sleep_multiplier_factor(multiplier);
+}
+
+double
+ddca_get_default_sleep_multiplier()
+{
+   return tsd_get_default_sleep_multiplier_factor();
+}
+
 void
 ddca_set_global_sleep_multiplier(double multiplier)
 {
-   set_sleep_multiplier_factor(multiplier);
+   ddca_set_default_sleep_multiplier(multiplier);
 }
 
 double
 ddca_get_global_sleep_multiplier()
 {
-   return get_sleep_multiplier_factor();
+   return ddca_get_default_sleep_multiplier();
+}
+
+// for current thread
+void
+ddca_set_sleep_multiplier(double multiplier)
+{
+   // bool debug = true;
+   // DBGMSF(debug, "Setting %5.2f", multiplier);
+   tsd_set_sleep_multiplier_factor(multiplier);
+   // DBGMSF(debug, "Done");
+}
+
+double
+ddca_get_sleep_multiplier()
+{
+   // bool debug = true;
+   // DBGMSF(debug, "Starting");
+   double result = tsd_get_sleep_multiplier_factor();
+   // DBGMSF(debug, "Returning %5.2f", result);
+   return result;
 }
 
 
@@ -556,6 +630,7 @@ ddca_trace_group_name_to_value(char * name) {
 void
 ddca_set_trace_options(DDCA_Trace_Options  options) {
    // global variables in core.c
+   // DBGMSG("options = 0x%02x", options);
    dbgtrc_show_time = options & DDCA_TRCOPT_TIMESTAMP;
    dbgtrc_show_thread_id = options & DDCA_TRCOPT_THREAD_ID;
 }
@@ -570,15 +645,52 @@ ddca_set_trace_options(DDCA_Trace_Options  options) {
 
 // TODO: Add functions to access ddcutil's runtime error statistics
 
+
+#ifdef UNUSED
+void
+ddca_register_thread_dref(DDCA_Display_Ref dref) {
+   ptd_register_thread_dref( (Display_Ref *) dref);
+}
+#endif
+
+void
+ddca_set_thread_description(
+      const char * description)
+{
+   ptd_set_thread_description( description );
+}
+
+void
+ddca_append_thread_description(
+      const char * description)
+{
+   ptd_append_thread_description(description);
+}
+
+const char *
+ddca_get_thread_descripton() {
+   return ptd_get_thread_description_t();
+}
+
+
+
 void
 ddca_reset_stats(void) {
+   // DBGMSG("Executing");
    ddc_reset_stats_main();
 }
 
 // TODO: Functions that return stats in data structures
 void
-ddca_show_stats(DDCA_Stats_Type stats_types, int depth) {
+ddca_show_stats(
+      DDCA_Stats_Type stats_types,
+      bool            by_thread,
+      int             depth)
+{
+   // DBGMSG("stats_types = %d, depth=%d", stats_types, depth);
+
    ddc_report_stats_main( stats_types,    // stats to show
+                          by_thread,
                           depth);         // logical indentation depth
 }
 
