@@ -12,6 +12,7 @@
 #include "public/ddcutil_c_api.h"
 
 #include "util/report_util.h"
+#include "util/string_util.h"
 
 #include "base/core.h"
 #include "base/displays.h"
@@ -48,13 +49,15 @@ void ddca_feature_list_clear(DDCA_Feature_List* vcplist) {
 }
 
 
-void ddca_feature_list_add(DDCA_Feature_List * vcplist, uint8_t vcp_code) {
+DDCA_Feature_List
+ddca_feature_list_add(DDCA_Feature_List * vcplist, uint8_t vcp_code) {
    feature_list_add(vcplist, vcp_code);
+   return *vcplist;
 }
 
 
-bool ddca_feature_list_contains(DDCA_Feature_List * vcplist, uint8_t vcp_code) {
-   return feature_list_contains(vcplist, vcp_code);
+bool ddca_feature_list_contains(DDCA_Feature_List vcplist, uint8_t vcp_code) {
+   return feature_list_contains(&vcplist, vcp_code);
 }
 
 
@@ -65,25 +68,28 @@ ddca_feature_list_id_name(
    char * result = NULL;
    switch (feature_subset_id) {
    case DDCA_SUBSET_KNOWN:
-      result = "VCP_SUBSET_KNOWN";
+      result = "DDCA_SUBSET_KNOWN";
       break;
    case DDCA_SUBSET_COLOR:
-      result = "VCP_SUBSET_COLOR";
+      result = "DDCA_SUBSET_COLOR";
       break;
    case DDCA_SUBSET_PROFILE:
-      result = "VCP_SUBSET_PROFILE";
+      result = "DDCA_SUBSET_PROFILE";
       break;
    case DDCA_SUBSET_MFG:
-      result = "VCP_SUBSET_MFG";
+      result = "DDCA_SUBSET_MFG";
       break;
    case DDCA_SUBSET_UNSET:
-      result = "VCP_SUBSET_NONE";
+      result = "DDCA_SUBSET_NONE";
       break;
    case DDCA_SUBSET_CAPABILITIES:
-      result = "VCP_SUBSET_CAPABILITIES";      // ???
+      result = "DDCA_SUBSET_CAPABILITIES";      // ???
       break;
    case DDCA_SUBSET_SCAN:
-      result = "VCP_SUBSET_SCAN";
+      result = "DDCA_SUBSET_SCAN";
+      break;
+   case DDCA_SUBSET_CUSTOM:
+      result = "DDCA_SUBSET_CUSTOM";      // or VCP_SUBSET_NONE?
       break;
    }
    return result;
@@ -100,7 +106,7 @@ ddca_get_feature_list(
 {
    bool debug = false;
    DBGMSF(debug, "Starting. feature_subset_id=%d, vcp_version=%d.%d, include_table_features=%s, p_feature_list=%p",
-          feature_subset_id, vspec.major, vspec.minor, bool_repr(include_table_features), p_feature_list);
+          feature_subset_id, vspec.major, vspec.minor, sbool(include_table_features), p_feature_list);
 
    DDCA_Status ddcrc = 0;
    // Whether a feature is a table feature can vary by version, so can't
@@ -166,15 +172,19 @@ ddca_get_feature_list_by_dref(
       DDCA_Feature_Subset_Id  feature_set_id,
       DDCA_Display_Ref        ddca_dref,
       bool                    include_table_features,
-      DDCA_Feature_List*      p_feature_list)
+      DDCA_Feature_List*      feature_list_loc)
 {
    WITH_DR(
          ddca_dref,
          {
                bool debug = false;
-               DBGMSF(debug, "Starting. feature_subset_id=%d, dref=%p=%s, include_table_features=%s, p_feature_list=%p",
-                      feature_set_id, dref, dref_repr_t(dref), bool_repr(include_table_features), p_feature_list);
-               assert(p_feature_list);
+               DBGMSF(debug, "Starting. feature_subset_id=%d=0x%08x=%s, dref=%p=%s, "
+                             "include_table_features=%s, feature_list_loc=%p",
+                      feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
+                      dref, dref_repr_t(dref),
+                      sbool(include_table_features),
+                      feature_list_loc);
+               assert(feature_list_loc);
 
                DDCA_MCCS_Version_Spec vspec = dref->vcp_version;
                DBGMSF(debug, "vspec=%p=%s=%d.%d", &dref->vcp_version, format_vspec(dref->vcp_version), dref->vcp_version.major, dref->vcp_version.minor);
@@ -204,9 +214,17 @@ ddca_get_feature_list_by_dref(
                   subset = VCP_SUBSET_NONE;
                   // Currently handled in ddcui
                   DBGMSG("DDCA_SUBSET_CAPABILITIES -> VCP_SUBSET_NONE");
+                  break;
                case DDCA_SUBSET_SCAN:
                   subset = VCP_SUBSET_SCAN;
+                  break;
+               case DDCA_SUBSET_CUSTOM:
+                  subset = VCP_SUBSET_NONE;
+                  // handled in ddcui
+                  DBGMSG("DDCA_SUBSET_CUSTOM -> VCP_SUBSET_NONE");
+                  break;
                }
+               DBGMSF(debug, "subset=%d=%s", subset, feature_subset_name( subset));
                Feature_Set_Flags flags = 0x00;
                if (!include_table_features)
                   flags |= FSF_NOTABLE;
@@ -215,13 +233,15 @@ ddca_get_feature_list_by_dref(
 
                // TODO: function variant that takes result location as a parm, avoid memcpy
                DDCA_Feature_List result = feature_list_from_dyn_feature_set(fset);
-               memcpy(p_feature_list, &result, 32);
+               memcpy(feature_list_loc, &result, 32);
                dyn_free_feature_set(fset);
 
             // bye:
-               DBGMSF(debug, "Done. Returning: %s", psc_desc(psc));
+               DBGMSF(debug, "Done. feature_set_id=%d=0x%08x=%s, subset=%d=%s, Returning: %s",
+                     feature_set_id, feature_set_id, ddca_feature_list_id_name(feature_set_id),
+                     subset, feature_subset_name(subset), psc_desc(psc));
                if (debug) {
-                  DBGMSG("Feature list: %s", feature_list_string(p_feature_list, "", ","));
+                  DBGMSG(    "      Feature list: %s", feature_list_string(feature_list_loc, "", ","));
                   // rpt_hex_dump((Byte*) p_feature_list, 32, 1);
                }
          }
@@ -229,30 +249,39 @@ ddca_get_feature_list_by_dref(
 }
 
 
+bool
+ddca_feature_list_eq(
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2)
+{
+   return memcmp(&vcplist1, &vcplist2, sizeof(DDCA_Feature_List)) == 0;
+}
+
+
 DDCA_Feature_List
 ddca_feature_list_or(
-      DDCA_Feature_List* vcplist1,
-      DDCA_Feature_List* vcplist2)
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2)
 {
-   return feature_list_or(vcplist1, vcplist2);
+   return feature_list_or(&vcplist1, &vcplist2);
 }
 
 
 DDCA_Feature_List
 ddca_feature_list_and(
-      DDCA_Feature_List* vcplist1,
-      DDCA_Feature_List* vcplist2)
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2)
 {
-   return feature_list_and(vcplist1, vcplist2);
+   return feature_list_and(&vcplist1, &vcplist2);
 }
 
 
 DDCA_Feature_List
 ddca_feature_list_and_not(
-      DDCA_Feature_List* vcplist1,
-      DDCA_Feature_List* vcplist2)
+      DDCA_Feature_List vcplist1,
+      DDCA_Feature_List vcplist2)
 {
-   return feature_list_and_not(vcplist1, vcplist2);
+   return feature_list_and_not(&vcplist1, &vcplist2);
 }
 
 
@@ -293,19 +322,19 @@ void ddca_feature_list_to_codes(
 
 int
 ddca_feature_list_count(
-      DDCA_Feature_List * feature_list)
+      DDCA_Feature_List feature_list)
 {
-   return feature_list_count(feature_list);
+   return feature_list_count(&feature_list);
 }
 
 
 const char *
 ddca_feature_list_string(
-      DDCA_Feature_List * feature_list,
-      char * value_prefix,
-      char * sepstr)
+      DDCA_Feature_List feature_list,
+      const char * value_prefix,
+      const char * sepstr)
 {
-   return feature_list_string(feature_list, value_prefix, sepstr);
+   return feature_list_string(&feature_list, value_prefix, sepstr);
 }
 
 
