@@ -7,7 +7,7 @@
  *  within the code that implements the API.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDCUTIL_TYPES_H_
@@ -133,18 +133,19 @@ typedef enum {
 //!
 //! Used as bitflags to specify multiple trace types
 typedef enum {
- DDCA_TRC_BASE = 0x0080,        /**< base functions          */
- DDCA_TRC_I2C  = 0x0040,        /**< I2C layer               */
- DDCA_TRC_ADL  = 0x0020,        /**< ADL layer               */
- DDCA_TRC_DDC  = 0x0010,        /**< DDC layer               */
- DDCA_TRC_USB  = 0x0008,        /**< USB connected display functions */
- DDCA_TRC_TOP  = 0x0004,        /**< ddcutil mainline        */
- DDCA_TRC_ENV  = 0x0002,        /**< environment command     */
- DDCA_TRC_API  = 0x0001,        /**< top level API functions */
- DDCA_TRC_UDF  = 0x0100,        /**< user-defined, aka dynamic, features */
- DDCA_TRC_VCP  = 0x0200,        /**< VCP layer, feature definitions */
+ DDCA_TRC_BASE  = 0x0080,       /**< base functions          */
+ DDCA_TRC_I2C   = 0x0040,       /**< I2C layer               */
+ DDCA_TRC_ADL   = 0x0020,       /**< ADL layer               */
+ DDCA_TRC_DDC   = 0x0010,       /**< DDC layer               */
+ DDCA_TRC_USB   = 0x0008,       /**< USB connected display functions */
+ DDCA_TRC_TOP   = 0x0004,       /**< ddcutil mainline        */
+ DDCA_TRC_ENV   = 0x0002,       /**< environment command     */
+ DDCA_TRC_API   = 0x0001,       /**< top level API functions */
+ DDCA_TRC_UDF   = 0x0100,       /**< user-defined, aka dynamic, features */
+ DDCA_TRC_VCP   = 0x0200,       /**< VCP layer, feature definitions */
  DDCA_TRC_DDCIO = 0x0400,       /**< DDC IO functions */
  DDCA_TRC_SLEEP = 0x0800,       /**< low level sleeps */
+ DDCA_TRC_RETRY = 0x1000,       /**< successful retries, subset of DDCA_TRC_SLEEP */
 
  DDCA_TRC_NONE = 0x0000,        /**< all tracing disabled    */
  DDCA_TRC_ALL  = 0xffff         /**< all tracing enabled     */
@@ -152,7 +153,7 @@ typedef enum {
 
 
 typedef enum {
-   DDCA_TRCOPT_TIMESTAMP = 0x00,
+   DDCA_TRCOPT_TIMESTAMP = 0x01,
    DDCA_TRCOPT_THREAD_ID = 0x02,
 } DDCA_Trace_Options;
 
@@ -309,7 +310,8 @@ typedef enum {
    DDCA_SUBSET_PROFILE,        ///< Features saved and restored by loadvcp/setvcp
    DDCA_SUBSET_MFG,            ///< Feature codes reserved for manufacturer use (0x0e..0xff)
    DDCA_SUBSET_CAPABILITIES,   ///< Feature codes specified in capabilities string
-   DDCA_SUBSET_SCAN            ///< All feature codes other than known write-only or table
+   DDCA_SUBSET_SCAN,            ///< All feature codes other than known write-only or table
+   DDCA_SUBSET_CUSTOM
 } DDCA_Feature_Subset_Id;
 
 
@@ -488,6 +490,8 @@ struct {
    uint8_t *                            cmd_codes;       /**< array of command codes */
    int                                  vcp_code_ct;     /**< number of features in vcp() field */
    DDCA_Cap_Vcp *                       vcp_codes;       /**< array of pointers to structs describing each vcp code */
+   int                                  msg_ct;
+   char **                              messages;
 } DDCA_Capabilities;
 
 
