@@ -1,28 +1,12 @@
 /* demo_vcpinfo.c
  *
  * Query VCP feature information and capabilities string
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
  */
 
+// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+
 #define _GNU_SOURCE        // for asprintf()
 
 #include <assert.h>
@@ -153,7 +137,7 @@ void test_single_feature_info(
    //        feature_code, vspec.major, vspec.minor);
    // printf("Feature name: %s\n", ddca_feature_name_by_vspec(feature_code, vspec));
 
-   DDCA_Feature_Metadata metadata;
+   DDCA_Feature_Metadata* metadata;
    DDCA_Status rc = ddca_get_feature_metadata_by_vspec(
                        feature_code,
                        vspec,
@@ -166,13 +150,13 @@ void test_single_feature_info(
       DDC_ERRMSG("ddca_get_feature_info_by_vcp_version", rc);
    }
    else {
-      if (metadata.feature_flags & DDCA_SYNTHETIC) {
+      if (metadata->feature_flags & DDCA_SYNTHETIC) {
          if (feature_code >= 0xe0)
             printf("\nCreated synthetic metadata for manufacturer-specific feature.\n");
          else
             printf("\nCreated synthetic metadata for unrecognized feature code.\n");
       }
-      show_feature_metadata(&metadata);
+      show_feature_metadata(metadata);
    }
    // printf("%s) Done.\n", __func__);
 }
