@@ -4,7 +4,7 @@
  * hiddev ioctl() calls.
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <assert.h>
@@ -41,9 +41,7 @@ static DDCA_Trace_Group TRACE_GROUP = DDCA_TRC_USB;
  *
  * Arguments:
  *   hiddev_devname
- *   readonly         if true, open read only
- *                    if false, open for reading and writing
- *   emit_error_msg   if true, output message if error
+ *   calloptions      checks CALLOPT_RDONLY, CALLOPT_ERR_MSG
  *
  * Returns:
  *   file descriptor ( >= 0) if success
@@ -166,21 +164,20 @@ hiddev_get_device_info(
       struct hiddev_devinfo *  dev_info,
       Byte                     calloptions)
 {
+   bool debug = false;
+   DBGMSF(debug, "Starting.");
    assert(dev_info);
 
    int rc = ioctl(fd, HIDIOCGDEVINFO, dev_info);
    if (rc != 0) {
       int errsv = errno;
-      if (calloptions & CALLOPT_ERR_MSG)
+      if ( (calloptions & CALLOPT_ERR_MSG) || debug)
          REPORT_IOCTL_ERROR("HIDIOCGDEVINFO", errsv);
-#ifdef OLD
-      if (calloptions & CALLOPT_ERR_ABORT)
-         DDC_ABORT(errsv);
-#endif
       rc = -errsv;
   }
 
   assert(rc <= 0);
+  DBGMSF(debug, "Done.     Returning: %s", psc_desc(rc));
   return rc;
 }
 
