@@ -1,14 +1,14 @@
 /** @file usb_displays.c
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <glib.h>
+#include <glib-2.0/glib.h>
 #include <linux/hiddev.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
@@ -382,33 +382,38 @@ GPtrArray * get_usb_monitor_list() {
       char * hiddev_fn = g_ptr_array_index(hiddev_names, devname_ndx);
       DBGTRC(debug, TRACE_GROUP, "Examining device: %s", hiddev_fn);
 
-      if (!is_possible_monitor_by_hiddev_name(hiddev_fn))
-         break;
-
+      if (!is_possible_monitor_by_hiddev_name(hiddev_fn)) {
+         DBGTRC(debug, TRACE_GROUP, "Not a possible monitor: %s", hiddev_fn);
+         continue;
+      }
 
       // will need better message handling for API
       Byte calloptions = CALLOPT_RDONLY;
-      if (ol >= DDCA_OL_VERBOSE)
+      // if (ol >= DDCA_OL_VERBOSE)  // always give the user a clue as to why detection failed
          calloptions |= CALLOPT_ERR_MSG;
       int fd = usb_open_hiddev_device(hiddev_fn, calloptions);
-      if (fd < 0 && ol >= DDCA_OL_VERBOSE) {
-         Usb_Detailed_Device_Summary * devsum =
-               lookup_udev_usb_device_by_devname(hiddev_fn, /* verbose = */ false);
-         if (devsum) {
+      if (fd < 0) {
+         if (ol >= DDCA_OL_VERBOSE) {
             DBGTRC(debug, TRACE_GROUP, "Open failed");
-            // report_usb_detailed_device_summary(devsum, 4);
+            Usb_Detailed_Device_Summary * devsum =
+               lookup_udev_usb_device_by_devname(hiddev_fn, /* verbose = */ false);
+            if (devsum) {
 
-            f0printf(fout(), "  USB bus %s, device %s, vid:pid: %s:%s - %s:%s\n",
+               // report_usb_detailed_device_summary(devsum, 4);
+
+               f0printf(fout(), "  USB bus %s, device %s, vid:pid: %s:%s - %s:%s\n",
                            devsum->busnum_s,
                            devsum->devnum_s,
                            devsum->vendor_id,
                            devsum->product_id,
                            devsum->vendor_name,
                            devsum->product_name);
-            free_usb_detailed_device_summary(devsum);
+               free_usb_detailed_device_summary(devsum);
+            }
          }
       }
-      else if (fd > 1) {     // fd == 0 should never occur
+      else {     // fd == 0 should never occur
+         assert(fd != 0);
          DBGTRC(debug, TRACE_GROUP, "open succeeded");
          // DBGTRC(debug, TRACE_GROUP, "Open succeeded");
          // Declare variables here and initialize them to NULL so that code at label close: works
@@ -437,6 +442,7 @@ GPtrArray * get_usb_monitor_list() {
             goto close;
          }
 
+         DBGTRC(debug, TRACE_GROUP, "Collecting USB reports...");
          vcp_reports = collect_vcp_reports(fd);
 
          moninfo = calloc(1,sizeof(Usb_Monitor_Info));
@@ -471,6 +477,7 @@ GPtrArray * get_usb_monitor_list() {
             free(cgname);
          // TODO, free device summary
          usb_close_device(fd, hiddev_fn, CALLOPT_NONE); // return error if failure
+         DBGTRC(debug, TRACE_GROUP, "Closed");
       }  // monitor opened
    } // loop over device names
 
@@ -479,7 +486,7 @@ GPtrArray * get_usb_monitor_list() {
 
    if ( debug || IS_TRACING() ) {
       DBGMSG("Returning  %d monitors ", usb_monitors->len);
-      // report_usb_monitors(usb_monitors,1);
+      report_usb_monitors(usb_monitors,1);
    }
 
    return usb_monitors;
