@@ -1,7 +1,7 @@
 /** @file hiddev_util.c
  */
 
-// Copyright (C) 2016-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2016-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -79,8 +79,19 @@ static int is_hiddev(const struct dirent *ent) {
  * Returns:   GPtrArray of device device names.
  */
 GPtrArray * get_hiddev_device_names_using_filesys() {
+   bool debug = false;
+   if (debug) printf("(%s) Starting...\n", __func__);
+
    const char *hidraw_paths[] = { "/dev/", "/dev/usb/", NULL };
-   return get_filenames_by_filter(hidraw_paths, is_hiddev);
+   GPtrArray * dev_names = get_filenames_by_filter(hidraw_paths, is_hiddev);
+
+   if (debug) {
+      printf("(%s) Done.     Found %d hiddev devices:\n", __func__, dev_names->len);
+      for (int ndx = 0; ndx < dev_names->len; ndx++) {
+         printf("              %s\n", (char *) g_ptr_array_index(dev_names,ndx));
+      }
+   }
+   return dev_names;
 }
 
 
@@ -141,6 +152,12 @@ get_hiddev_device_names_using_udev() {
     udev_unref(udev);
 
 bye:
+   if (debug) {
+      printf("(%s) Done.     Found %d hiddev devices:\n", __func__, dev_names->len);
+      for (int ndx = 0; ndx < dev_names->len; ndx++) {
+         printf("              %s\n", (char *) g_ptr_array_index(dev_names,ndx));
+      }
+   }
    return dev_names;
 }
 
@@ -153,9 +170,11 @@ bye:
  * Allows for easily switching between alternative implementations.
  */
 GPtrArray * get_hiddev_device_names() {
-
-   return get_hiddev_device_names_using_udev();
-   // return get_hiddev_device_names_using_filesys();
+   // temp, call both for testing
+   GPtrArray * result = get_hiddev_device_names_using_udev();
+   // g_ptr_array_free(result1, true);
+   // GPtrArray * result2 = get_hiddev_device_names_using_filesys();
+   return result;
 }
 
 
@@ -186,7 +205,7 @@ bool force_hiddev_monitor(int fd) {
 bye:
    if (debug)
       printf("(%s) vid=0x%04x, pid=0x%04x, returning: %s\n", __func__,
-             dev_info.vendor, dev_info.product, bool_repr(result));
+             dev_info.vendor, dev_info.product, sbool(result));
    return result;
 }
 
@@ -235,7 +254,7 @@ bool is_hiddev_monitor(int fd) {
    //    result = force_hiddev_monitor(fd);
 
    if (debug)
-      printf("(%s) Returning: %s\n", __func__, bool_repr(result));
+      printf("(%s) Returning: %s\n", __func__, sbool(result));
    return result;
 }
 
@@ -428,7 +447,7 @@ bool hiddev_is_field_edid(int fd, struct hiddev_report_info * rinfo, int field_i
       all_usages_edid = ( hiddev_get_identical_ucode(fd, &finfo, field_index) == 0x00800002 );
 
    if (debug)
-      printf("(%s) Returning: %s", __func__, bool_repr(all_usages_edid));
+      printf("(%s) Returning: %s", __func__, sbool(all_usages_edid));
    return all_usages_edid;
 }
 
@@ -488,7 +507,7 @@ test_field_ucode(
    bool debug = false;
    if (debug)
       printf("(%s) report_type=%d, report_id=%d, field index=%d, ucode=0x%08x, require_all_match=%s\n",
-             __func__, report_type, report_id, field_index, ucode, bool_repr(require_all_match));
+             __func__, report_type, report_id, field_index, ucode, sbool(require_all_match));
 
    struct hiddev_field_info * result = NULL;
    int rc;
@@ -573,7 +592,7 @@ hiddev_find_report(int fd, __u32 report_type, __u32 ucode, bool match_all_ucodes
    bool debug = false;
    if (debug)
       printf("(%s) Starting.  report_type=%d, ucode=0x%08x, match_all_ucodes=%s\n",
-            __func__, report_type, ucode, bool_repr(match_all_ucodes));
+            __func__, report_type, ucode, sbool(match_all_ucodes));
 
    struct hid_field_locator * result = NULL;
 
