@@ -1,7 +1,7 @@
 /** @file hidraw_util.c
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include <config.h>
@@ -336,7 +336,7 @@ bye:
    if (fd >= 0)    // really shouldn't be closing 0..2 stdin..stderr, but coverity complains
       close(fd);
    if (debug)
-      printf("(%s) devname=%s, returning %s\n", __func__, devname, bool_repr(is_monitor));
+      printf("(%s) devname=%s, returning %s\n", __func__, devname, sbool(is_monitor));
    return is_monitor;
 }
 
