@@ -1,32 +1,15 @@
-/* libusb_reports.c
+/** \file libusb_reports.c
  *
  * Report libusb data structures
  *
  * libusb is not currently used by ddcutil.  This code is retained for reference.
- *
- * <copyright>
- * Copyright (C) 2016-2019 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- *
- * Portions adapted from lsusb.c (command lsusb) by Thomas Sailer and David Brownell
  */
 
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+//
+// Portions adapted from lsusb.c (command lsusb) by Thomas Sailer and David Brownell
+
 // Adapted from usbplay2 file libusb_util.c
 
 #include <assert.h>
@@ -419,7 +402,7 @@ bool call_read_control_msg(
 bye:
    *pbytes_read = bytes_read;
    if (debug)
-      printf("(%s) Returning: %s, *pbytes_read=%d\n", __func__, bool_repr(ok), *pbytes_read);
+      printf("(%s) Returning: %s, *pbytes_read=%d\n", __func__, sbool(ok), *pbytes_read);
    return ok;
 }
 
@@ -920,7 +903,7 @@ void report_open_dev(
 {
    bool debug = false;
    if (debug)
-      printf("(%s) Starting.  dev=%p, dh=%p, show_hubs=%s\n", __func__, dev, dh, bool_repr(show_hubs));
+      printf("(%s) Starting.  dev=%p, dh=%p, show_hubs=%s\n", __func__, dev, dh, sbool(show_hubs));
 
    assert(dev);
    assert(dh);
@@ -945,7 +928,7 @@ void report_libusb_device(
 {
    bool debug = false;
    if (debug)
-      printf("(%s) Starting. dev=%p, show_hubs=%s\n", __func__, dev, bool_repr(show_hubs));
+      printf("(%s) Starting. dev=%p, show_hubs=%s\n", __func__, dev, sbool(show_hubs));
 
    int d1 = depth+1;
    int rc;
@@ -1222,7 +1205,7 @@ void report_hid_descriptor(
                     sizeof(dbuf),
                     &bytes_read);
             if (!ok)
-               printf("(%s) get_raw_report_descriptor() returned %s\n", __func__, bool_repr(ok));
+               printf("(%s) get_raw_report_descriptor() returned %s\n", __func__, sbool(ok));
             if (ok) {
                puts("");
                rpt_hex_dump(dbuf, bytes_read, d1);
