@@ -2,7 +2,7 @@
  *  General purpose data structures
  */
 
-// Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
@@ -208,14 +208,17 @@ Byte * bva_bytes(Byte_Value_Array bva) {
  *  @return string representation of data, caller must free
  */
 char * bva_as_string(Byte_Value_Array bva, bool as_hex, char * sep) {
+   assert(bva);
    GByteArray* ga = (GByteArray*) bva;
+   if (!sep)
+      sep = "";
    int len = ga->len;
    Byte * bytes = ga->data;
-   int sepsz = (sep) ? strlen(sep) : 0;
+   int sepsz = strlen(sep);
    int alloc_sz = len * (3+sepsz) + 1;  // slightly large, but simpler to compute
    char * buf = calloc(1, alloc_sz);
    for (int ndx = 0; ndx < len; ndx++) {
-      char * cursep = (ndx > 0 && sep) ? sep : "";
+      char * cursep = (ndx > 0) ? sep : "";
       if (as_hex)
          snprintf(buf + strlen(buf), alloc_sz-strlen(buf), "%s%02x", cursep, bytes[ndx]);
       else
@@ -370,7 +373,7 @@ bool bbf_is_set(Byte_Bit_Flags bbflags, Byte val) {
    bool result = flags->byte[flagndx] & flagbit;
    // printf("(%s) bbflags=0x%s, val=0x%02x, returning: %d\n",
    //        __func__, hexstring( (unsigned char *)flags->byte,32), val, result);
-   // printf("(%s) val = 0x%02x, returning %s\n",  __func__, val, bool_repr(result));
+   // printf("(%s) val = 0x%02x, returning %s\n",  __func__, val, sbool(result));
    return result;
 }
 
@@ -720,7 +723,7 @@ bool store_bytehex_list(char * start, int len, void * data_struct, Byte_Appender
          hexok = hhc_to_byte_in_buf(buf, &byteVal);
       }
       if (!hexok) {
-         printf("(%s) Invalid hex value in list: %s\n", __func__, nexttok);
+         // printf("(%s) Invalid hex value in list: %s\n", __func__, nexttok);
          ok = false;
       }
       // printf("(%s) byteVal=0x%02x  \n", __func__, byteVal );
@@ -729,7 +732,7 @@ bool store_bytehex_list(char * start, int len, void * data_struct, Byte_Appender
    }
 
    free(buf);
-   // printf("(%s) Returning %s\n", __func__, bool_repr(ok));
+   // printf("(%s) Returning %s\n", __func__, sbool(ok));
    return ok;
 }
 
@@ -1184,7 +1187,7 @@ char * vnt_interpret_flags(
    bool debug = false;
    if (debug)
       printf("(%s) Starting. flags_val=0x%08x, bitname_table=%p, use_title=%s, sepstr=|%s|\n",
-             __func__, flags_val, (void*)bitname_table, bool_repr(use_title), sepstr);
+             __func__, flags_val, (void*)bitname_table, sbool(use_title), sepstr);
 
    GString * sbuf = g_string_sized_new(200);
    bool first = true;
