@@ -2,7 +2,7 @@
  * Lookup PCI and USB device ids
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -62,7 +62,6 @@ char * devid_find_file(Device_Id_Type id_type) {
    bool debug = false;
 
    char * known_pci_ids_dirs[] = {
-         "/usr/share/libosinfo/db",
          "/usr/share",
          "/usr/share/misc",
          "/usr/share/hwdata",
@@ -992,7 +991,6 @@ char * devid_hid_descriptor_country_code(ushort id) {
 }
 
 
-
 //
 // *** Initialization ***
 //
@@ -1021,7 +1019,7 @@ bool devid_ensure_initialized() {
       }
    }
    if (debug)
-      printf("(%s) Returning: %s\n", __func__, bool_repr(ok));
+      printf("(%s) Returning: %s\n", __func__, sbool(ok));
    return ok;
 }
 
