@@ -1,27 +1,5 @@
-/* ddc_error.c
+/** \f error_info.c
  *
- * <copyright>
- * Copyright (C) 2017-2918 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
- */
-
-/** \f
  *  Struct for reporting errors.
  *
  *  #Error_Info provides a pseudo-exception framework that can be integrated
@@ -31,10 +9,13 @@
  *  error is retained for use by higher levels in the call stack.
  */
 
+// Copyright (C) 2017-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+
 /** \cond */
 
 // #define _GNU_SOURCE     // for reallocarray() in stdlib.h
-
 #include <assert.h>
 #include <glib-2.0/glib.h>
 #include <stdlib.h>
@@ -543,6 +524,72 @@ default_status_code_desc(int rc) {
 }
 
 
+
+GString *
+errinfo_array_summary_gs(
+      struct error_info **  errors,    ///<  pointer to array of pointers to Error_Info
+      int                   error_ct,
+      GString *             gs)     ///<  number of causal errors
+{
+      bool first = true;
+
+      int ndx = 0;
+      while (ndx < error_ct) {
+         // printf("(%s) this error = %p\n", __func__, errors[ndx]);
+         int this_psc = errors[ndx]->status_code;
+         int cur_ct = 1;
+
+         for (int i = ndx+1; i < error_ct; i++) {
+            if (errors[i]->status_code != this_psc)
+               break;
+            cur_ct++;
+         }
+
+         if (first)
+            first = false;
+         else
+            g_string_append(gs, ", ");
+         if (errinfo_name_func)
+            g_string_append(gs, errinfo_name_func(this_psc));
+         else {
+            char buf[20];
+            snprintf(buf, 20, "%d",this_psc);
+            buf[19] = '\0';
+            g_string_append(gs, buf);
+         }
+         if (cur_ct > 1)
+            g_string_append_printf(gs, "(%d)", cur_ct);
+         ndx += cur_ct;
+      }
+
+   return gs;
+}
+
+
+/** Returns a comma separated string of the status code names of the
+ *  causes in an array of #Error_Info.
+ *  Multiple consecutive identical names are replaced with a
+ *  single name and a parenthesized instance count.
+ *
+ *  \param  erec  pointer to array of #Error_Info instances
+ *  \return comma separated string, caller is responsible for freeing
+ */
+char *
+errinfo_array_summary(
+      struct error_info **  errors,    ///<  pointer to array of pointers to Error_Info
+      int                   error_ct)
+{
+   GString * gs = g_string_new(NULL);
+   // printf("(%s) errors=%p\n", __func__, errors);
+   errinfo_array_summary_gs(errors, error_ct, gs);
+   char * result = gs->str;
+   g_string_free(gs, false);
+
+   // DBGMSF(debug, "Done.  Returning: |%s|", result);
+   return result;
+}
+
+
 /** Returns a comma separated string of the status code names in the
  *  causes of the specified #Error_Info.
  *  Multiple consecutive identical names are replaced with a
@@ -554,10 +601,27 @@ default_status_code_desc(int rc) {
 char *
 errinfo_causes_string(Error_Info * erec) {
    // bool debug = false;
-   // DBGMSF(debug, "history=%p, history->ct=%d", history, history->ct);
 
    GString * gs = g_string_new(NULL);
 
+   if (erec) {
+      assert(memcmp(erec->marker, ERROR_INFO_MARKER, 4) == 0);
+
+      errinfo_array_summary_gs(erec->causes, erec->cause_ct, gs);
+   }
+   char * result = gs->str;
+   g_string_free(gs, false);
+
+   // DBGMSF(debug, "Done.  Returning: |%s|", result);
+   return result;
+
+
+
+
+
+#ifdef OLD
+   GString * gs = g_string_new(NULL);
+
    if (erec) {
       assert(memcmp(erec->marker, ERROR_INFO_MARKER, 4) == 0);
 
@@ -597,6 +661,7 @@ errinfo_causes_string(Error_Info * erec) {
 
    // DBGMSF(debug, "Done.  Returning: |%s|", result);
    return result;
+#endif
 }
 
 
