@@ -1,26 +1,11 @@
-/* file_util.c
- *
- * <copyright>
- * Copyright (C) 2014-2018 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** \file file_util.c
+ *  File related utility functions
  */
 
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+
 /** \cond */
 #include <assert.h>
 #include <errno.h>
@@ -67,10 +52,9 @@ int file_getlines(const char * fn,  GPtrArray* line_array, bool verbose) {
       // if line == NULL && len == 0, then getline allocates buffer for line
       char * line = NULL;
       size_t len = 0;
-      ssize_t read;
       int     linectr = 0;
       errno = 0;
-      while ((read = getline(&line, &len, fp)) != -1) {
+      while (getline(&line, &len, fp) >= 0) {
          linectr++;
          rtrim_in_place(line);     // strip trailing newline
          g_ptr_array_add(line_array, line);
@@ -210,10 +194,9 @@ file_get_last_lines(
       // if line == NULL && len == 0, then getline allocates buffer for line
       char * line = NULL;
       size_t len = 0;
-      ssize_t read;
       int     linectr = 0;
       errno = 0;
-      while ((read = getline(&line, &len, fp)) != -1) {
+      while (getline(&line, &len, fp) >= 0) {
          linectr++;
          rtrim_in_place(line);     // strip trailing newline
          clb_add(clb, line);
@@ -435,7 +418,7 @@ GPtrArray * get_filenames_by_filter(const char * dirnames[], Dirent_Filter filte
  *               The caller is responsible for freeing this memory
  *
  * @retval 0      success
- * @retval -errno if error (see readlink() doc for possible error numbers
+ * @retval -errno if error (see readlink() doc for possible error numbers)
  */
 int filename_for_fd(int fd, char** p_fn) {
    char * result = calloc(1, PATH_MAX+1);
@@ -458,3 +441,21 @@ int filename_for_fd(int fd, char** p_fn) {
    return rc;
 }
 
+
+char * filename_for_fd_t(int fd) {
+   static GPrivate  key = G_PRIVATE_INIT(g_free);
+   char * fn_buf = get_thread_fixed_buffer(&key, PATH_MAX+1);
+
+   char * result = NULL;  // value to resturn
+
+   char * filename_loc;
+   int rc = filename_for_fd(fd, &filename_loc);
+   if (rc == 0) {
+      g_strlcpy(fn_buf, filename_loc, PATH_MAX+1);
+      free(filename_loc);
+      result = fn_buf;
+   }
+   return result;
+}
+
+
