@@ -1,31 +1,11 @@
-/* glib_util.h
+/** @file glib_util.h
  *
- * Utility functions for glib.
- *
- * <copyright>
- * Copyright (C) 2014-2016 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+ *  Utility functions for glib.
  */
 
-/** @file glib_util.h
- *  Functions for use with glib
- */
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 
 #ifndef GLIB_UTIL_H_
 #define GLIB_UTIL_H_
@@ -44,13 +24,13 @@ gchar * gaux_vasprintf(size_t reqd_buf_size, gchar * fmt, va_list ap);
 gchar * gaux_asprintf(gchar * fmt, ...);
 #endif
 
-gchar *
+void *
 get_thread_dynamic_buffer(
       GPrivate * buf_key_ptr,
       GPrivate * bufsz_key_ptr,
       guint16    required_size);
 
-gchar *
+void *
 get_thread_fixed_buffer(
       GPrivate * buf_key_ptr,
       guint16    required_size);
