@@ -1,34 +1,13 @@
-/* libdrm_util.c
- *
- * Utilities for interpreting libdrm data structures.
- *
- * <copyright>
- * Copyright (C) 2017 Sanford Rockowitz <rockowitz@minsoft.com>
- *
- * Licensed under the GNU General Public License Version 2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- * </endcopyright>
+/** @file libdrm_util.c
+ * Utilities for interpreting libdrm data structures
  */
 
-/** @file libdrm_util.c
- * Utilities for use with libdrm
- */
+// Copyright (C) 2017-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 /** \cond */
 #include <glib-2.0/glib.h>
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -474,8 +453,9 @@ void report_drmModeConnector( int fd, drmModeConnector * p, int depth) {
 
    rpt_vstring(d1, "%-20s %d",  "count_props", p->count_props);
    for (int ndx = 0; ndx < p->count_props; ndx++) {
-       rpt_vstring(d2, "index=%d, property id (props)=%u, property value (prop_values)=%u  0x%08x",
-                        ndx, p->props[ndx], p->prop_values[ndx], p->prop_values[ndx]);
+       uint64_t curval = p->prop_values[ndx];   // coverity workaround
+       rpt_vstring(d2, "index=%d, property id (props)=%" PRIu32 ", property value (prop_values)=%" PRIu64 ,
+                        ndx, p->props[ndx], curval);
 
        drmModePropertyPtr prop_ptr = drmModeGetProperty(fd, p->props[ndx]);
        if (prop_ptr) {
@@ -483,7 +463,7 @@ void report_drmModeConnector( int fd, drmModeConnector * p, int depth) {
           drmModeFreeProperty(prop_ptr);
        }
        else {
-          rpt_vstring(d2, "Unrecognized property id: %d, value=%u", p->props[ndx], p->prop_values[ndx]);
+          rpt_vstring(d2, "Unrecognized property id: %d, value=%"PRIu64, p->props[ndx], p->prop_values[ndx]);
        }
    }
 
@@ -544,14 +524,14 @@ void report_property_value(
    int d1 = depth+1;
    rpt_vstring(depth, "Property id:   %d", prop_ptr->prop_id);
    rpt_vstring(d1, "Name:          %s", prop_ptr->name);
-   rpt_vstring(d1, "Flags:         0x%04x - %s", prop_ptr->flags,
+   rpt_vstring(d1, "Flags:         0x%04x - %s", (unsigned int) prop_ptr->flags,
                                                  interpret_property_flags(prop_ptr->flags) );
-   rpt_vstring(d1, "prop_value:    %d  0x%08x", prop_value, prop_value);
+   rpt_vstring(d1, "prop_value:    %"PRIu64"  0x%08x", prop_value, (unsigned int) prop_value);
 
    if (prop_ptr->flags & DRM_MODE_PROP_ENUM) {
       for (int i = 0; i < prop_ptr->count_enums; i++) {
          if (prop_ptr->enums[i].value == prop_value) {
-            rpt_vstring(d1, "Property value(enum) = %d - %s", prop_value, prop_ptr->enums[i].name);
+            rpt_vstring(d1, "Property value(enum) = %"PRIu64" - %s", prop_value, prop_ptr->enums[i].name);
             break;
          }
       }
@@ -565,15 +545,15 @@ void report_property_value(
               not_truncated = sbuf_append(buf, bufsz, ", ", prop_ptr->enums[i].name);
            }
         }
-      rpt_vstring(d1, "Property value(bitmask) = 0x%04x - %s", prop_value, buf);
+      rpt_vstring(d1, "Property value(bitmask) = 0x%04x - %s", (unsigned int) prop_value, buf);
    }
 
    else if (prop_ptr->flags & DRM_MODE_PROP_RANGE) {
       if (prop_ptr->count_values != 2) {
-         rpt_vstring(d1, "Property value = %d, Missing min or max value", prop_value);
+         rpt_vstring(d1, "Property value = %"PRIu64", Missing min or max value", prop_value);
       }
       else {
-         rpt_vstring(d1, "Property value(range) = %d, min=%d, max=%d",
+         rpt_vstring(d1, "Property value(range) = %" PRIu64 ", min=%" PRIu64 ", max=%" PRIu64,
                          prop_value, prop_ptr->values[0], prop_ptr->values[1]);
       }
    }
@@ -591,23 +571,23 @@ void report_property_value(
    }
 
    else if (drm_property_type_is(prop_ptr, DRM_MODE_PROP_OBJECT)) {
-      rpt_vstring(d1, "Object type, name = %s, value=%d",
+      rpt_vstring(d1, "Object type, name = %s, value=%"PRIu64,
                       prop_ptr->name, prop_value);
    }
 
    else if (drm_property_type_is(prop_ptr, DRM_MODE_PROP_SIGNED_RANGE)) {
          if (prop_ptr->count_values != 2) {
-            rpt_vstring(d1, "Signed property value = %d, Missing min or max value", prop_value);
+            rpt_vstring(d1, "Signed property value = %"PRIu64", Missing min or max value", prop_value);
          }
         else {
-           rpt_vstring(d1, "Property value(range) = %d, min=%d, max=%d",
-                           (int64_t) prop_value,
-                           (int64_t) prop_ptr->values[0], (int64_t) prop_ptr->values[1]);
+           rpt_vstring(d1, "Property value(range) = %" PRIu64 ", min=%" PRIu64 ", max=%" PRIu64,
+                           prop_value,
+                           prop_ptr->values[0], prop_ptr->values[1]);
         }
    }
 
    else {
-      rpt_vstring(d1, "Unrecognized type flags=0x%08x, value = %d", prop_ptr->flags, prop_value);
+      rpt_vstring(d1, "Unrecognized type flags=0x%08x, value = %" PRIu64, prop_ptr->flags, prop_value);
    }
 }
 
@@ -652,7 +632,9 @@ void report_drm_modeProperty(drmModePropertyRes * p, int depth) {
    }
    rpt_vstring(d1, "%-20s %d",          "count_enums:", p->count_enums);
    for (int ndx = 0; ndx < p->count_enums; ndx++) {
-      rpt_vstring(d2, "enums[%d] = %u: %s", ndx, p->enums[ndx].value,  p->enums[ndx].name);
+      uint64_t curval = p->enums[ndx].value;   // coverity workaround
+      rpt_vstring(d2, "enums[%d] = %" PRIu64 ": %s",
+                      ndx, curval,  (char *) p->enums[ndx].name);
    }
    rpt_vstring(d1, "%-20s %d",          "count_blobs:", p->count_blobs);
    for (int ndx = 0; ndx < p->count_blobs; ndx++) {
