@@ -2,7 +2,7 @@
  *  String utility functions
  */
 
-// Copyright (C) 2014-2019 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2020 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 
@@ -35,15 +35,16 @@
 // General
 //
 
+#ifdef NOT_INLINE
 /** Returns a character string representation of an integer as a boolean value.
  *
  * @param value value to represent
  * @return "true" or "false"
  */
-char * bool_repr(int value) {
-   char * answer = (value) ? "true" : "false";
-   return answer;
+char * sbool(int value) {
+   return (value) ? "true" : "false";
 }
+#endif
 
 
 //
@@ -110,7 +111,7 @@ bool str_starts_with(const char * value_to_test, const char * prefix) {
  *
  * @param  value_to_test  value to examine
  * @param  suffix         substring to check for
- * @return true/flase
+ * @return true/false
  *
  * @remark Consider using lib function g_str_suffix() ?? instead
  */
@@ -130,6 +131,21 @@ bool str_ends_with(const char * value_to_test, const char * suffix) {
    return result;
 }
 
+int str_contains(const char * value_to_test, const char * segment) {
+   int result = -1;
+   if (value_to_test && segment) {
+      int seglen = strlen(segment);
+      int laststart = strlen(value_to_test) - seglen;
+      for (int ndx = 0; ndx < laststart; ndx++) {
+         if (str_starts_with(value_to_test+ndx, segment)) {
+            result = ndx;
+            break;
+         }
+      }
+   }
+   return result;
+}
+
 
 /** Are all characters in the string printable?
  *
@@ -844,9 +860,9 @@ bool str_to_int(const char * sval, int * p_ival, int base)
 
    if (debug) {
       if (ok)
-        printf("(%s) sval=%s, Returning: %s, *ival = %d\n", __func__, sval, bool_repr(ok), *p_ival);
+        printf("(%s) sval=%s, Returning: %s, *ival = %d\n", __func__, sval, sbool(ok), *p_ival);
       else
-        printf("(%s) sval=%s, Returning: %s\n", __func__, sval, bool_repr(ok));
+        printf("(%s) sval=%s, Returning: %s\n", __func__, sval, sbool(ok));
    }
    return ok;
 }
@@ -883,9 +899,9 @@ bool str_to_float(const char * sval, float * p_fval)
 
    if (debug) {
       if (ok)
-        printf("(%s) sval=%s, Returning: %s, *p_fval = %16.7f\n", __func__, sval, bool_repr(ok), *p_fval);
+        printf("(%s) sval=%s, Returning: %s, *p_fval = %16.7f\n", __func__, sval, sbool(ok), *p_fval);
       else
-        printf("(%s) sval=%s, Returning: %s\n", __func__, sval, bool_repr(ok));
+        printf("(%s) sval=%s, Returning: %s\n", __func__, sval, sbool(ok));
    }
    return ok;
 }
@@ -1080,7 +1096,7 @@ char * hexstring2(
 {
    // if (len > 1)
    // printf("(%s) bytes=%p, len=%d, sepstr=|%s|, uppercase=%s, buffer=%p, bufsz=%d\n", __func__,
-   //       bytes, len, sepstr, bool_repr(uppercase), buffer, bufsz);
+   //       bytes, len, sepstr, sbool(uppercase), buffer, bufsz);
    int sepsize = 0;
    if (sepstr) {
       sepsize = strlen(sepstr);
@@ -1153,7 +1169,7 @@ char * hexstring3_t(
    static GPrivate  hexstring3_len_key = G_PRIVATE_INIT(g_free);
 
    // printf("(%s) bytes=%p, len=%d, sepstr=|%s|, uppercase=%s\n", __func__,
-   //       bytes, len, sepstr, bool_repr(uppercase));
+   //       bytes, len, sepstr, sbool(uppercase));
    if (hunk_size == 0)
       sepstr = NULL;
    else if (sepstr == NULL)
@@ -1340,7 +1356,7 @@ int f0puts(const char * msg, FILE * stream)
  * in which case no output is written.
  *
  * @param stream     if null do nothing
- * @param  format     format string
+ * @param format     format string
  *
  * @return  result of underlying vfprintf(), or 0 if stream is NULL
  */
